@@ -97,47 +97,60 @@ weed_plant_t *weed_plugin_info_init (weed_bootstrap_f weed_boot, int num_version
   weed_plant_t *host_info=weed_boot((weed_default_getter_f *)&weed_default_get,num_versions,api_versions),*plugin_info;
   if (host_info==NULL) return NULL; // matching version was not found
 
+
+  //////////// get api version /////////
+  weed_default_get(host_info,"api_version",0,&api_version);
+
+
+  // depending on the api version we could have different functions
+
+  // we must use the default getter to get our API functions
+
+  weed_default_get(host_info,"weed_malloc_func",0,(void *)&weedmalloc);
+  weed_malloc=weedmalloc[0];
+
+  weed_default_get(host_info,"weed_free_func",0,(void *)&weedfree);
+  weed_free=weedfree[0];
+
+  weed_default_get(host_info,"weed_memset_func",0,(void *)&weedmemset);
+  weed_memset=weedmemset[0];
+
+  weed_default_get(host_info,"weed_memcpy_func",0,(void *)&weedmemcpy);
+  weed_memcpy=weedmemcpy[0];
+
   weed_default_get(host_info,"weed_leaf_get_func",0,(void *)&wlg);
   weed_leaf_get=wlg[0];
 
-  //////////// get api version /////////
-  weed_leaf_get(host_info,"api_version",0,&api_version);
-
-  ///////////////////////
-  // get our api functions
-
-  weed_leaf_get(host_info,"weed_leaf_set_func",0,(void *)&wls);
+  weed_default_get(host_info,"weed_leaf_set_func",0,(void *)&wls);
   weed_leaf_set=wls[0];
 
-  weed_leaf_get(host_info,"weed_plant_new_func",0,(void *)&wpn);
+  weed_default_get(host_info,"weed_plant_new_func",0,(void *)&wpn);
   weed_plant_new=wpn[0];
 
-  weed_leaf_get(host_info,"weed_plant_list_leaves_func",0,(void *)&wpll);
+  weed_default_get(host_info,"weed_plant_list_leaves_func",0,(void *)&wpll);
   weed_plant_list_leaves=wpll[0];
 
-  weed_leaf_get(host_info,"weed_leaf_num_elements_func",0,&wlne);
+  weed_default_get(host_info,"weed_leaf_num_elements_func",0,(void *)&wlne);
   weed_leaf_num_elements=wlne[0];
 
-  weed_leaf_get(host_info,"weed_leaf_element_size_func",0,&wles);
+  weed_default_get(host_info,"weed_leaf_element_size_func",0,(void *)&wles);
   weed_leaf_element_size=wles[0];
 
-  weed_leaf_get(host_info,"weed_leaf_seed_type_func",0,&wlst);
+  weed_default_get(host_info,"weed_leaf_seed_type_func",0,(void *)&wlst);
   weed_leaf_seed_type=wlst[0];
 
-  weed_leaf_get(host_info,"weed_leaf_get_flags_func",0,&wlgf);
+  weed_default_get(host_info,"weed_leaf_get_flags_func",0,(void *)&wlgf);
   weed_leaf_get_flags=wlgf[0];
 
-  weed_leaf_get(host_info,"weed_malloc_func",0,&weedmalloc);
-  weed_malloc=weedmalloc[0];
 
-  weed_leaf_get(host_info,"weed_free_func",0,&weedfree);
-  weed_free=weedfree[0];
+  // get any additional functions for higher API versions ////////////
 
-  weed_leaf_get(host_info,"weed_memset_func",0,&weedmemset);
-  weed_memset=weedmemset[0];
 
-  weed_leaf_get(host_info,"weed_memcpy_func",0,&weedmemcpy);
-  weed_memcpy=weedmemcpy[0];
+
+  //////////////////////////////////////////////////////////////////////
+
+  // we can now use the normal API functions
+
 
   plugin_info=weed_plant_new(WEED_PLANT_PLUGIN_INFO);
 
