@@ -306,7 +306,7 @@ def run(command):
     """
 
     if verbose:
-        print 'Running: \n' + command + '\n=== ... ==='
+        print('Running: \n' + command + '\n=== ... ===')
         std = ''
     else:
         std = ' > /dev/null 2>&1'
@@ -321,7 +321,7 @@ def do_enhance():
     """
 
     if not quiet:
-        print 'Enhancing images... please wait, this might take long...'
+        print('Enhancing images... please wait, this might take long...')
 
     enh_opts = "-enhance -sharpen '0.0x0.5' -gamma 1.2 -contrast -depth 8"
 
@@ -425,7 +425,7 @@ def do_encode():
     all_vars.update(locals())
 
     if not quiet:
-        print 'Creating "%s"-quality Dirac file' % vtype
+        print('Creating "%s"-quality Dirac file' % vtype)
 
     yuvcommand = """cd %(source_dir)s ; \\
 %(mencoder)s mf://%(syml)s%(img_pre)s*%(ext)s %(mencoder_opts)s%(yuvv)s %(std)s
@@ -440,7 +440,7 @@ def do_encode():
     for tfile in [dyuvv, dyuvv + '.yuv', dyuvv + '.imt']:
         if os.path.exists(tfile):
             if verbose:
-                print 'Removing ' + tfile
+                print('Removing ' + tfile)
             os.remove(tfile)
 
 # Audio not supported yet. Planned to use Matroska container but
@@ -448,7 +448,7 @@ def do_encode():
     audio = False
     if audio:
         if not quiet:
-            print 'Creating "%s"-quality ogg vorbis file' % vtype
+            print('Creating "%s"-quality ogg vorbis file' % vtype)
 
         ogg = tempfile.mkstemp('.ogg', '', work_dir)[1]
         if rawsndf:
@@ -472,7 +472,7 @@ def do_encode():
         all_vars.update(locals())
 
         if not quiet:
-            print 'Multiplexing...'
+            print('Multiplexing...')
         command = """
 %(mkvplex)s --aspect-ratio '1:%(aspect)s' -o "%(vidname)s" %(vidv)s -y '0:0%(delaysh)s' %(ogg)s %(std)s""" % all_vars
         run(command)
@@ -493,7 +493,7 @@ commands e.g. if it's "test.drc" then do:
 dirac_decoder -v test test
 mplayer -demuxer rawvideo -rawvideo fps=%(ffps)s:size=%(fbytes)s:w=%(fx)s:h=%(fy)s test.yuv
 """ % all_vars
-        if not quiet: print note
+        if not quiet: print(note)
 
     if frame_range:
         lframes = os.path.join(source_dir, syml)
@@ -507,7 +507,7 @@ def do_clean():
     """
 
     if not quiet:
-        print 'Deleting all enhanced images (if any)'
+        print('Deleting all enhanced images (if any)')
 
     eframes = os.path.join(work_dir, 'eimg')
     rframes = os.path.join(work_dir, 'rimg')
@@ -552,11 +552,11 @@ def is_installed(prog):
     wprog = which(prog)
 
     if wprog == '':
-        print prog + ': command not found'
-        raise SystemExit
+        print(prog + ': command not found')
+        raise SystemExit, 1
     else:
         if verbose:
-            print wprog + ': found'
+            print(wprog + ': found')
 
 
 if __name__ == '__main__':
@@ -571,30 +571,30 @@ if __name__ == '__main__':
 
     try:
         if sys.version_info[0:3] < (2, 3, 0):
-            raise SystemExit
+            raise SystemExit, 1
     except:
-        print 'You need Python 2.3.0 or greater to run me!'
-        raise SystemExit
+        print('You need Python 2.3.0 or greater to run me!')
+        raise SystemExit, 1
 
     try:
         (opts, args) = getopt.getopt(sys.argv[1:], \
                                          'ho:p:d:w:a:qvt:ekc:r:s:b:f:VCD:L:')
     except:
-        print "Something's wrong. Try the '-h' flag."
-        raise SystemExit
+        print("Something's wrong. Try the '-h' flag.")
+        raise SystemExit, 1
 
     opts = dict(opts)
 
     if not opts and not args:
-        print usage
-        raise SystemExit
+        print(usage)
+        raise SystemExit, 1
 
     if '-h' in opts:
-        print usage + help
+        print(usage + help)
         raise SystemExit
 
     if '-V' in opts:
-        print 'dirac_encoder.py version ' + version
+        print('dirac_encoder.py version ' + version)
         raise SystemExit
 
     if ('-v' in opts) or ('-C' in opts):
@@ -615,8 +615,8 @@ if __name__ == '__main__':
     img_pre = opts.get('-p', '')
 
     if img_pre not in ['', 'eimg', 'rimg']:
-         print 'Improper image name prefix.'
-         raise SystemExit
+         print('Improper image name prefix.')
+         raise SystemExit, 1
 
     temp_dir = ''
     img_dir = opts.get('-d', '.')
@@ -627,14 +627,14 @@ if __name__ == '__main__':
         img_dir = temp_dir + '/img_dir'
 
     if not os.path.isdir(img_dir):
-        print 'The image source directory: '  + img_dir + \
-              ' does not exist!'
-        raise SystemExit
+        print('The image source directory: '  + img_dir + \
+              ' does not exist!')
+        raise SystemExit, 1
 
     if len(args) not in [0, 2]:
-        print 'If frames are specified both first and last ' + \
-              'image numbers must be chosen.'
-        raise SystemExit
+        print('If frames are specified both first and last ' + \
+              'image numbers must be chosen.')
+        raise SystemExit, 1
     elif len(args) == 0:
         args = [None, None]
 
@@ -656,9 +656,9 @@ if __name__ == '__main__':
 
     aspectc = opts.get('-a', '2')
 
-    if aspectc not in [str(i) for i in xrange(1,5)]:
-        print 'Invalid aspect ratio.'
-        raise SystemExit
+    if aspectc not in [str(i) for i in range(1,5)]:
+        print('Invalid aspect ratio.')
+        raise SystemExit, 1
     else:
         if aspectc == '1': aspect = '1:1'
         elif aspectc == '2': aspect = '4:3'
@@ -668,15 +668,15 @@ if __name__ == '__main__':
     vtype = opts.get('-t', 'ml')
 
     if vtype not in ['hi', 'mh', 'ml', 'lo']:
-        print 'Invalid video type.'
-        raise SystemExit
+        print('Invalid video type.')
+        raise SystemExit, 1
 
     out_drc = opts.get('-o', vtype + '_movie.drc')
 
     fpsc = opts.get('-f', '4')
 
-    if fpsc not in [str(i) for i in xrange(1,9)]:
-        if not quiet: print 'Invalid fps code, attempting float fps.'
+    if fpsc not in [str(i) for i in range(1,9)]:
+        if not quiet: print('Invalid fps code, attempting float fps.')
         foundfps = False
     else:
         if fpsc == '1': fps = '24000/1001'
@@ -698,14 +698,14 @@ if __name__ == '__main__':
                 fpsnum = fpsnum/divisor
                 fpsden = fpsden/divisor
             fps = '%s/%s' % (fpsnum, fpsden)
-            if not quiet: print 'Using fps = ' + fps
+            if not quiet: print('Using fps = ' + fps)
             if fpsnum > 0: foundfps = True
         except:
             pass
 
     if not foundfps:
-        print 'Invalid fps code or rate.'
-        raise SystemExit
+        print('Invalid fps code or rate.')
+        raise SystemExit, 1
 
     if '-e' not in opts:
         enhance = False
@@ -713,8 +713,8 @@ if __name__ == '__main__':
         enhance = True
 
     if enhance and img_pre:
-        print 'Sorry, you cannot enhance already-enhanced images'
-        raise SystemExit
+        print('Sorry, you cannot enhance already-enhanced images')
+        raise SystemExit, 1
 
     if '-k' not in opts:
         keep = False
@@ -725,23 +725,23 @@ if __name__ == '__main__':
     rgeom = opts.get('-r', '')
 
     if (cgeom or rgeom) and not enhance:
-        print 'Missing "-e" option.'
-        raise SystemExit
+        print('Missing "-e" option.')
+        raise SystemExit, 1
 
     delay = opts.get('-D', '0')
-    if verbose: print 'Linear audio delay (ms): ' + delay
+    if verbose: print('Linear audio delay (ms): ' + delay)
 
     lv1file = opts.get('-L', None)
     if lv1file:
-        if not quiet: print 'Opening lv1 file...'
+        if not quiet: print('Opening lv1 file...')
         try:
             lv1 = tarfile.open(os.path.abspath(lv1file))
         except:
-            print 'This does not appear to be a valid LiVES file!'
-            raise SystemExit
+            print('This does not appear to be a valid LiVES file!')
+            raise SystemExit, 1
         if 'header.tar' not in lv1.getnames():
-            print 'This does not appear to be a valid LiVES file!'
-            raise SystemExit
+            print('This does not appear to be a valid LiVES file!')
+            raise SystemExit, 1
         for tfile in lv1.getmembers():
             lv1.extract(tfile, img_dir)
         for tfile in glob.glob(os.path.join(img_dir, '*.tar')):
@@ -756,24 +756,24 @@ if __name__ == '__main__':
     elif os.path.isfile(test_file + '.png'):
         ext = '.png'
     else:
-        print 'Cannot find any appropriate %s or %s files!' % ('.jpg','.png')
-        raise SystemExit
+        print('Cannot find any appropriate %s or %s files!' % ('.jpg','.png'))
+        raise SystemExit, 1
     first_frame = test_file + ext
     last_frame = os.path.join(img_dir, img_pre + last_frame_num + ext)
 
-    if not quiet: print 'Found: ' + first_frame
+    if not quiet: print('Found: ' + first_frame)
     
     work_dir = opts.get('-w', img_dir)
     work_dir = os.path.abspath(work_dir)
     if not os.path.isdir(work_dir):
-        if not quiet: print 'Creating ' + work_dir
+        if not quiet: print('Creating ' + work_dir)
         try:
             os.makedirs(work_dir)
-            os.chmod(work_dir, 0755)
+            os.chmod(work_dir, 0o755)
         except:
-            print 'Could not create the work directory ' + \
-                  work_dir
-            raise SystemExit
+            print('Could not create the work directory ' + \
+                  work_dir)
+            raise SystemExit, 1
     if ' ' in work_dir:
         if temp_dir == '':
             temp_dir = tempfile.mkdtemp('', '.lives-', '/tmp/')
@@ -790,7 +790,7 @@ if __name__ == '__main__':
         audio = True
         if sndf[-4:] == '.wav':
             rawsndf = False
-        if not quiet: print 'Found audio file: ' + sndf
+        if not quiet: print('Found audio file: ' + sndf)
 
     sndr = opts.get('-b', '44100')
 
@@ -810,7 +810,7 @@ if __name__ == '__main__':
         do_clean()
     if temp_dir != '':
         shutil.rmtree(temp_dir)
-    if not quiet: print "Done!"
+    if not quiet: print("Done!")
 
 
 """
