@@ -516,11 +516,11 @@ def is_installed(prog):
         if prog == theora_encoder:
             if which(theora_encoder2) == '':
                 print theora_encoder + ' or ' + theora_encoder2 + ': command not found'
-                raise SystemExit
+                raise SystemExit, 1
         else:
             if prog != theora_encoder2:
                 print prog + ': command not found'
-                raise SystemExit
+                raise SystemExit, 1
         
     else:
         if verbose:
@@ -539,23 +539,23 @@ if __name__ == '__main__':
 
     try:
         if sys.version_info[0:3] < (2, 3, 0):
-            raise SystemExit
+            raise SystemExit, 1
     except:
         print 'You need Python 2.3.0 or greater to run me!'
-        raise SystemExit
+        raise SystemExit, 1
 
     try:
         (opts, args) = getopt.getopt(sys.argv[1:], \
                                          'ho:p:d:w:a:qvt:ekc:r:s:b:f:VCD:L:')
     except:
         print "Something's wrong. Try the '-h' flag."
-        raise SystemExit
+        raise SystemExit, 1
 
     opts = dict(opts)
 
     if not opts and not args:
         print usage
-        raise SystemExit
+        raise SystemExit, 1
 
     if '-h' in opts:
         print usage + help
@@ -584,7 +584,7 @@ if __name__ == '__main__':
 
     if img_pre not in ['', 'eimg', 'rimg']:
          print 'Improper image name prefix.'
-         raise SystemExit
+         raise SystemExit, 1
 
     temp_dir = ''
     img_dir = opts.get('-d', '.')
@@ -597,12 +597,12 @@ if __name__ == '__main__':
     if not os.path.isdir(img_dir):
         print 'The image source directory: '  + img_dir + \
               ' does not exist!'
-        raise SystemExit
+        raise SystemExit, 1
 
     if len(args) not in [0, 2]:
         print 'If frames are specified both first and last ' + \
               'image numbers must be chosen.'
-        raise SystemExit
+        raise SystemExit, 1
     elif len(args) == 0:
         args = [None, None]
 
@@ -626,7 +626,7 @@ if __name__ == '__main__':
 
     if aspectc not in [str(i) for i in xrange(1,5)]:
         print 'Invalid aspect ratio.'
-        raise SystemExit
+        raise SystemExit, 1
     else:
         if aspectc == '1': aspect = '1:1'
         elif aspectc == '2': aspect = '4:3'
@@ -637,7 +637,7 @@ if __name__ == '__main__':
 
     if vtype not in ['hi', 'mh', 'ml', 'lo']:
         print 'Invalid video type.'
-        raise SystemExit
+        raise SystemExit, 1
 
     out_ogg = opts.get('-o', vtype + '_movie.ogg')
 
@@ -673,7 +673,7 @@ if __name__ == '__main__':
 
     if not foundfps:
         print 'Invalid fps code or rate.'
-        raise SystemExit
+        raise SystemExit, 1
 
     if '-e' not in opts:
         enhance = False
@@ -682,7 +682,7 @@ if __name__ == '__main__':
 
     if enhance and img_pre:
         print 'Sorry, you cannot enhance already-enhanced images'
-        raise SystemExit
+        raise SystemExit, 1
 
     if '-k' not in opts:
         keep = False
@@ -694,7 +694,7 @@ if __name__ == '__main__':
 
     if (cgeom or rgeom) and not enhance:
         print 'Missing "-e" option.'
-        raise SystemExit
+        raise SystemExit, 1
 
     delay = opts.get('-D', '0')
     if verbose: print 'Linear audio delay (ms): ' + delay
@@ -706,10 +706,10 @@ if __name__ == '__main__':
             lv1 = tarfile.open(os.path.abspath(lv1file))
         except:
             print 'This does not appear to be a valid LiVES file!'
-            raise SystemExit
+            raise SystemExit, 1
         if 'header.tar' not in lv1.getnames():
             print 'This does not appear to be a valid LiVES file!'
-            raise SystemExit
+            raise SystemExit, 1
         for tfile in lv1.getmembers():
             lv1.extract(tfile, img_dir)
         for tfile in glob.glob(os.path.join(img_dir, '*.tar')):
@@ -725,7 +725,7 @@ if __name__ == '__main__':
         ext = '.png'
     else:
         print 'Cannot find any appropriate %s or %s files!' % ('.jpg','.png')
-        raise SystemExit
+        raise SystemExit, 1
     first_frame = test_file + ext
     geom = os.popen(identify + ' -format "%w %h" ' + first_frame).read().strip().split()
     last_frame = os.path.join(img_dir, img_pre + last_frame_num + ext)
@@ -742,7 +742,7 @@ if __name__ == '__main__':
         except:
             print 'Could not create the work directory ' + \
                   work_dir
-            raise SystemExit
+            raise SystemExit, 1
     if ' ' in work_dir:
         if temp_dir == '':
             temp_dir = tempfile.mkdtemp('', '.lives-', '/tmp/')
