@@ -14,6 +14,17 @@ extern "C"
 
 #include <inttypes.h>
 
+
+#ifdef IS_DARWIN
+#ifndef lseek64
+#define lseek64 lseek
+#endif
+#ifndef off64_t
+#define off64_t off_t
+#endif
+#endif
+
+
 typedef int boolean;
 #define TRUE 1
 #define FALSE 0
@@ -56,6 +67,7 @@ boolean get_frame(char *URI, int64_t frame, void **pixel_data);
 
 // opt fns
 boolean rip_audio (char *URI, char *fname, int stframe, int frames);
+void rip_audio_cleanup(void);
 boolean set_palette(int palette);
 void module_unload(void);
 
