@@ -1,6 +1,6 @@
 // audio.c
 // LiVES (lives-exe)
-// (c) G. Finch 2005 - 2009
+// (c) G. Finch 2005 - 2010
 // Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
@@ -21,7 +21,9 @@ inline void sample_silence_dS (float *dst, unsigned long nsamples) {
 
 
 void sample_move_d8_d16(short *dst, unsigned char *src,
-			unsigned long nsamples, size_t tbytes, float scale, int nDstChannels, int nSrcChannels) {
+			unsigned long nsamples, size_t tbytes, float scale, int nDstChannels, int nSrcChannels, int swap_sign) {
+  // convert 8 bit audio to 16 bit audio
+
   register int nSrcCount, nDstCount;
   register float src_offset_f=0.f;
   register int src_offset_i=0;
@@ -29,7 +31,7 @@ void sample_move_d8_d16(short *dst, unsigned char *src,
   unsigned char *ptr;
   unsigned char *src_end;
 
-  // take care of (rounding errors ?)
+  // take care of rounding errors
   src_end=src+tbytes-nSrcChannels;
 
   if(!nSrcChannels) return;
@@ -50,14 +52,11 @@ void sample_move_d8_d16(short *dst, unsigned char *src,
       nDstCount--;
 
       ptr=src+ccount+src_offset_i;
-      if (scale<0.f) {
-	ptr=ptr>src?ptr:src;
-      }
-      else {
-	ptr=ptr<src_end?ptr:src_end;
-      }
+      ptr=ptr>src?(ptr<(src_end+ccount)?ptr:(src_end+ccount)):src;
       
-      *(dst++) = *(ptr)<<8; /* copy the data over */
+      if (!swap_sign) *(dst++) = *(ptr)<<8;
+      else if (swap_sign==SWAP_U_TO_S) *(dst++)=((short)(*(ptr))-128)<<8;
+      else *((unsigned short *)(dst++))=((short)(*(ptr))+128)<<8;
       ccount++;
 
       /* if we ran out of source channels but not destination channels */
@@ -77,7 +76,7 @@ void sample_move_d8_d16(short *dst, unsigned char *src,
 
 /* convert from any number of source channels to any number of destination channels */
 void sample_move_d16_d16(short *dst, short *src,
-			 unsigned long nsamples, size_t tbytes, float scale, int nDstChannels, int nSrcChannels, gboolean swap_endian, gboolean swap_sign) {
+			 unsigned long nsamples, size_t tbytes, float scale, int nDstChannels, int nSrcChannels, int swap_endian, int swap_sign) {
   register int nSrcCount, nDstCount;
   register float src_offset_f=0.f;
   register int src_offset_i=0;
@@ -92,7 +91,7 @@ void sample_move_d16_d16(short *dst, short *src,
     src_offset_i=(int)src_offset_f*nSrcChannels;
   }
 
-  // take care of (rounding errors ?)
+  // take care of rounding errors
   src_end=src+tbytes/sizeof(short)-nSrcChannels;
 
   while (nsamples--) {
@@ -101,12 +100,7 @@ void sample_move_d16_d16(short *dst, short *src,
       // same number of channels
 
       ptr=src+src_offset_i;
-      if (scale<0.f) {
-	ptr=ptr>src?ptr:src;
-      }
-      else {
-	ptr=ptr<src_end?ptr:src_end;
-      }
+      ptr=ptr>src?(ptr<src_end?ptr:src_end):src;
 
       w_memcpy(dst,ptr,nSrcChannels*sizeof(short));
       dst+=nDstCount;
@@ -120,17 +114,23 @@ void sample_move_d16_d16(short *dst, short *src,
 	nDstCount--;
 	
 	ptr=src+ccount+src_offset_i;
-	if (scale<0.f) {
-	  ptr=ptr>src?ptr:src;
-	}
-	else {
-	  ptr=ptr<src_end?ptr:src_end;
-	}
+	ptr=ptr>src?(ptr<(src_end+ccount)?ptr:(src_end+ccount)):src;
 
 	/* copy the data over */
-	if (!swap_endian) *(dst++) = swap_sign?(uint16_t)(*ptr+SAMPLE_MAX_16BITI):*ptr;
+	if (!swap_endian) {
+	  if (!swap_sign) *(dst++) = *ptr;
+	  else if (swap_sign==SWAP_S_TO_U) *((uint16_t *)dst++) = (uint16_t)(*ptr+SAMPLE_MAX_16BITI);
+	  else *(dst++)=*ptr-SAMPLE_MAX_16BITI;
+	}
+	else if (swap_endian==SWAP_X_TO_L) {
+	  if (!swap_sign) *(dst++)=(((*ptr)&0x00FF)<<8)+((*ptr)>>8);
+	  else if (swap_sign==SWAP_S_TO_U) *((uint16_t *)dst++)=(uint16_t)(((*ptr&0x00FF)<<8)+(*ptr>>8)+SAMPLE_MAX_16BITI);
+	  else *(dst++)=((*ptr&0x00FF)<<8)+(*ptr>>8)-SAMPLE_MAX_16BITI;
+	}
 	else {
-	  *(dst++)=swap_sign?(uint16_t)(((((*ptr)&0x00FF)<<8)+((*ptr)>>8))+SAMPLE_MAX_16BITI):(((*ptr)&0x00FF)<<8)+((*ptr)>>8);
+	  if (!swap_sign) *(dst++)=(((*ptr)&0x00FF)<<8)+((*ptr)>>8);
+	  else if (swap_sign==SWAP_S_TO_U) *((uint16_t *)dst++)=(uint16_t)(((((uint16_t)(*ptr+SAMPLE_MAX_16BITI))&0x00FF)<<8)+(((uint16_t)(*ptr+SAMPLE_MAX_16BITI))>>8));
+	  else *(dst++)=((((int16_t)(*ptr-SAMPLE_MAX_16BITI))&0x00FF)<<8)+(((int16_t)(*ptr-SAMPLE_MAX_16BITI))>>8);
 	}
 
 	ccount++;
@@ -151,7 +151,7 @@ void sample_move_d16_d16(short *dst, short *src,
 
 /* convert from any number of source channels to any number of destination channels - 8 bit output */
 void sample_move_d16_d8(uint8_t *dst, short *src,
-			 unsigned long nsamples, size_t tbytes, float scale, int nDstChannels, int nSrcChannels, gboolean swap_sign) {
+			 unsigned long nsamples, size_t tbytes, float scale, int nDstChannels, int nSrcChannels, int swap_sign) {
   register int nSrcCount, nDstCount;
   register float src_offset_f=0.f;
   register int src_offset_i=0;
@@ -166,7 +166,6 @@ void sample_move_d16_d8(uint8_t *dst, short *src,
     src_offset_i=(int)src_offset_f*nSrcChannels;
   }
 
-  // take care of (rounding errors ?)
   src_end=src+tbytes/sizeof(short)-nSrcChannels;
 
   while (nsamples--) {
@@ -181,64 +180,71 @@ void sample_move_d16_d8(uint8_t *dst, short *src,
       nDstCount--;
       
       ptr=src+ccount+src_offset_i;
-      if (scale<0.f) {
-	ptr=ptr>src?ptr:src;
-      }
-      else {
-	ptr=ptr<src_end?ptr:src_end;
-      }
-      
-      /* copy the data over */
-      *(dst++) = swap_sign?(uint8_t)(((int8_t)(*ptr>>8))+128):(int8_t)(*ptr>>8);
-	ccount++;
-	
-	/* if we ran out of source channels but not destination channels */
-	/* then start the src channels back where we were */
-	if(!nSrcCount && nDstCount) {
-	  ccount=0;
-	  nSrcCount = nSrcChannels;
-	}
-      }
+      ptr=ptr>src?(ptr<(src_end+ccount)?ptr:src_end+ccount):src;
 
+      /* copy the data over */
+      if (!swap_sign) *(dst++) = (*ptr>>8);
+      else if (swap_sign==SWAP_S_TO_U) *(dst++) = (uint8_t)((int8_t)(*ptr>>8)+128);
+      else *((int8_t *)dst++) = (int8_t)((uint8_t)(*ptr>>8)-128);
+      ccount++;
+
+      /* if we ran out of source channels but not destination channels */
+      /* then start the src channels back where we were */
+      if(!nSrcCount && nDstCount) {
+	ccount=0;
+	  nSrcCount = nSrcChannels;
+      }
+    }
+    
     /* advance the the position */
     src_offset_i=(int)(src_offset_f+=scale)*nSrcChannels;
   }
 }
 
 
-/* convert from 16 bit to floating point */
-/* channels to a buffer that will hold a single channel stream */
-/* src_skip is in terms of 16bit samples */
+
 void sample_move_d16_float (float *dst, short *src, unsigned long nsamples, unsigned long src_skip, int is_unsigned, float vol) {
-  register float svol;
-  float val;
+  // convert 16 bit audio to float audio
+
+
+  register float svolp,svoln;
 
 #ifdef ENABLE_OIL
-  double x;
+  float val=0.;  // set a value to stop valgrind complaining
+  double xn,xp,xa;
   double y=0.f;
+#else
+  register float val;
+  register short valss;
 #endif
 
   if (vol==0.) vol=0.0000001f;
-  svol=SAMPLE_MAX_16BIT/vol;
+  svolp=SAMPLE_MAX_16BIT_P/vol;
+  svoln=SAMPLE_MAX_16BIT_N/vol;
 
 #ifdef ENABLE_OIL
-  x=1./svol;
+  xp=1./svolp;
+  xn=1./svoln;
+  xa=2.*vol/(SAMPLE_MAX_16BIT_P+SAMPLE_MAX_16BIT_N);
 #endif
 
   while (nsamples--) {
+
     if (!is_unsigned) {
 #ifdef ENABLE_OIL
-      oil_scaleconv_f32_s16(&val,src,1,&y,&x);
+      oil_scaleconv_f32_s16(&val,src,1,&y,val>0?&xp:&xn);
 #else
-      if ((val = (float)((*src) / svol))>1.0f) val=1.0f;
+      if ((val = (float)((float)(*src) / (*src>0?svolp:svoln) ))>1.0f) val=1.0f;
       else if (val<-1.0f) val=-1.0f;
 #endif
     }
     else {
 #ifdef ENABLE_OIL
-      oil_scaleconv_f32_u16(&val,(unsigned short *)src,1,&y,&x);
+      oil_scaleconv_f32_u16(&val,(unsigned short *)src,1,&y,&xa);
+      val-=vol;
 #else
-      if ((val=((float)((unsigned short)(*src)) / SAMPLE_MAX_16BIT - 1.0f)*vol)>1.0f) val=1.0f;
+      valss=(unsigned short)*src-SAMPLE_MAX_16BITI;
+      if ((val = (float)((float)(valss) / (valss>0?svolp:svoln) ))>1.0f) val=1.0f;
       else if (val<-1.0f) val=-1.0f;
 #endif
     }
@@ -250,11 +256,15 @@ void sample_move_d16_float (float *dst, short *src, unsigned long nsamples, unsi
 
 
 
-long sample_move_float_int(void *holding_buff, float **float_buffer, int nsamps, float scale, int chans, int asamps, int usigned, int swap_endian, float vol) {
+
+
+
+
+
+
+long sample_move_float_int(void *holding_buff, float **float_buffer, int nsamps, float scale, int chans, int asamps, int usigned, gboolean swap_endian, float vol) {
   // convert float samples back to int
   long frames_out=0l;
-  short val;
-  unsigned short valu=0;
   register int i;
   register int offs=0,coffs=0;
   register float coffs_f=0.f;
@@ -262,26 +272,47 @@ long sample_move_float_int(void *holding_buff, float **float_buffer, int nsamps,
   unsigned short *hbuffu=(unsigned short *)holding_buff;
   unsigned char *hbuffc=(guchar *)holding_buff;
 
+#ifdef ENABLE_OIL
+  double x=(SAMPLE_MAX_16BIT_N+SAMPLE_MAX_16BIT_P)/2.;
+  double y=0.f;
+  short val=0;
+  unsigned short valu=0;
+#else
+  register short val;
+  register unsigned short valu=0;
+  register float valf;
+#endif
+
   while ((nsamps-coffs)>0) {
     frames_out++;
     for (i=0;i<chans;i++) {
-      val=(short)(*(float_buffer[i]+coffs)*vol*SAMPLE_MAX_16BIT);
-      if (usigned) {
-	valu=val+SAMPLE_MAX_16BITI;
-      }
+#ifdef ENABLE_OIL
+      oil_scaleconv_s16_f32(&val,float_buffer[i]+coffs,1,&y,&x);
+#else
+      valf=*(float_buffer[i]+coffs);
+      val=(short)(valf*(valf>0.?SAMPLE_MAX_16BIT_P:SAMPLE_MAX_16BIT_N));
+#endif
+      if (usigned) valu=(val+SAMPLE_MAX_16BITI)*vol;
+      val*=vol;
+
       if (asamps==16) {
 	if (!swap_endian) {
-	  if (usigned) *(hbuffu+offs)=(float)valu*vol;
-	  else *(hbuffs+offs)=(float)val*vol;
+	  if (usigned) *(hbuffu+offs)=valu;
+	  else *(hbuffs+offs)=val;
 	}
 	else {
-	  *(hbuffc+offs)=(float)(val&0x00FF)*vol;
-	  *(hbuffc+(++offs))=(float)((val&0xFF00)>>8)*vol;
+	  if (usigned) {
+	    *(hbuffc+offs)=valu&0x00FF;
+	    *(hbuffc+(++offs))=(valu&0xFF00)>>8;
+	  }
+	  else {
+	    *(hbuffc+offs)=val&0x00FF;
+	    *(hbuffc+(++offs))=(val&0xFF00)>>8;
+	  }
 	}
       }
       else {
-	if (usigned) *(hbuffc+offs)=(guchar)((float)(valu>>8)*vol);
-	else *(hbuffc+offs)=(guchar)((float)val/256.*vol);
+	*(hbuffc+offs)=(guchar)((float)val/256.);
       }
       offs++;
     }
@@ -374,7 +405,6 @@ long sample_move_abuf_float (float **obuf, int nchans, int nsamps, int out_arate
       pthread_mutex_lock(&mainw->abuf_mutex);
       // request main thread to fill another buffer
       mainw->abufs_to_fill++;
-
       if (mainw->jackd->read_abuf<0) {
 	// playback ended while we were processing
 	pthread_mutex_unlock(&mainw->abuf_mutex);
@@ -527,10 +557,12 @@ static size_t chunk_to_int16_abuf(lives_audio_buf_t *abuf, float **float_buffer,
   int chans=abuf->achans;
   register size_t offs=abuf->samples_filled*chans;
   register int i;
+  register float valf;
 
   while (frames_out<nsamps) {
     for (i=0;i<chans;i++) {
-      abuf->data.int16buf[offs+i]=(short)(float_buffer[i][frames_out]*SAMPLE_MAX_16BIT);
+      valf=float_buffer[i][frames_out];
+      abuf->data.int16buf[offs+i]=(short)(valf*(valf>0.?SAMPLE_MAX_16BIT_P:SAMPLE_MAX_16BIT_N));
     }
     frames_out++;
     offs+=chans;
@@ -544,19 +576,36 @@ static size_t chunk_to_int16_abuf(lives_audio_buf_t *abuf, float **float_buffer,
 
 //#define DEBUG_ARENDER
 
-static void pad_with_silence(int out_fd, off64_t oins_size, long ins_size) {
-  // fill to ins_pt with zeros
+static void pad_with_silence(int out_fd, off64_t oins_size, long ins_size, int asamps, int aunsigned, gboolean big_endian) {
+  // fill to ins_pt with zeros (or 0x80.. for unsigned)
   guchar *zero_buff;
   size_t sblocksize=SILENCE_BLOCK_SIZE;
   gint sbytes=ins_size-oins_size;
-  int i;
+  register int i;
 
 #ifdef DEBUG_ARENDER
   g_print("sbytes is %d\n",sbytes);
 #endif
   if (sbytes>0) {
     lseek64(out_fd,oins_size,SEEK_SET);
-    zero_buff=g_malloc0(SILENCE_BLOCK_SIZE);
+    if (!aunsigned) zero_buff=g_malloc0(SILENCE_BLOCK_SIZE);
+    else {
+      zero_buff=g_malloc(SILENCE_BLOCK_SIZE);
+      if (asamps==1) memset(zero_buff,0x80,SILENCE_BLOCK_SIZE);
+      else {
+	for (i=0;i<SILENCE_BLOCK_SIZE;i+=2) {
+	  if (big_endian) {
+	    memset(zero_buff+i,0x80,1);
+	    memset(zero_buff+i+1,0x00,1);
+	  }
+	  else {
+	    memset(zero_buff+i,0x00,1);
+	    memset(zero_buff+i+1,0x80,1);
+	  }
+	}
+      }
+    }
+
     for (i=0;i<sbytes;i+=SILENCE_BLOCK_SIZE) {
       if (sbytes-i<SILENCE_BLOCK_SIZE) sblocksize=sbytes-i;
       dummyvar=write (out_fd,zero_buff,sblocksize);
@@ -573,7 +622,7 @@ static void pad_with_silence(int out_fd, off64_t oins_size, long ins_size) {
 
 long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *avels, gdouble *fromtime, weed_timecode_t tc_start, weed_timecode_t tc_end, gdouble *chvol, gdouble opvol_start, gdouble opvol_end, lives_audio_buf_t *obuf) {
   // called during multitrack rendering to create the actual audio file
-  // (or in-memory buffer for preview playback)
+  // (or in-memory buffer for preview playback in multitrack)
 
   // also used for fade-in/fade-out in the clip editor (opvol_start, opvol_end)
 
@@ -592,7 +641,7 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
 
   // the small chunks are processed and mixed, converted from float to int, and then written to the outfile
 
-  // if obuf != NULL we write to float to obuf instead
+  // if obuf != NULL we write to obuf instead
 
 
   // TODO - allow MAX_AUDIO_MEM to be configurable; currently this is fixed at 8 MB
@@ -660,6 +709,7 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
 
   if (out_achans==0) return 0l;
 
+
   if (to_file>-1) {
     // prepare outfile stuff
     outfilename=g_strdup_printf("%s/%s/audio",prefs->tmpdir,outfile->handle);
@@ -678,9 +728,7 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
     else out_reverse_endian=FALSE;
     
     // fill to ins_pt with zeros
-    pad_with_silence(out_fd,cur_size,ins_size);
-    sync();
-
+    pad_with_silence(out_fd,cur_size,ins_size,out_asamps,out_unsigned,out_bendian);
 
   }
   else {
@@ -746,15 +794,15 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
   }
 
   if (first_nonsilent==-1) {
-
+    // all in tracks are empty
     // output silence
     if (to_file>-1) {
       long oins_size=ins_size;
       ins_pt=tc_end/U_SEC;
       ins_pt*=out_achans*out_arate*out_asamps;
       ins_size=((long)(ins_pt/out_achans/out_asamps)+.5)*out_achans*out_asamps;
-      pad_with_silence(out_fd,oins_size,ins_size);
-      sync();
+      pad_with_silence(out_fd,oins_size,ins_size,out_asamps,out_unsigned,out_bendian);
+      //sync();
       close (out_fd);
     }
     else {
@@ -764,12 +812,17 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
 	    obuf->data.floatbuf[i][j]=0.;
 	  }
 	  else {
-	    obuf->data.int16buf[j*out_achans+i]=0;
+	    if (!out_unsigned) obuf->data.int16buf[j*out_achans+i]=0;
+	    else {
+	      if (out_bendian) obuf->data.int16buf[j*out_achans+i]=0x8000;
+	      else obuf->data.int16buf[j*out_achans+i]=0x80;
+	    }
 	  }
 	}
       }
       obuf->samples_filled+=tsamples;
     }
+
     return tsamples;
   }
 
@@ -827,28 +880,33 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
 
       zavel=avels[track]*(gdouble)in_arate[track]/(gdouble)out_arate;
 
-      tbytes=(gdouble)xsamples*ABS(zavel)*in_asamps[track]*in_achans[track];
-      tbytes=((size_t)(tbytes/in_achans[track]/(in_asamps[track])))*in_achans[track]*(in_asamps[track]);
+      tbytes=(gint)((gdouble)xsamples*ABS(zavel)+((gdouble)fastrand()/(gdouble)G_MAXUINT32))*in_asamps[track]*in_achans[track];
 
       in_buff=g_malloc(tbytes);
 
       if (zavel<0.) lseek64(in_fd[track],seekstart[track]-tbytes,SEEK_SET);
-    
+
       bytes_read=read(in_fd[track],in_buff,tbytes);
       //g_print("read %ld bytes\n",bytes_read);
 
-      if (zavel<0.) seekstart[track]-=tbytes;
+      if (zavel<0.) seekstart[track]-=bytes_read;
 
-      if (bytes_read<tbytes) memset(in_buff+bytes_read,0,tbytes-bytes_read);
+      if (bytes_read<tbytes) {
+	if (zavel>0) memset(in_buff+bytes_read,0,tbytes-bytes_read);
+	else {
+	  memmove(in_buff+tbytes-bytes_read,in_buff,bytes_read);
+	  memset(in_buff,0,tbytes-bytes_read);
+	}
+      }
 
       nframes=tbytes/(in_asamps[track])/in_achans[track]/ABS(zavel);
 
       // convert to float
       if (in_asamps[track]==1) {
-	sample_move_d8_d16 (holding_buff,(guchar *)in_buff,nframes,tbytes,zavel,out_achans,in_achans[track]);
+	sample_move_d8_d16 (holding_buff,(guchar *)in_buff,nframes,tbytes,zavel,out_achans,in_achans[track],0);
       }
       else {
-	sample_move_d16_d16(holding_buff,(short*)in_buff,nframes,tbytes,zavel,out_achans,in_achans[track],in_reverse_endian[track],FALSE);
+	sample_move_d16_d16(holding_buff,(short*)in_buff,nframes,tbytes,zavel,out_achans,in_achans[track],in_reverse_endian[track]?SWAP_X_TO_L:0,0);
       }
 
       g_free(in_buff);
@@ -884,15 +942,16 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
       }
       
       if (to_file>-1) {
+	// output to file
 	// convert back to int; use out_scale of 1., since we did our resampling in sample_move_*_d16
 	frames_out=sample_move_float_int((void *)finish_buff,chunk_float_buffer,blocksize,1.,out_achans,out_asamps*8,out_unsigned,out_reverse_endian,opvol);
 	dummyvar=write (out_fd,finish_buff,frames_out*out_asamps*out_achans);
 #ifdef DEBUG_ARENDER
 	g_print(".");
 #endif
-	sync();
       }
       else {
+	// output to memory buffer
 	if (prefs->audio_player==AUD_PLAYER_JACK) {
 	  frames_out=chunk_to_float_abuf(obuf,chunk_float_buffer,blocksize);
 	}
@@ -986,7 +1045,7 @@ void jack_rec_audio_end(void) {
   // recording ended
 
   // stop recording
-  jack_close_device(mainw->jackd_read,TRUE);
+  jack_close_device(mainw->jackd_read);
   mainw->jackd_read=NULL;
 
   // close file
@@ -1063,6 +1122,9 @@ void pulse_rec_audio_end(void) {
 
 
 static lives_audio_track_state_t *resize_audstate(lives_audio_track_state_t *ostate, int nostate, int nstate) {
+  // increase the element size of the audstate array (ostate)
+  // from nostate elements to nstate elements
+
   lives_audio_track_state_t *audstate=(lives_audio_track_state_t *)g_malloc(nstate*sizeof(lives_audio_track_state_t));
   int i;
 
@@ -1089,6 +1151,9 @@ static lives_audio_track_state_t *resize_audstate(lives_audio_track_state_t *ost
 
 
 static lives_audio_track_state_t *aframe_to_atstate(weed_plant_t *event) {
+  // parse an audio frame, and set the track file, seek and velocity values
+
+
   int error,atrack;
   int num_aclips=weed_leaf_num_elements(event,"audio_clips");
   int *aclips=weed_get_int_array(event,"audio_clips",&error);
@@ -1135,10 +1200,11 @@ lives_audio_track_state_t *get_audio_and_effects_state_at(weed_plant_t *event_li
   weed_timecode_t last_tc=0,fill_tc;
   int i;
 
+  // gets effects state (initing any effects which should be active)
+
   // optionally: gets audio state at audio frame prior to st_event, sets atstate[0].tc
   // and initialises audio buffers
 
-  // also gets effects state (initing any effects which should be active)
 
 
   fill_tc=get_event_timecode(st_event);
@@ -1237,6 +1303,8 @@ void fill_abuffer_from(lives_audio_buf_t *abuf, weed_plant_t *event_list, weed_p
   abuf->start_sample=0; // read level
 
   if (st_event!=NULL) {
+    // this is only called for the first buffered read
+
     event=st_event;
     last_tc=get_event_timecode(event);
 
@@ -1245,6 +1313,7 @@ void fill_abuffer_from(lives_audio_buf_t *abuf, weed_plant_t *event_list, weed_p
     if (aseeks!=NULL) g_free(aseeks);
 
     if (mainw->multitrack!=NULL) nfiles=weed_leaf_num_elements(mainw->multitrack->avol_init_event,"in_tracks");
+
     else nfiles=1;
 
     from_files=(int *)g_malloc(nfiles*sizint);
@@ -1256,6 +1325,16 @@ void fill_abuffer_from(lives_audio_buf_t *abuf, weed_plant_t *event_list, weed_p
       avels[i]=aseeks[i]=0.;
     }
 
+    // TODO - actually what we should do here is get the audio state for
+    // the *last* frame in the buffer and then adjust the seeks back to the
+    // beginning of the buffer, in case an audio track starts during the
+    // buffering period. The current way is fine for a preview, but when we 
+    // implement rendering of partial event lists we will need to do this
+
+    // a negative seek value would mean that we need to pad silence at the 
+    // start of the track buffer
+
+
     atstate=get_audio_and_effects_state_at(event_list,event,TRUE,exact);
     
     if (atstate!=NULL) {
@@ -1278,6 +1357,7 @@ void fill_abuffer_from(lives_audio_buf_t *abuf, weed_plant_t *event_list, weed_p
   }
 
   if (mainw->multitrack!=NULL) {
+    // get channel volumes from the mixer
     for (i=0;i<nfiles;i++) {
       if (mainw->multitrack!=NULL&&mainw->multitrack->audio_vols!=NULL) {
 	chvols[i]=(gdouble)GPOINTER_TO_INT(g_list_nth_data(mainw->multitrack->audio_vols,i))/1000000.;
@@ -1327,15 +1407,15 @@ void fill_abuffer_from(lives_audio_buf_t *abuf, weed_plant_t *event_list, weed_p
   }
   
   if (last_tc<fill_tc) {
+    // flush the rest of the audio
     render_audio_segment(nfiles, from_files, -1, avels, aseeks, last_tc, fill_tc, chvols, 1., 1., abuf);
-
     for (i=0;i<nfiles;i++) {
       // increase seek values
       aseeks[i]+=avels[i]*(fill_tc-last_tc)/U_SEC;
     }
   }
 
-  
+
   mainw->write_abuf++;
   if (mainw->write_abuf>=prefs->num_rtaudiobufs) mainw->write_abuf=0;
 
@@ -1352,8 +1432,6 @@ void fill_abuffer_from(lives_audio_buf_t *abuf, weed_plant_t *event_list, weed_p
 
 
 
-
-
 void init_jack_audio_buffers (gint achans, gint arate, gboolean exact) {
 #ifdef ENABLE_JACK
 
@@ -1414,6 +1492,7 @@ void free_jack_audio_buffers(void) {
       g_free(mainw->jackd->abufs[i]);
    }
   }
+  g_free(mainw->jackd->abufs);
 #endif
 }
 
@@ -1439,6 +1518,47 @@ void free_pulse_audio_buffers(void) {
 
 
 
+gboolean resync_audio(gint frameno) {
+  // if we are using a realtime audio player, resync to frameno
+  // and return TRUE
+
+  // otherwise return FALSE
+
+
+  // this is called for example when the play position jumps, either due
+  // to external transport changes, (jack transport, osc retrigger or goto)
+  // or if we are looping a video selection
+
+  // this is only active if "audio follows video rate/fps changes" is set
+
+  if (!(prefs->audio_opts&AUDIO_OPTS_FOLLOW_FPS)) return FALSE;
+
+#ifdef ENABLE_JACK
+  if (prefs->audio_player==AUD_PLAYER_JACK&&mainw->jackd!=NULL&&!mainw->is_rendering) {
+    jack_audio_seek_frame(mainw->jackd,frameno);
+    mainw->rec_aclip=mainw->current_file;
+    mainw->rec_avel=cfile->pb_fps/cfile->fps;
+    mainw->rec_aseek=(gdouble)mainw->jackd->seek_pos/(gdouble)(cfile->arate*cfile->achans*cfile->asampsize/8);
+    return TRUE;
+  }
+#endif
+
+#ifdef HAVE_PULSE_AUDIO
+  if (prefs->audio_player==AUD_PLAYER_PULSE&&mainw->pulsed!=NULL&&!mainw->is_rendering) {
+    pulse_audio_seek_frame(mainw->pulsed,frameno);
+    mainw->rec_aclip=mainw->current_file;
+    mainw->rec_avel=cfile->pb_fps/cfile->fps;
+    mainw->rec_aseek=(gdouble)mainw->pulsed->seek_pos/(gdouble)(cfile->arate*cfile->achans*cfile->asampsize/8);
+    return TRUE;
+  }
+#endif
+  return FALSE;
+
+}
+
+
+
+
 //////////////////////////////////////////////////////////////////////////
 
 
