@@ -1,6 +1,6 @@
 // effects.c
 // LiVES (lives-exe)
-// (c) G. Finch 2003 - 2008
+// (c) G. Finch 2003 - 2010
 // Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
@@ -54,6 +54,7 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
   gint current_file=mainw->current_file;
 
   gint new_file=current_file;
+  gint ldfile;
 
   gboolean got_no_frames=FALSE;
   gchar *tmp;
@@ -78,10 +79,10 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
 
     if (rfx->num_in_channels==2) {
       // transition has a few extra bits
-      pdefault=g_strdup_printf ("%s %d %d %d %d %d %d \"%s/%s\"",cfile->handle,rfx->status,cfile->progress_start,cfile->progress_end,cfile->hsize,cfile->vsize,clipboard->start,prefs->tmpdir,clipboard->handle);
+      pdefault=g_strdup_printf ("%s %d %d %d %d %d %s %s %d \"%s/%s\"",cfile->handle,rfx->status,cfile->progress_start,cfile->progress_end,cfile->hsize,cfile->vsize,cfile->img_type==IMG_TYPE_JPEG?"jpg":"png",clipboard->img_type==IMG_TYPE_JPEG?"jpg":"png",clipboard->start,prefs->tmpdir,clipboard->handle);
     }
     else {
-      pdefault=g_strdup_printf ("%s %d %d %d %d %d",cfile->handle,rfx->status,cfile->progress_start,cfile->progress_end,cfile->hsize,cfile->vsize);
+      pdefault=g_strdup_printf ("%s %d %d %d %d %d %s",cfile->handle,rfx->status,cfile->progress_start,cfile->progress_end,cfile->hsize,cfile->vsize,cfile->img_type==IMG_TYPE_JPEG?"jpg":"png");
     }
     // and append params
     if (is_preview) {
@@ -89,10 +90,12 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
       mainw->show_procd=FALSE;
     }
     else cmd=g_strdup("fxrender");
-    fxcommand=g_strconcat ("smogrify ",cmd,"_",plugin_name," ", pdefault, param_marshall (rfx, FALSE), NULL);
+    fxcommand=g_strconcat ("smogrify ",cmd,"_",plugin_name," ", pdefault, (tmp=param_marshall (rfx, FALSE)), NULL);
+
     g_free(plugin_name);
     g_free(cmd);
-    g_free (pdefault);
+    g_free(pdefault);
+    g_free(tmp);
   }
 
   if (!mainw->keep_pre) unlink(cfile->info_file);
@@ -117,7 +120,6 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
     cfile->undo_start=oundo_start;
     cfile->undo_end=oundo_end;
     mainw->current_file=current_file;
-    mainw->is_generating=FALSE;
     return TRUE;
   }
 
@@ -132,6 +134,7 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
     }
     else {
       if (rfx->num_in_channels==0) {
+	mainw->no_switch_dprint=TRUE;
 	if (mainw->gen_to_clipboard) {
 	  tmp=g_strdup(_("%s to clipboard..."));
 	  text=g_strdup_printf(tmp,_(rfx->action_desc));
@@ -147,9 +150,14 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
       }
     }
   }
+
+  if (!mainw->no_switch_dprint) d_print(""); // force switch text
+  ldfile=mainw->last_dprint_file;
+
   d_print(text);
   g_free(text);
   g_free(tmp);
+  mainw->last_dprint_file=ldfile;
 
   cfile->redoable=cfile->undoable=FALSE;
   gtk_widget_set_sensitive (mainw->redo, FALSE);
@@ -194,12 +202,14 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
   else cfile->fx_frame_pump=0;
 
   if (!do_progress_dialog(TRUE,TRUE,effectstring)||mainw->error) {
+    mainw->last_dprint_file=ldfile;
     do_rfx_cleanup(rfx);
     mainw->show_procd=TRUE;
     mainw->keep_pre=FALSE;
     if (mainw->error) {
       do_error_dialog (mainw->msg);
       d_print_failed();
+      mainw->last_dprint_file=ldfile;
     }
     cfile->undo_start=oundo_start;
     cfile->undo_end=oundo_end;
@@ -208,19 +218,26 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
     mainw->resizing=FALSE;
     cfile->nokeep=FALSE;
     cfile->fx_frame_pump=0;
+
     if (rfx->num_in_channels==0) {
       mainw->suppress_dprint=TRUE;
       close_current_file(current_file);
       mainw->suppress_dprint=FALSE;
     }
     else mainw->current_file=current_file;
+
     mainw->is_generating=FALSE;
+    mainw->no_switch_dprint=FALSE;
+
+    if (mainw->multitrack!=NULL) {
+      mainw->pre_src_file=-1;
+    }
+
     return FALSE;
   }
 
   do_rfx_cleanup(rfx);
 
-  mainw->is_generating=FALSE;
   mainw->resizing=FALSE;
   cfile->nokeep=FALSE;
   cfile->fx_frame_pump=0;
@@ -247,7 +264,7 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
 	  cfile->fps=cfile->pb_fps=strtod(array[3],NULL);
 	  if (cfile->fps==0.) cfile->fps=cfile->pb_fps=prefs->default_fps;
 	  cfile->end=cfile->frames=atoi(array[4]);
-	  cfile->bpp=24;
+	  cfile->bpp=cfile->img_type==IMG_TYPE_JPEG?24:32;
 	}
 	g_strfreev(array);
       }
@@ -275,7 +292,7 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
 
   if (mainw->keep_pre) {
     // this comes from a preview which then turned into processing
-    gchar *com=g_strdup_printf("smogrify mv_pre %s %d %d",cfile->handle,cfile->progress_start,cfile->progress_end);
+    gchar *com=g_strdup_printf("smogrify mv_pre %s %d %d %s",cfile->handle,cfile->progress_start,cfile->progress_end,cfile->img_type==IMG_TYPE_JPEG?"jpg":"png");
     dummyvar=system(com);
     g_free(com);
     mainw->keep_pre=FALSE;
@@ -309,6 +326,7 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
 	mainw->suppress_dprint=FALSE;
 	
 	if (!got_no_frames) mainw->current_file=new_file;
+	}
       }
       else {
 	gchar *tfile=g_strdup_printf("%s/%s/%08d.%s",prefs->tmpdir,cfile->handle,cfile->frames,prefs->image_ext);
@@ -321,6 +339,7 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
       }
 
       if (got_no_frames||cfile->frames==0) {
+	mainw->is_generating=FALSE;
 	if (!mainw->cancelled) {
 	  do_error_dialog(_("\nNo frames were generated.\n"));
 	  d_print_failed();
@@ -331,10 +350,13 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
 	  close_current_file(current_file);
 	  mainw->suppress_dprint=FALSE;
 	}
+	mainw->last_dprint_file=ldfile;
+	mainw->no_switch_dprint=FALSE;
+	if (mainw->multitrack!=NULL) mainw->current_file=mainw->multitrack->render_file;
 	return FALSE;
       }
     }
-
+  
     if (rfx->num_in_channels!=0||mainw->gen_to_clipboard) {
       if (rfx->num_in_channels==0) {
 
@@ -368,18 +390,32 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
 	cfile->is_loaded=TRUE;
 	add_to_winmenu();
 	save_clip_values(new_file);
-	if (prefs->crash_recovery) add_to_recovery_file(mainw->files[new_file]->handle);
+
+	if (mainw->multitrack!=NULL) {
+	  mt_init_clips(mainw->multitrack,mainw->current_file,TRUE);
+	  mt_clip_select(mainw->multitrack,TRUE);
+	}
+
       }
       cfile->changed=TRUE;
-      switch_to_file ((mainw->current_file=0),new_file);
+      if (mainw->multitrack==NULL) switch_to_file ((mainw->current_file=0),new_file);
+      else {
+	mainw->current_file=mainw->multitrack->render_file;
+	mainw->pre_src_file=-1;
+      }
+      
 #ifdef ENABLE_OSC
     lives_osc_notify(LIVES_OSC_NOTIFY_CLIP_OPENED,"");
 #endif
     }
-  }
 
-  d_print_done();
-  return TRUE;
+
+    mainw->is_generating=FALSE;
+    
+    d_print_done();
+    mainw->no_switch_dprint=FALSE;
+    mainw->last_dprint_file=ldfile;
+    return TRUE;
 }
 
 
@@ -426,24 +462,27 @@ gint realfx_progress (gboolean reset) {
 
   frameticks=(i-cfile->start+1.)/cfile->fps*U_SECL;
 
-  if (!pull_frame(layer,prefs->image_ext,frameticks)) {
+  if (!pull_frame(layer,cfile->img_type==IMG_TYPE_JPEG?"jpg":"png",frameticks)) {
     g_snprintf (mainw->msg,256,"error|missing image %d",i);
     return 1;
   }
 
   layer=on_rte_apply (layer, 0, 0, (weed_timecode_t)frameticks);
   layer_palette=weed_get_int_value(layer,"current_palette",&weed_error);
-  if (layer_palette!=WEED_PALETTE_RGB24&&layer_palette!=WEED_PALETTE_RGBA32) convert_layer_palette(layer,WEED_PALETTE_RGB24,0);
+
+  if (cfile->img_type==IMG_TYPE_JPEG&&layer_palette!=WEED_PALETTE_RGB24&&layer_palette!=WEED_PALETTE_RGBA32) convert_layer_palette(layer,WEED_PALETTE_RGB24,0);
+  else if (cfile->img_type==IMG_TYPE_PNG&&layer_palette!=WEED_PALETTE_RGBA32) convert_layer_palette(layer,WEED_PALETTE_RGBA32,0);
+
   if (resize_instance==NULL) resize_layer(layer,cfile->hsize,cfile->vsize,GDK_INTERP_HYPER);
   pixbuf=layer_to_pixbuf(layer);
   weed_plant_free(layer);
 
   g_snprintf(oname,256,"%s/%s/%08d.mgk",prefs->tmpdir,cfile->handle,i);
 
-  if (!strcmp (prefs->image_ext,"jpg")) {
+  if (cfile->img_type==IMG_TYPE_JPEG) {
     gdk_pixbuf_save (pixbuf, oname, "jpeg", &error,"quality", "100", NULL);
   }
-  else if (!strcmp (prefs->image_ext,"png")) {
+  else if (cfile->img_type==IMG_TYPE_PNG) {
     gdk_pixbuf_save (pixbuf, oname, "png", &error, NULL);
   }
   else {
@@ -457,7 +496,7 @@ gint realfx_progress (gboolean reset) {
   }
 
   if (++i>cfile->end) {
-    com=g_strdup_printf ("smogrify mv_mgk %s %d %d",cfile->handle,cfile->start,cfile->end);
+    com=g_strdup_printf ("smogrify mv_mgk %s %d %d %s",cfile->handle,cfile->start,cfile->end,cfile->img_type==IMG_TYPE_JPEG?"jpg":"png");
     dummyvar=system (com);
     g_free (com);
     mainw->internal_messaging=FALSE;
@@ -603,64 +642,28 @@ void deinterlace_frame(weed_plant_t *layer, weed_timecode_t tc) {
 weed_plant_t *get_blend_layer(weed_timecode_t tc) {
   file *blend_file;
   weed_plant_t *layer;
-  static gdouble blend_add_frames;
-  int oframeno;
-  static weed_timecode_t blend_tc;
-  gint blend_file_step;
+  static weed_timecode_t blend_tc=0;
+  weed_timecode_t ntc=tc;
 
   if (mainw->blend_file==-1||mainw->files[mainw->blend_file]==NULL) return NULL;
   blend_file=mainw->files[mainw->blend_file];
 
   if (mainw->blend_file!=mainw->last_blend_file) {
+    // mainw->last_blend_file is set to -1 on playback start
     mainw->last_blend_file=mainw->blend_file;
-    oframeno=blend_file->frameno;
-    blend_add_frames=0.;
+    blend_tc=tc;
   }
-  else blend_add_frames+=(gdouble)((tc-blend_tc)/U_SEC)*ABS(blend_file->pb_fps);
+  
+  blend_file->last_frameno=blend_file->frameno;
 
-  blend_tc=tc;
+  blend_file->frameno=calc_new_playback_position(mainw->blend_file,blend_tc,&ntc);
 
-  oframeno=blend_file->frameno;
-
-  blend_file_step=blend_file->pb_fps>0?1:-1;
-
-  blend_file->frameno+=(int)(blend_add_frames+.5)*blend_file_step;
-
-  blend_add_frames-=(gdouble)((blend_file->frameno-oframeno)*blend_file_step);
-
-  if (blend_file->frameno>blend_file->end) {
-    blend_add_frames-=blend_file->frameno-blend_file->end;
-    if (blend_add_frames<0.) blend_add_frames=0.;
-    if (mainw->ping_pong) {
-      blend_file->frameno=blend_file->end-(int)blend_add_frames;
-      blend_file->pb_fps=-ABS(blend_file->pb_fps);
-    }
-    else {
-      blend_file->frameno=blend_file->start+(int)blend_add_frames;
-      blend_file->pb_fps=ABS(blend_file->pb_fps);
-    }
-    blend_add_frames-=(int)blend_add_frames;
-    oframeno=blend_file->frameno;
-  }
-  else if (blend_file->frameno<blend_file->start) {
-    blend_add_frames-=blend_file->start-blend_file->frameno;
-    if (blend_add_frames<0.) blend_add_frames=0.;
-    if (mainw->ping_pong) {
-      blend_file->frameno=blend_file->start+(int)blend_add_frames;
-      blend_file->pb_fps=ABS(blend_file->pb_fps);
-    }
-    else {
-      blend_file->frameno=blend_file->end-(int)blend_add_frames;
-      blend_file->pb_fps=-ABS(blend_file->pb_fps);
-    }
-    blend_add_frames-=(int)blend_add_frames;
-    oframeno=blend_file->frameno;
-  }
+  blend_tc=ntc;
 
   layer=weed_plant_new(WEED_PLANT_CHANNEL);
   weed_set_int_value(layer,"clip",mainw->blend_file);
   weed_set_int_value(layer,"frame",blend_file->frameno);
-  if (!pull_frame(layer,prefs->image_ext,tc)) {
+  if (!pull_frame(layer,mainw->files[mainw->blend_file]->img_type==IMG_TYPE_JPEG?"jpg":"png",tc)) {
     weed_plant_free(layer);
     layer=NULL;
   }
