@@ -1,6 +1,6 @@
 // framedraw.c
 // LiVES
-// (c) G. Finch (salsaman@xs4all.nl)
+// (c) G. Finch (salsaman@xs4all.nl) 2002 - 2010
 // see file COPYING for licensing details : released under the GNU GPL 3 or later
 
 // functions for the 'framedraw' widget - lets users draw on frames :-)
@@ -422,7 +422,7 @@ void load_rfx_preview(lives_rfx_t *rfx) {
   int max_frame=0,tot_frames=0;
   gint current_file=mainw->current_file;
   weed_timecode_t tc;
-  gchar *img_ext;
+  const gchar *img_ext;
 
   if (mainw->framedraw_frame==0) mainw->framedraw_frame=1;
 
@@ -430,7 +430,9 @@ void load_rfx_preview(lives_rfx_t *rfx) {
 
   if (cfile->clip_type==CLIP_TYPE_FILE) {
     // pull some frames to get started
-    virtual_to_images(mainw->current_file,mainw->framedraw_frame,mainw->framedraw_frame+FX_FRAME_PUMP_VAL);
+    gint vend=mainw->framedraw_frame+FX_FRAME_PUMP_VAL;
+    if (vend>cfile->end) vend=cfile->end;
+    virtual_to_images(mainw->current_file,mainw->framedraw_frame,vend);
   }
 
   clear_mainw_msg();
@@ -474,18 +476,19 @@ void load_rfx_preview(lives_rfx_t *rfx) {
   }
 
   if (rfx->num_in_channels>0) {
-    img_ext=g_strdup("pre");
+    img_ext="pre";
   }
   else {
-    img_ext=g_strdup(prefs->image_ext);
+    img_ext=cfile->img_type==IMG_TYPE_JPEG?"jpg":"png";
   }
 
   tc=((mainw->framedraw_frame-1.))/cfile->fps*U_SECL;
-  pixbuf=pull_gdk_pixbuf_at_size(mainw->current_file,mainw->framedraw_frame,img_ext,tc,(gdouble)cfile->hsize/mainw->fd_scale,(gdouble)cfile->vsize/mainw->fd_scale,GDK_INTERP_HYPER);
+  pixbuf=pull_gdk_pixbuf_at_size(mainw->current_file,mainw->framedraw_frame,(gchar *)img_ext,tc,(gdouble)cfile->hsize/mainw->fd_scale,(gdouble)cfile->vsize/mainw->fd_scale,GDK_INTERP_HYPER);
 
   gtk_image_set_from_pixbuf(GTK_IMAGE(mainw->framedraw_image), pixbuf);
   gtk_widget_queue_draw (mainw->framedraw_image);
   mainw->current_file=current_file;
+  if (pixbuf!=NULL) gdk_pixbuf_unref(pixbuf);
 
   lives_set_cursor_style(LIVES_CURSOR_NORMAL,NULL);
 }
@@ -517,15 +520,18 @@ load_framedraw_image(GdkPixbuf *pixbuf) {
 
   GdkBitmap *dummy_bitmap;
   weed_timecode_t tc;
+  gboolean needs_free=FALSE;
 
   if (mainw->framedraw_frame>cfile->frames) mainw->framedraw_frame=cfile->frames;
 
   // can happen if we preview for rendered generators
-  if (mainw->multitrack==NULL&&mainw->framedraw_frame==0) return;
+  if ((mainw->multitrack==NULL||mainw->current_file!=mainw->multitrack->render_file)&&mainw->framedraw_frame==0) return;
 
   if (pixbuf==NULL) {
+    const gchar *img_ext=cfile->img_type==IMG_TYPE_JPEG?"jpg":"png";
     tc=((mainw->framedraw_frame-1.))/cfile->fps*U_SECL;
-    pixbuf=pull_gdk_pixbuf_at_size(mainw->current_file,mainw->framedraw_frame,prefs->image_ext,tc,(gdouble)cfile->hsize/mainw->fd_scale,(gdouble)cfile->vsize/mainw->fd_scale,GDK_INTERP_HYPER);
+    pixbuf=pull_gdk_pixbuf_at_size(mainw->current_file,mainw->framedraw_frame,img_ext,tc,(gdouble)cfile->hsize/mainw->fd_scale,(gdouble)cfile->vsize/mainw->fd_scale,GDK_INTERP_HYPER);
+    needs_free=TRUE;
   }
 
   if (mainw->framedraw_orig_pixmap!=NULL) {
@@ -535,8 +541,9 @@ load_framedraw_image(GdkPixbuf *pixbuf) {
   // we just want the pixmap
 
   gdk_pixbuf_render_pixmap_and_mask (pixbuf, &mainw->framedraw_orig_pixmap, &dummy_bitmap, 128);
+  if (needs_free&&pixbuf!=NULL) gdk_pixbuf_unref(pixbuf);
 
-  if (!(dummy_bitmap==NULL)) {
+  if (dummy_bitmap!=NULL) {
     gdk_pixmap_unref (dummy_bitmap);
   }
 
