@@ -1,6 +1,6 @@
 // jack.c
 // LiVES (lives-exe)
-// (c) G. Finch 2005 - 2009
+// (c) G. Finch 2005 - 2010
 // Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
@@ -17,13 +17,19 @@
 static jack_client_t *jack_transport_client;
 
 int lives_start_ready_callback (jack_transport_state_t state, jack_position_t *pos, void *arg) {
-  // we will return FALSE until the first frame is loaded, and the audio seek is ready
-  // TODO
-  //return mainw->video_seek_ready;
-  return TRUE; // for now
+  // we will return FALSE until we call gettimeofday for the origin
+
+  gboolean seek_ready=mainw->video_seek_ready;
+
+  // reset for next time
+  mainw->video_seek_ready=FALSE;
+
+  return seek_ready;
 }
 
-void lives_jack_init (void) {
+
+
+gboolean lives_jack_init (void) {
   gchar *jt_client=g_strdup_printf("LiVES-%d",getpid());
   const char *server_name="default";
   jack_options_t options=JackServerName;
@@ -44,7 +50,7 @@ void lives_jack_init (void) {
 	com=g_strdup_printf("echo \"%s -Z -d alsa\">%s",jackd_loc,prefs->jack_aserver);
 #else
 #ifdef IS_SOLARIS
-	// use OSS on Darwin
+	// use OSS on Solaris
 	com=g_strdup_printf("echo \"%s -Z -d oss\">%s",jackd_loc,prefs->jack_aserver);
 #else
 	// use coreaudio on Darwin
@@ -68,8 +74,13 @@ void lives_jack_init (void) {
 
   // startup the server
   jack_transport_client=jack_client_open (jt_client, options, &status, server_name);
+  g_free(jt_client);
+
+  if (jack_transport_client==NULL) return FALSE;
 
 #ifdef ENABLE_JACK_TRANSPORT
+  mainw->video_seek_ready=TRUE;
+  jack_activate(jack_transport_client);
   if (jack_transport_client!=NULL) jack_set_sync_callback (jack_transport_client, lives_start_ready_callback, NULL);
   gtk_timeout_add(KEY_RPT_INTERVAL,&lives_jack_poll,NULL);
 #else
@@ -81,7 +92,7 @@ void lives_jack_init (void) {
     d_print (_("JACK server started\n"));
   }
 
-  g_free(jt_client);
+  return TRUE;
 
 }
 
@@ -89,21 +100,46 @@ void lives_jack_init (void) {
 // transport handling
 
 
+
+gdouble jack_transport_get_time(void) {
+#ifdef ENABLE_JACK_TRANSPORT
+  jack_nframes_t srate; 
+  jack_position_t pos;
+  jack_transport_state_t jacktstate;
+
+  jacktstate=jack_transport_query (jack_transport_client, &pos);
+  srate=jack_get_sample_rate(jack_transport_client);
+  return (gdouble)pos.frame/(gdouble)srate;
+#endif
+  return 0.;
+}
+
+
+
+
+
+
 static void jack_transport_check_state (void) {
 #ifdef ENABLE_JACK_TRANSPORT
   jack_position_t pos;
   jack_transport_state_t jacktstate;
 
+  if (mainw->go_away) return;
+
   if (!(prefs->jack_opts&JACK_OPTS_TRANSPORT_CLIENT)) return;
 
   if (jack_transport_client==NULL) return;
 
   jacktstate=jack_transport_query (jack_transport_client, &pos);
 
-  if (mainw->jack_can_start&&(jacktstate==JackTransportRolling||jacktstate==JackTransportStarting)&&mainw->playing_file==-1&&mainw->current_file>0) {
+  if (mainw->jack_can_start&&(jacktstate==JackTransportRolling||jacktstate==JackTransportStarting)&&mainw->playing_file==-1&&mainw->current_file>0&&!mainw->is_processing) {
+    mainw->jack_can_start=FALSE;
     mainw->jack_can_stop=TRUE;
     on_playall_activate(NULL,NULL);
   }
+  else if (jacktstate==JackTransportRolling||jacktstate==JackTransportStarting) {
+    mainw->video_seek_ready=TRUE;
+  }
   if (jacktstate==JackTransportStopped) {
      if (mainw->playing_file>-1&&mainw->jack_can_stop) on_stop_activate (NULL,NULL);
      mainw->jack_can_start=TRUE;
@@ -121,10 +157,14 @@ gboolean lives_jack_poll(gpointer data) {
 }
 
 void lives_jack_end (void) {
+  jack_client_t *client=jack_transport_client;
+  jack_transport_client=NULL; // stop polling transport
 #ifdef ENABLE_JACK_TRANSPORT
-  if (jack_transport_client!=NULL) jack_client_close (jack_transport_client);
+  if (client!=NULL) {
+    jack_deactivate (client);
+    jack_client_close (client);
+  }
 #endif
-  jack_transport_client=NULL;
 }
 
 
@@ -167,9 +207,11 @@ static float set_pulse(float *buf, size_t bufsz, int step) {
 
 
 void jack_get_rec_avals(jack_driver_t *jackd) {
-  mainw->rec_aseek=jackd->seek_pos/(gdouble)(afile->arate*afile->achans*afile->asampsize/8);
   mainw->rec_aclip=jackd->playing_file;
-  mainw->rec_avel=afile->pb_fps/afile->fps;
+  if (mainw->rec_aclip!=-1) {
+    mainw->rec_aseek=jackd->seek_pos/(gdouble)(afile->arate*afile->achans*afile->asampsize/8);
+    mainw->rec_avel=afile->pb_fps/afile->fps;
+  }
 }
 
 
@@ -211,6 +253,7 @@ static int audio_process (nframes_t nframes, void *arg) {
 	  else (jackd->aPlayPtr->data)=NULL;
 	  jackd->seek_pos=0;
 	  jackd->playing_file=new_file;
+	  jackd->frames_written=0;
 	}
 	g_free(filename);
       }
@@ -259,7 +302,7 @@ static int audio_process (nframes_t nframes, void *arg) {
   /* handle playing state */
   if (jackd->state==JackTransportRolling||jackd->play_when_stopped) {
     gulong jackFramesAvailable = nframes; /* frames we have left to write to jack */
-    gulong inputFramesAvailable;          /* frames we have available this loop */
+   gulong inputFramesAvailable;          /* frames we have available this loop */
     gulong numFramesToWrite;              /* num frames we are writing this loop */
     glong in_frames=0;
     gulong in_bytes=0;
@@ -313,7 +356,7 @@ static int audio_process (nframes_t nframes, void *arg) {
       else {
 	if (G_LIKELY(jackd->fd>0)) {
 	  jackd->aPlayPtr->size=0;
-	  in_bytes=ABS((in_frames=((gdouble)jackd->sample_in_rate/(gdouble)jackd->sample_out_rate*(gdouble)jackFramesAvailable+(fastrand()/G_MAXUINT32))))*jackd->num_input_channels*jackd->bytes_per_channel;
+	  in_bytes=ABS((in_frames=((gdouble)jackd->sample_in_rate/(gdouble)jackd->sample_out_rate*(gdouble)jackFramesAvailable+((gdouble)fastrand()/(gdouble)G_MAXUINT32))))*jackd->num_input_channels*jackd->bytes_per_channel;
 	  if ((shrink_factor=(gfloat)in_frames/(gfloat)jackFramesAvailable)<0.f) {
 	    // reverse playback
 	    if ((jackd->seek_pos-=in_bytes)<0) {
@@ -434,20 +477,21 @@ static int audio_process (nframes_t nframes, void *arg) {
 	
 	numFramesToWrite = MIN(jackFramesAvailable, inputFramesAvailable/ABS(shrink_factor)); /* write as many bytes as we have space remaining, or as much as we have data to write */
 	
+
 #ifdef DEBUG_AJACK
 	g_printerr("inputFramesAvailable after conversion %ld\n", (gulong)((gdouble)inputFramesAvailable/shrink_factor));
-	g_printerr("nframes == %d, jackFramesAvailable == %ld,\n\tjackd->num_input_channels == %ld, jackd->num_output_channels == %ld\n",  nframes, jackFramesAvailable, jackd->num_input_channels, jackd->num_output_channels);
+	g_printerr("nframes == %d, jackFramesAvailable == %ld,\n\tjackd->num_input_channels == %ld, jackd->num_output_channels == %ld, nf2w %ld, in_bytes %d, sf %.8f\n",  nframes, jackFramesAvailable, jackd->num_input_channels, jackd->num_output_channels, numFramesToWrite, in_bytes, shrink_factor);
 #endif
 	
 	/* convert from 8 bit to 16 bit and mono to stereo if necessary */
 	/* resample as we go */
 	if(jackd->bytes_per_channel==1) {
-	  sample_move_d8_d16 ((short *)(jackd->sound_buffer + (nframes-jackFramesAvailable) / sizeof(short)),(guchar *)buffer, numFramesToWrite, in_bytes, shrink_factor, jackd->num_output_channels, jackd->num_input_channels);
+	  sample_move_d8_d16 ((short *)(jackd->sound_buffer + (nframes-jackFramesAvailable) / sizeof(short)),(guchar *)buffer, numFramesToWrite, in_bytes, shrink_factor, jackd->num_output_channels, jackd->num_input_channels,0);
 	}
 	/* 16 bit input samples */
 	/* resample as we go */
 	else {
-	  sample_move_d16_d16((short*)jackd->sound_buffer + ((nframes - jackFramesAvailable) * jackd->bytes_per_channel * jackd->num_output_channels) / sizeof(short), (short*)buffer, numFramesToWrite, in_bytes, shrink_factor, jackd->num_output_channels, jackd->num_input_channels, jackd->reverse_endian, FALSE);
+	  sample_move_d16_d16((short*)jackd->sound_buffer + ((nframes - jackFramesAvailable) * jackd->bytes_per_channel * jackd->num_output_channels) / sizeof(short), (short*)buffer, numFramesToWrite, in_bytes, shrink_factor, jackd->num_output_channels, jackd->num_input_channels, jackd->reverse_endian?SWAP_X_TO_L:0, 0);
 	}
 	
 	jackd->frames_written+=numFramesToWrite;
@@ -597,10 +641,11 @@ int jack_get_srate (nframes_t nframes, void *arg) {
 }
 
 
+
 void jack_shutdown(void* arg) {
   jack_driver_t* jackd = (jack_driver_t*)arg;
 
-  jackd->client = 0; /* reset client */
+  jackd->client = NULL; /* reset client */
   jackd->jackd_died = TRUE;
   jackd->msgq=NULL;
 
@@ -630,34 +675,30 @@ static void jack_reset_driver(jack_driver_t *jackd) {
 }
 
 
-void jack_close_device(jack_driver_t* jackd, gboolean close_client) {
+void jack_close_device(jack_driver_t* jackd) {
   int i;
 
-  if (close_client) {
-    //g_printerr("closing the jack client thread\n");
-    if (jackd->client) {
-      jack_deactivate(jackd->client); /* supposed to help the jack_client_close() to succeed */
-      //g_printerr("after jack_deactivate()\n");
-      jack_client_close(jackd->client);
-    }
-
-    jack_reset_driver(jackd);
-    jackd->client=0;
-    free(jackd->sound_buffer);
-    jackd->sound_buffer=NULL;
-    jackd->buffer_size=0;
-
-    /* free up the port strings */
-    //g_printerr("freeing up port strings\n");
-    if (jackd->jack_port_name_count>1) {
-      for (i=0;i<jackd->jack_port_name_count;i++) free(jackd->jack_port_name[i]);
-      free(jackd->jack_port_name);
-    }
+  //g_printerr("closing the jack client thread\n");
+  if (jackd->client) {
+    jack_deactivate(jackd->client); /* supposed to help the jack_client_close() to succeed */
+    //g_printerr("after jack_deactivate()\n");
+    jack_client_close(jackd->client);
   }
-  else {
-    if(!jackd->client) {
-      g_printerr("critical error, closing a device that has no client\n");
-    }
+  
+  jack_reset_driver(jackd);
+  jackd->client=NULL;
+  free(jackd->sound_buffer);
+  jackd->sound_buffer=NULL;
+  jackd->buffer_size=0;
+  
+  jackd->is_active=FALSE;
+
+
+  /* free up the port strings */
+  //g_printerr("freeing up port strings\n");
+  if (jackd->jack_port_name_count>1) {
+    for (i=0;i<jackd->jack_port_name_count;i++) free(jackd->jack_port_name[i]);
+    free(jackd->jack_port_name);
   }
 }
 
@@ -688,12 +729,14 @@ int jack_open_device(jack_driver_t *jackd) {
   jackd->sound_buffer=NULL;
   jackd->buffer_size=0;
 
+  jackd->is_active=FALSE;
+
   /* set up an error handler */
   jack_set_error_function(jack_error_func);
   jackd->client=NULL;
 
   gettimeofday(&otv, NULL);
-  stime=otv.tv_sec*1000000000+otv.tv_usec;
+  stime=otv.tv_sec*1000000+otv.tv_usec;
 
   while (jackd->client==NULL&&ntime<JACK_START_WAIT) {
     jackd->client = jack_client_open (client_name, options, &status, server_name);
@@ -701,7 +744,7 @@ int jack_open_device(jack_driver_t *jackd) {
     g_usleep(prefs->sleep_time);
 
     gettimeofday(&otv, NULL);
-    ntime=(otv.tv_sec*1000000000+otv.tv_usec-stime)>>10;
+    ntime=(otv.tv_sec*1000000+otv.tv_usec-stime);
   }
 
 
@@ -747,6 +790,7 @@ int jack_open_device(jack_driver_t *jackd) {
      the sample rate of the system changes. */
   jack_set_sample_rate_callback(jackd->client, jack_get_srate, jackd);
 
+
   /* tell the JACK server to call `jack_shutdown()' if
      it ever shuts down, either entirely, or if it
      just decides to stop calling us. */
@@ -855,6 +899,8 @@ int jack_driver_activate (jack_driver_t *jackd) {
   const char** ports;
   gboolean failed=FALSE;
 
+  if (jackd->is_active) return 0; // already running
+
   /* tell the JACK server that we are ready to roll */
   if (jack_activate(jackd->client)) {
     //ERR( "cannot activate client\n");
@@ -936,10 +982,12 @@ int jack_driver_activate (jack_driver_t *jackd) {
     }
   }
 
+  jackd->is_active=TRUE;
+
   /* if something failed we need to shut the client down and return 0 */
   if (failed) {
     g_printerr("failed, closing and returning error\n");
-    jack_close_device(jackd,TRUE);
+    jack_close_device(jackd);
     return 1;
   }
 
@@ -1045,7 +1093,7 @@ int jack_read_driver_activate (jack_driver_t *jackd) {
   /* if something failed we need to shut the client down and return 0 */
   if (failed) {
     g_printerr("failed, closing and returning error\n");
-    jack_close_device(jackd,TRUE);
+    jack_close_device(jackd);
     return 1;
   }
 
@@ -1073,7 +1121,7 @@ jack_driver_t *jack_get_driver(gint dev_idx, gboolean is_output) {
 #endif
   
   /* should we try to restart the jack server? */
-  if (jackd->jackd_died&&jackd->client==0) {
+  if (jackd->jackd_died&&jackd->client==NULL) {
     struct timeval now;
     gettimeofday(&now, 0);
     
@@ -1126,6 +1174,7 @@ int jack_audio_init(void) {
     jackd->out_chans_available=0;
     jackd->is_output=TRUE;
     jackd->read_abuf=-1;
+    jackd->playing_file=-1;
   }
   return 0;
 }
@@ -1170,20 +1219,31 @@ volatile aserver_message_t *jack_get_msgq(jack_driver_t *jackd) {
   return jackd->msgq;
 }
 
-gint64 lives_jack_get_time(jack_driver_t *jackd) {
+gint64 lives_jack_get_time(jack_driver_t *jackd, gboolean absolute) {
+  // get the time in ticks since either playback started or since last seek
+
   volatile aserver_message_t *msg=jackd->msgq;
   gdouble frames_written=jackd->frames_written;
   if (frames_written<0.) frames_written=0.;
   if (msg!=NULL&&msg->command==ASERVER_CMD_FILE_SEEK) while (jack_get_msgq(jackd)!=NULL); // wait for seek
-  if (jackd->is_output) return jackd->audio_ticks+(gint64)(frames_written/(gdouble)jackd->sample_out_rate*U_SEC);
-  return jackd->audio_ticks+(gint64)(frames_written/(gdouble)jackd->sample_in_rate*U_SEC);
+  if (jackd->is_output) return jackd->audio_ticks*absolute+(gint64)(frames_written/(gdouble)jackd->sample_out_rate*U_SEC);
+  return jackd->audio_ticks*absolute+(gint64)(frames_written/(gdouble)jackd->sample_in_rate*U_SEC);
 }
 
 
+gdouble lives_jack_get_pos(jack_driver_t *jackd) {
+  // get current time position (seconds) in audio file
+  return jackd->seek_pos/(gdouble)(afile->arate*afile->achans*afile->asampsize/8);
+}
+
+
+
 static aserver_message_t jack_message;
 
 void jack_audio_seek_frame (jack_driver_t *jackd, gint frame) {
-  // seek to frame "frame" in current file
+  // seek to frame "frame" in current audio file
+  // position will be adjusted to (floor) nearest sample
+
   volatile aserver_message_t *jmsg;
   if (frame<1) frame=1;
   long seekstart;
@@ -1198,7 +1258,11 @@ void jack_audio_seek_frame (jack_driver_t *jackd, gint frame) {
 
 
 long jack_audio_seek_bytes (jack_driver_t *jackd, long bytes) {
-  // seek to relative "secs" in current file
+  // seek to position "bytes" in current audio file
+  // position will be adjusted to (floor) nearest sample
+
+  // if the position is > size of file, we will seek to the end of the file
+
   volatile aserver_message_t *jmsg;
   long seekstart;
   do {
