@@ -1,6 +1,6 @@
 // main.h
 // LiVES
-// (c) G. Finch (salsaman@xs4all.nl) 2003 - 2009
+// (c) G. Finch (salsaman@xs4all.nl) 2003 - 2010
 // see file ../COPYING for full licensing details
 
 /*  This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,7 @@
 /*
     NO WARRANTY
 
-      BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
+    BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
       FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
 OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
       PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
@@ -66,7 +66,8 @@ POSSIBILITY OF SUCH DAMAGES.
 // i.e. $GUI_BOOTSTRAP_FILE in smogrify
 #define BOOTSTRAP_NAME "/tmp/.smogrify"
 
-#define MAX_FILES 65536
+// max files is actually 1 more than this, since file 0 is the clipboard
+#define MAX_FILES 65535
 
 // this must match AC_PREFIX_DEFAULT in configure.in
 // TODO - when lives-plugins is a separate package, use pkg-config to get PREFIX and remove PREFIX_DEFAULT
@@ -99,6 +100,14 @@ POSSIBILITY OF SUCH DAMAGES.
 #define LIVES_FEATURE_URL "http://sourceforge.net/tracker/?group_id=64341&atid=507142"
 #define LIVES_TRANSLATE_URL "https://translations.launchpad.net/lives/trunk"
 
+#ifdef IS_DARWIN
+#ifndef off64_t
+#define off64_t off_t
+#endif
+#ifndef lseek64
+#define lseek64 lseek
+#endif
+#endif
 
 // insert methods
 #define AFTER_END 1
@@ -126,9 +135,20 @@ POSSIBILITY OF SUCH DAMAGES.
 
 #ifndef IS_SOLARIS
 #define LIVES_INLINE inline
+#else
+#define LIVES_INLINE
 #endif
 
 
+
+#ifdef __GNUC__
+#  define WARN_UNUSED  __attribute__((warn_unused_result))
+#else
+#  define WARN_UNUSED
+#endif
+
+
+
 // this struct is used only when physically resampling frames on the disk
 // we create an array of these and write them to the disk
 typedef struct {
@@ -146,6 +166,7 @@ typedef struct {
   GtkWidget *pause_button;
   GtkWidget *preview_button;
   GtkWidget *cancel_button;
+  GtkWidget *scrolledwindow;
   guint frames_done;
 } process;
 
@@ -313,8 +334,6 @@ typedef struct {                // corresponds to one clip in the GUI
 #define UNDO_RESIZABLE 2
 #define UNDO_MERGE 3
 
-  // reorder - deprecated
-#define UNDO_REORDER 4
 
 #define UNDO_RESAMPLE 5
 #define UNDO_TRIM_AUDIO 6
@@ -380,15 +399,12 @@ typedef struct {                // corresponds to one clip in the GUI
   // size must be >= frames, MUST be contiguous in memory
 
   int *frame_index_back; // for undo
-  //int nbfi; // number of backup frame indices
 
   gint fx_frame_pump; // rfx frame pump for virtual clips (CLIP_TYPE_FILE)
 
 #define FX_FRAME_PUMP_VAL 100 // how many frames at a time to pump to rfx
 
-  //int **forward_frame_index; // for rendo
-  //int nffi; // number of forward frame indices
-
+#define IMG_BUFF_SIZE 4096 // chunk size for reading images
 
   gboolean ratio_fps; // if the fps was set by a ratio
 
@@ -401,6 +417,18 @@ typedef struct {                // corresponds to one clip in the GUI
 
   gboolean deinterlace;
 
+#define IMG_TYPE_UNKNOWN (1<<0)
+#define IMG_TYPE_JPEG (1<<1)
+#define IMG_TYPE_PNG (1<<2)
+
+  guint img_type;
+
+  // layout map for the current layout
+  gint stored_layout_frame;
+  gint stored_layout_idx;
+  gdouble stored_layout_audio;
+  gdouble stored_layout_fps;
+
   // TODO - change to lives_clip_t
 } file;
 
@@ -426,6 +454,7 @@ typedef struct {
   gboolean has_jackd;
   gboolean has_pulse_audio;
   gboolean has_xwininfo;
+  gboolean has_gdb;
 
   // home directory - default location for config file - locale encoding
   gchar home_dir[256];
@@ -443,6 +472,8 @@ typedef struct {
   gchar *myname_full;
   gchar *myname;
 
+  gboolean has_stderr;
+
 } capability;
 
 
@@ -499,6 +530,8 @@ void resize_play_window (void);
 void kill_play_window (void);
 void make_preview_box (void);
 void add_to_playframe (void);
+GtkWidget* create_cdtrack_dialog (gint type, gpointer user_data);
+GtkTextView *create_output_textview(void);
 
 // dialogs.c
 gboolean do_progress_dialog(gboolean visible, gboolean cancellable, const gchar *text);
@@ -509,6 +542,7 @@ void do_error_dialog(const gchar *text);
 void do_error_dialog_with_check(const gchar *text, gint warn_mask_number);
 void do_blocking_error_dialog(const gchar *text);
 void do_error_dialog_with_check_transient(const gchar *text, gboolean is_blocking,gint warn_mask_number, GtkWindow *transient);
+void add_warn_check (GtkBox *box, gint warn_mask_number);
 void do_memory_error_dialog (void);
 void too_many_files(void);
 void tempdir_warning (void);
@@ -522,16 +556,17 @@ gboolean do_comments_dialog (void);
 void do_auto_dialog(const gchar *text, gint type);
 void do_encoder_acodec_error (void);
 void do_encoder_sox_error(void);
-gboolean rdet_suggest_values (gint width, gint height, gdouble fps, gint fps_num, gint fps_denom, gint arate, gboolean anr, gboolean ignore_fps);
-gboolean do_encoder_restrict_dialog (gint width, gint height, gdouble fps, gint fps_num, gint fps_denom, gint arate, gboolean anr);
+gboolean rdet_suggest_values (gint width, gint height, gdouble fps, gint fps_num, gint fps_denom, gint arate, gint asigned, gboolean anr, gboolean ignore_fps);
+gboolean do_encoder_restrict_dialog (gint width, gint height, gdouble fps, gint fps_num, gint fps_denom, gint arate, gint asigned, gboolean anr);
 void do_keys_window (void);
 void do_mt_keys_window (void);
 void do_messages_window (void);
 void do_firstever_dialog (void);
 void do_upgrade_error_dialog(void);
+void do_no_mplayer_sox_error(void);
 void do_aud_during_play_error(void);
 void do_rendered_fx_dialog(void);
-void do_layout_scrap_file_error(GtkWindow *window);
+void do_layout_scrap_file_error(void);
 void do_set_load_lmap_error(void);
 gboolean do_set_duplicate_warning (gchar *new_set);
 gboolean do_set_rename_old_layouts_warning(gchar *new_set);
@@ -541,12 +576,12 @@ gboolean do_yuv4m_open_warning(void);
 void do_mt_undo_mem_error(void);
 void do_mt_undo_buf_error(void);
 void do_mt_set_mem_error(gboolean has_mt, gboolean trans);
-void do_mt_audchan_error(lives_mt *mt, gint warn_mask);
-void do_mt_no_audchan_error(lives_mt *mt);
-void do_mt_no_jack_error(lives_mt *mt, gint warn_mask);
-gboolean do_mt_rect_prompt(lives_mt *mt);
+void do_mt_audchan_error(gint warn_mask);
+void do_mt_no_audchan_error(void);
+void do_mt_no_jack_error(gint warn_mask);
+gboolean do_mt_rect_prompt(void);
 void do_audrate_error_dialog(void);
-gboolean do_event_list_warning(lives_mt *mt);
+gboolean do_event_list_warning(void);
 void do_nojack_rec_error(void);
 void do_vpp_palette_error (void);
 void do_vpp_fps_error (void);
@@ -556,12 +591,20 @@ gboolean do_original_lost_warning(gchar *fname);
 void do_no_decoder_error(gchar *fname);
 void do_jack_noopen_warn(void);
 void do_jack_noopen_warn2(void);
+void do_file_perm_error(gchar *file_name);
+void do_dir_perm_error(gchar *dir_name);
+void do_encoder_img_ftm_error(render_details *rdet);
+void do_after_crash_warning (void);
+void do_bad_layout_error(void);
+void do_card_in_use_error(void);
+void do_dev_busy_error(gchar *devstr);
 
 gboolean process_one (gboolean visible);
 void do_threaded_dialog(gchar *text, gboolean has_cancel);
 void end_threaded_dialog(void);
 void response_ok (GtkButton *button, gpointer user_data);
 void response_cancel (GtkButton *button, gpointer user_data);
+void pump_io_chan(GIOChannel *iochan);
 
 void do_splash_progress(void);
 
@@ -583,7 +626,7 @@ void deduce_file(gchar *filename, gdouble start_time, gint end);
 void open_file (const gchar *filename);
 void open_file_sel(const gchar *file_name,gdouble start_time, gint frames);
 void open_fw_device (void);
-gboolean get_new_handle(gint index, gchar *name);
+gboolean get_new_handle(gint index, const gchar *name);
 gboolean get_temp_handle(gint index, gboolean create);
 void get_handle_from_info_file(gint index);
 void create_cfile(void);
@@ -593,7 +636,7 @@ gboolean save_frame(gint frame, const gchar *filename);
 void wait_for_stop (const gchar *stop_command);
 void save_clip_values(gint which_file);
 void add_to_recovery_file (gchar *handle);
-void rewrite_recovery_file(gint closed_file); // closed_file < 0 for no closed file
+void rewrite_recovery_file(void);
 gboolean check_for_recovery_files (gboolean auto_recover);
 void recover_layout_map(gint numclips);
 gchar *get_deinterlace_string(void);
@@ -613,7 +656,8 @@ void open_set_file (gchar *set_name, gint clipnum);
 
 // saveplay.c scrap file
 gboolean open_scrap_file (void);
-gint save_to_scrap_file (GdkPixbuf *pixbuf);
+gint save_to_scrap_file (weed_plant_t *layer);
+gboolean load_from_scrap_file(weed_plant_t *layer, int frame);
 void close_scrap_file (void);
 
 
@@ -628,9 +672,9 @@ void load_start_image(gint frame);
 void load_end_image(gint frame);
 void load_preview_image(gboolean update_always);
 
-gboolean pull_frame(weed_plant_t *layer, gchar *image_ext, weed_timecode_t tc);
-gboolean pull_frame_at_size (weed_plant_t *layer, gchar *image_ext, weed_timecode_t tc, int width, int height, int target_palette);
-GdkPixbuf *pull_gdk_pixbuf_at_size(gint clip, gint frame, gchar *image_ext, weed_timecode_t tc, gint width, gint height, GdkInterpType interp);
+gboolean pull_frame(weed_plant_t *layer, const gchar *image_ext, weed_timecode_t tc);
+gboolean pull_frame_at_size (weed_plant_t *layer, const gchar *image_ext, weed_timecode_t tc, int width, int height, int target_palette);
+GdkPixbuf *pull_gdk_pixbuf_at_size(gint clip, gint frame, const gchar *image_ext, weed_timecode_t tc, gint width, gint height, GdkInterpType interp);
 
 void load_frame_image(gint frame, gint last_frame);
 void sensitize(void);
@@ -642,7 +686,7 @@ void switch_to_file(gint old_file, gint new_file);
 void do_quick_switch (gint new_file);
 void resize (gdouble scale);
 GdkPixbuf *lives_scale_simple (GdkPixbuf *, gint width, gint height);
-void read_file_details(const gchar *file_name, gboolean only_check_for_audio);
+gboolean read_file_details(const gchar *file_name, gboolean only_check_for_audio);
 void do_start_messages(void);
 void set_palette_colours (void);
 void set_main_title(const gchar *filename, gint or_untitled_number);
@@ -696,6 +740,7 @@ void save_future_prefs(void);
 void set_menu_text(GtkWidget *menu, const gchar *text, gboolean use_mnemonic);
 void get_menu_text(GtkWidget *menu, gchar *text);
 void get_menu_text_long(GtkWidget *menuitem, gchar *text);
+gint get_box_child_index (GtkBox *box, GtkWidget *tchild);
 void reset_clip_menu (void);
 void get_play_times(void);
 void get_total_time (file *file);
@@ -715,8 +760,9 @@ void prepare_to_play_foreign(void);
 gboolean after_foreign_play(void);
 gboolean check_file(const gchar *file_name, gboolean check_exists);  // check if file exists
 gboolean check_dir_access (gchar *dir);
-glong get_file_size(int fd);
-glong sget_file_size(gchar *name);
+gulong get_file_size(int fd);
+gulong sget_file_size(gchar *name);
+gulong get_fs_free(const gchar *dir);
 gboolean ensure_isdir(gchar *fname);
 gchar *ensure_extension(gchar *fname, gchar *ext);
 void activate_url_inner(const gchar *link);
@@ -730,11 +776,12 @@ LIVES_INLINE gint calc_frame_from_time3 (gint filenum, gdouble time); // nearest
 
 gboolean check_for_ratio_fps (gdouble fps);
 gdouble get_ratio_fps(gchar *string);
+void calc_maxspect(gint rwidth, gint rheight, gint *cwidth, gint *cheight);
 
 gchar *remove_trailing_zeroes(gdouble val);
 void toggle_button_toggle (GtkToggleButton *tbutton);
 void remove_layout_files(GList *lmap);
-gboolean add_lmap_error(int lerror, gchar *name, gpointer user_data, gint clipno, gint frameno, gdouble atime);
+gboolean add_lmap_error(int lerror, gchar *name, gpointer user_data, gint clipno, gint frameno, gdouble atime, gboolean affects_current);
 void clear_lmap_errors(void);
 gboolean prompt_remove_layout_files(void);
 gboolean is_legal_set_name(gchar *set_name, gboolean allow_dupes);
@@ -746,10 +793,9 @@ void get_frame_count(gint idx);
 gint count_resampled_frames (gint in_frames, gdouble orig_fps, gdouble resampled_fps);
 gboolean int_array_contains_value(int *array, int num_elems, int value);
 gboolean check_for_lock_file(gchar *set_name, gint type);
-gint real_clips_available(void);
 void g_list_free_strings(GList *list);
 void gtk_tooltips_copy(GtkWidget *dest, GtkWidget *source);
-
+void adjustment_configure(GtkAdjustment *adjustment, gdouble value, gdouble lower, gdouble upper, gdouble step_increment, gdouble page_increment, gdouble page_size);
 gboolean create_event_space(gint length_in_eventsb);
 void add_to_recent(const gchar *filename, gdouble start, gint frames, const gchar *file_open_params);
 gint verhash (gchar *version);
@@ -759,12 +805,14 @@ void zero_spinbuttons (void);
 void draw_little_bars (void);
 void set_sel_label (GtkWidget *label);
 void clear_mainw_msg (void);
-gint get_token_count (gchar *string, int delim);
+gint get_token_count (const gchar *string, int delim);
 GdkPixmap* gdk_pixmap_copy (GdkPixmap *pixmap);
 GdkPixbuf *gdk_pixbuf_new_blank(gint width, gint height, int palette);
 void get_border_size (GtkWidget *win, gint *bx, gint *by);
 gchar *g_strappend (gchar *string, gint len, const gchar *new);
+GList *g_list_append_unique(GList *xlist, gchar *add);
 void find_when_to_stop (void);
+gint calc_new_playback_position(gint fileno, weed_timecode_t otc, weed_timecode_t *ntc);
 void minimise_aspect_delta (gdouble allowed_aspect,gint hblock,gint vblock,gint hsize,gint vsize,gint *width,gint *height);
 
 GList *get_set_list(const gchar *dir);
@@ -790,6 +838,14 @@ void lives_set_cursor_style(gint cstyle, GdkWindow *window);
 #define LIVES_CURSOR_BLOCK 1
 #define LIVES_CURSOR_AUDIO_BLOCK 2
 #define LIVES_CURSOR_BUSY 3
+#define LIVES_CURSOR_FX_BLOCK 4
+
+
+
+gchar *text_view_get_text(GtkTextView *textview);
+void text_view_set_text(GtkTextView *textview, const gchar *text);
+
+
 
 // effects.c
 void invalidate_pixel_buffers (void); // TODO - exterminate !
@@ -802,6 +858,7 @@ gboolean check_encoder_restrictions (gboolean get_extension, gboolean user_audio
 void lives_exit (void);
 void on_check_clicked(void);
 void on_fileread_clicked (GtkFileChooser *fch, gpointer widget);
+gboolean dirchange_callback (GtkAccelGroup *, GObject *, guint, GdkModifierType, gpointer user_data);
 
 // paramspecial.c
 gboolean mt_framedraw(lives_mt *, GdkPixbuf *);
