@@ -1,6 +1,6 @@
 // mainwindow.h
 // LiVES (lives-exe)
-// (c) G. Finch <salsaman@xs4all.nl> 2003 - 2009
+// (c) G. Finch <salsaman@xs4all.nl> 2003 - 2010
 // Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
@@ -57,6 +57,11 @@
 #define EXT_CNTL_MIDI 2
 
 
+// default size for generators
+#define DEF_GEN_WIDTH 640
+#define DEF_GEN_HEIGHT 480
+
+
 
 typedef struct {
   // set in set_palette_colours()
@@ -219,7 +224,16 @@ typedef struct {
 #define CANCEL_KILL 0  // normal - kill background processes working on current clip
 #define CANCEL_SOFT 1 // just cancel in GUI (for keep, etc)
 
-  weed_plant_t *event_list;
+  weed_plant_t *event_list; // current event_list, for recording
+  weed_plant_t *stored_event_list; // stored mt -> clip editor
+  gboolean stored_event_list_changed;
+  gboolean stored_layout_save_all_vals;
+  gchar stored_layout_name[256];
+
+  GList *stored_layout_undos;
+  size_t sl_undo_buffer_used;
+  unsigned char *sl_undo_mem;
+  gint sl_undo_offset;
 
   gshort endian;
 
@@ -366,7 +380,9 @@ typedef struct {
                      // close the current clip, or call load_frame_image()
   gint new_clip;
 
-  gboolean reverse_pb; 
+  gint aud_file_to_kill; // # of audio file to kill on crash
+
+  gboolean reverse_pb; // used in osc.c
 
   // TODO - make this a mutex and more finely grained : things we need to block are (clip switches, clip closure, effects on/off, etc)
   gboolean osc_block;
@@ -481,7 +497,6 @@ typedef struct {
   GdkPixbuf *imframe;
   GdkPixbuf *imsep;
   GtkWidget *LiVES;
-  GtkWidget *save;
   GtkWidget *open;
   GtkWidget *open_sel;
   GtkWidget *open_vcd_menu;
@@ -496,6 +511,7 @@ typedef struct {
   GtkWidget *open_device_submenu;
   GtkWidget *open_firewire;
   GtkWidget *open_hfirewire;
+  GtkWidget *add_live_menu;
   GtkWidget *recent_menu;
   GtkWidget *recent_submenu;
   GtkWidget *recent1;
@@ -599,6 +615,7 @@ typedef struct {
   GtkWidget *save_rte_defs;
   GtkWidget *vj_reset;
   GtkWidget *mt_menu;
+  GtkWidget *troubleshoot;
 
   // for the fileselection preview
   GtkWidget *fs_playarea;
@@ -624,7 +641,7 @@ typedef struct {
   GtkWidget *laudbar;
   GtkWidget *raudbar;
 
-#define MAIN_SPIN_SPACER 76 // pixel spacing for start/end spins for clip and multitrack editors
+#define MAIN_SPIN_SPACER 52 // pixel spacing for start/end spins for clip and multitrack editors
   GtkWidget *spinbutton_end;
   GtkWidget *spinbutton_start;
 
@@ -792,6 +809,13 @@ typedef struct {
   GList *affected_layouts_map; // map of layouts with errors
   GList *current_layouts_map; // map of all layouts for set
 
+  // list of pairs of marks in affected_layouts_map, text between them should be deleted when 
+  // stored_layout is deleted
+  GList *affected_layout_marks;
+
+  // immediately (to be) affected layout maps
+  GList *xlays;
+
   gchar *recovery_file;  // the filename of our recover file
   gboolean leave_recovery;
 
@@ -836,6 +860,7 @@ typedef struct {
   gchar *none_string;  // localised text saying "None", for playback plugin name, etc.
   gchar *recommended_string;  // localised text saying "recommended", for encoder and output format
   gchar *disabled_string;  // localised text saying "disabled !", for playback plugin name, etc.
+  gchar *cl_string; // localised text saying "*The current layout*", for layout warnings
 
   gint opening_frames; // count of frames so far opened, updated after preview (currently)
 
@@ -893,6 +918,15 @@ typedef struct {
 
   gboolean soft_debug; // for testing
 
+  // encoder text output
+  GIOChannel *iochan;
+  GtkTextView *optextview;
+
+  // error signalling during decoder plugin "rip_audio"
+  pthread_t sig_pid;
+  gchar *sig_file;
+
+  gboolean go_away;
 
 } mainwindow;
 
