@@ -9,10 +9,8 @@
 #ifndef __HAS_MULTITRACK_H__
 #define __HAS_MULTITRACK_H__
 
-#include "events.h"
-
-#define CLIP_THUMB_WIDTH 80
-#define CLIP_THUMB_HEIGHT 80
+#define CLIP_THUMB_WIDTH 72
+#define CLIP_THUMB_HEIGHT 72
 
 #define CLIP_LABEL_LENGTH 20
 
@@ -23,13 +21,17 @@
 #define BLOCK_DRAW_TYPE BLOCK_DRAW_THUMB
 
 #define MT_PLAY_WIDTH_SMALL 320
-#define MT_PLAY_HEIGHT_SMALL 240
+#define MT_PLAY_HEIGHT_SMALL 220
 
 #define MT_PLAY_WIDTH_EXP 480
 #define MT_PLAY_HEIGHT_EXP 360
 
 #define MT_CTX_WIDTH 320
-#define MT_CTX_HEIGHT 240
+#define MT_CTX_HEIGHT 220
+
+#define FX_BLOCK_WIDTH 80
+#define FX_BLOCK_HEIGHT 20
+
 
 #define TIMECODE_LENGTH 14 // length of timecode text entry, must be >12
 
@@ -50,8 +52,9 @@ struct _mt_opts {
   gboolean move_effects; // should we move effects attached to a block ?
   gboolean fx_auto_preview;
   gboolean snap_over; // snap to overlap
-  gboolean snapt; // block snap
+  gshort grav_mode;
   gshort mouse_mode;
+  gshort insert_mode;
   gboolean show_audio;
   gboolean show_ctx;
   gboolean ign_ins_sel;
@@ -67,6 +70,20 @@ struct _mt_opts {
 #define MOUSE_MODE_SELECT 2
 #define MOUSE_MODE_COPY 3
 
+#define INSERT_MODE_NORMAL 1  // the default (only insert if it fits)
+
+  // not implemented yet
+#define INSERT_MODE_OVERWRITE 2 // overwite existing blocks
+#define INSERT_MODE_EXPAND 3 // repeat to fill gap
+#define INSERT_MODE_FILL_START 4 // insert enough to fill gap (from selection start)
+#define INSERT_MODE_FILL_END 5 // insert enough to fill gap (to selection end)
+
+
+#define GRAV_MODE_NORMAL 1
+#define GRAV_MODE_LEFT 2
+#define GRAV_MODE_RIGHT 3
+
+
 struct _mt {
   // widgets
   GtkWidget *window;
@@ -124,13 +141,12 @@ struct _mt {
   GtkWidget *mm_menuitem;
   GtkWidget *mm_move;
   GtkWidget *mm_select;
-  GtkWidget *snapt_menuitem;
-  GtkWidget *snapt_on;
-  GtkWidget *snapt_off;
-  GtkWidget *insa_menuitem;
-  GtkWidget *insa_menuitemsep;
-  GtkWidget *insa_on;
-  GtkWidget *insa_off;
+  GtkWidget *ins_menuitem;
+  GtkWidget *ins_normal;
+  GtkWidget *grav_menuitem;
+  GtkWidget *grav_normal;
+  GtkWidget *grav_left;
+  GtkWidget *grav_right;
   GtkWidget *select_track;
   GtkWidget *view_events;
   GtkWidget *view_clips;
@@ -184,7 +200,6 @@ struct _mt {
   GtkWidget *fx_ibefore_button;
   GtkWidget *fx_iafter_button;
   GtkWidget *cback_audio;
-  GtkWidget *save_vals;
   GtkWidget *load_vals;
   GtkWidget *change_vals;
   GtkWidget *aparam_separator;
@@ -197,21 +212,65 @@ struct _mt {
   GtkWidget *clear_marks;
   GtkWidget *fd_frame;
   GtkWidget *apply_fx_button;
-  GtkWidget *cview_button;
   GtkWidget *eview_button;
   GtkWidget *follow_play;
   GtkWidget *change_max_disp;
   GtkWidget *add_vid_behind;
   GtkWidget *add_vid_front;
   GtkWidget *quit;
+  GtkWidget *troubleshoot;
   GtkWidget *fx_params_label;
   GtkWidget *amixer_button;
   GtkWidget *view_sel_events;
+  GtkWidget *adjust_start_end;
+  GtkWidget *in_eventbox;
+  GtkWidget *out_eventbox;
+  GtkWidget *start_in_label;
+  GtkWidget *end_out_label;
+  GtkWidget *context_frame;
+  GtkWidget *nb;
+  GtkWidget *nb_label;
+
+#define NB_ERROR_SEL 1
+#define NB_ERROR_NOEFFECT 2
+#define NB_ERROR_NOTRANS 3
+#define NB_ERROR_NOCOMP 4
+#define NB_ERROR_NOCLIP 5
+
+
+  GtkWidget *open_menu;
+  GtkWidget *recent_menu;
+  GtkWidget *recent1;
+  GtkWidget *recent2;
+  GtkWidget *recent3;
+  GtkWidget *recent4;
+
+  GtkWidget *time_scrollbar;
+  GtkWidget *show_layout_errors;
+
+  GtkWidget *load_set;
+  GtkWidget *save_set;
+
+  GtkWidget *close;
+
+  GtkWidget *clear_ds;
+
+  GtkWidget *gens_submenu;
+  GtkWidget *capture;
+
+  GtkWidget *insa_eventbox;
+  GtkWidget *insa_checkbutton;
+  GtkWidget *snapo_checkbutton;
+
+
+
+
+  GtkObject *spinbutton_in_adj;
+  GtkObject *spinbutton_out_adj;
 
   GdkCursor *cursor;
 
   GtkObject *hadjustment;
-  GtkWidget *time_scrollbar;
 
   GList *audio_draws; // list of audio boxes, 0 == backing audio, 1 == track 0 audio, etc.
 
@@ -236,19 +295,21 @@ struct _mt {
 
   gulong mm_move_func;
   gulong mm_select_func;
+
+  gulong ins_normal_func;
+
+  gulong grav_normal_func;
+  gulong grav_left_func;
+  gulong grav_right_func;
   
-  gulong snapt_on_func;
-  gulong snapt_off_func;
-
-  gulong insa_on_func;
-  gulong insa_off_func;
-
   gulong sepwin_func;
   gulong mute_audio_func;
   gulong loop_cont_func;
 
   gulong seltrack_func;
 
+  gulong tc_func;
+
   weed_plant_t *event_list;
 
   weed_plant_t *init_event;   // current editable values
@@ -271,15 +332,11 @@ struct _mt {
 #define POLY_NONE 0
 #define POLY_CLIPS 1
 #define POLY_IN_OUT 2
-#define POLY_EFFECT 3
-#define POLY_FX_LIST 4
-
-
-  gshort insert_mode;
-#define INSERT_MODE_NORMAL 1  // the default
-#define INSERT_MODE_OVERWRITE 2
-#define INSERT_MODE_EXPAND 3
-  //#define INSERT_MODE_FILL 4
+#define POLY_FX_STACK 3
+#define POLY_EFFECTS 4
+#define POLY_TRANS 5
+#define POLY_COMP 6
+#define POLY_PARAMS 7
 
   gint render_file;
 
@@ -304,7 +361,7 @@ struct _mt {
   gboolean is_rendering; // TRUE if we are in the process of rendering/pre-rendering to a clip, cf. mainw->is_rendering
   gboolean pr_audio; // TRUE if we are in the process of prerendering audio to a clip
 
-#define MAX_DISP_VTRACKS 4
+#define MAX_DISP_VTRACKS 5
   gint max_disp_vtracks;
   gint current_fx;
 
@@ -315,7 +372,7 @@ struct _mt {
   gchar layout_name[256];
 
   // cursor warping for mouse move mode
-  gint hotspot_x;
+  gdouble hotspot_x;
   gint hotspot_y;
 
   gboolean moving_block; // moving block flag
@@ -340,8 +397,8 @@ struct _mt {
   GList *undos;
   size_t undo_buffer_used;
   unsigned char *undo_mem;
-  gboolean did_backup;
   gint undo_offset;
+  gboolean did_backup;
 
   gchar undo_text[32];
   gchar redo_text[32];
@@ -389,7 +446,6 @@ struct _mt {
 
   gboolean layout_prompt; // on occasion, prompt user if they want to correct layout on disk or not
   gboolean layout_set_properties;
-  gboolean save_all_vals;
   gboolean ignore_load_vals;
 
   gdouble user_fps;
@@ -414,6 +470,7 @@ struct _mt {
   weed_plant_t *specific_event; // a pointer to some generally interesting event
 
   gdouble context_time; // this is set when the user right clicks on a track, otherwise -1.
+  gboolean use_context;
 
   gint cursor_style;
 
@@ -427,6 +484,10 @@ struct _mt {
   gint play_window_width;
   gint play_window_height;
 
+  gint selected_filter; // filter selected in poly window tab
+
+  gint top_track; // top (video) track in scrolled window
+
   gboolean redraw_block; // block drawing of playback cursor during track redraws
 
   gboolean was_undo_redo;
@@ -439,7 +500,12 @@ struct _mt {
 
   gboolean has_audio_file;
 
-  guint idlefunc;
+  gboolean tl_mouse;
+
+  guint idlefunc; // autobackup function 
+
+  GList *clip_labels;
+
 };  // lives_mt
 
 
@@ -483,9 +549,8 @@ typedef struct {
 
 
 // reasons for track invisibility (bitmap)
-#define TRACK_I_OUTSCROLLED_PRE (1<<0)
-#define TRACK_I_OUTSCROLLED_POST (1<<1)
-#define TRACK_I_HIDDEN_USER (1<<2)
+#define TRACK_I_HIDDEN_USER (1<<0)
+#define TRACK_I_HIDDEN_SCROLLED (1<<1)
 
 struct _track_rect {
   // track rectangles (blocks), we translate our event_list FRAME events into these, then when exposed, the eventbox draws them
@@ -536,12 +601,12 @@ typedef struct {
 // setup functions
 gboolean on_multitrack_activate (GtkMenuItem *, weed_plant_t *); // widget callback to create the multitrack window
 lives_mt *multitrack (weed_plant_t *, gint orig_file, gdouble fps); // create and return lives_mt struct
-void init_tracks (lives_mt *, gboolean set_min_max);  // add basic tracks, or set tracks from an event_list
-void init_clips (lives_mt *, gint orig_file, gboolean add); // init clip window (or add a new clip)
+void mt_init_tracks (lives_mt *, gboolean set_min_max);  // add basic tracks, or set tracks from an event_list
+
 
 // delete functions
 gboolean on_mt_delete_event (GtkWidget *, GdkEvent *, gpointer mt);
-gboolean multitrack_delete (lives_mt *); // free the lives_mt struct
+gboolean multitrack_delete (lives_mt *, gboolean save); // free the lives_mt struct
 gboolean multitrack_end (GtkMenuItem *, gpointer mt);
 
 // morph the poly window
@@ -554,14 +619,20 @@ void mt_sensitise (lives_mt *);
 // external control callbacks
 void insert_here_cb (GtkMenuItem *, gpointer mt);
 void insert_audio_here_cb (GtkMenuItem *, gpointer mt);
+void insert_at_ctx_cb (GtkMenuItem *, gpointer mt);
+void insert_audio_at_ctx_cb (GtkMenuItem *, gpointer mt);
 void multitrack_end_cb (GtkMenuItem *, gpointer mt);
 void delete_block_cb (GtkMenuItem *, gpointer mt);
 void selblock_cb (GtkMenuItem *, gpointer mt);
 void list_fx_here_cb (GtkMenuItem *, gpointer mt);
+void edit_start_end_cb (GtkMenuItem *, gpointer mt);
+void close_clip_cb (GtkMenuItem *, gpointer mt);
+void show_clipinfo_cb (GtkMenuItem *, gpointer mt);
 
 //// menuitem callbacks
 void on_add_video_track_activate (GtkMenuItem *, gpointer mt);
 void multitrack_insert (GtkMenuItem *, gpointer mt);
+void multitrack_adj_start_end (GtkMenuItem *, gpointer mt);
 void multitrack_audio_insert (GtkMenuItem *, gpointer mt);
 void multitrack_view_events (GtkMenuItem *, gpointer mt);
 void multitrack_view_sel_events (GtkMenuItem *, gpointer mt);
@@ -616,14 +687,14 @@ GtkWidget *add_audio_track (lives_mt *, gint trackno, gboolean behind);
 void add_video_track_behind (GtkMenuItem *, gpointer mt);
 void add_video_track_front (GtkMenuItem *, gpointer mt);
 void delete_video_track(lives_mt *, gint layer, gboolean full);
-void delete_audio_track(lives_mt *mt, GtkWidget *eventbox, gboolean full);
-void delete_audio_tracks(lives_mt *mt, GList *list, gboolean full);
+void delete_audio_track(lives_mt *, GtkWidget *eventbox, gboolean full);
+void delete_audio_tracks(lives_mt *, GList *list, gboolean full);
 void remove_gaps (GtkMenuItem *, gpointer mt);
 void remove_first_gaps (GtkMenuItem *, gpointer mt);
 void on_insgap_sel_activate (GtkMenuItem *, gpointer mt);
 void on_insgap_cur_activate (GtkMenuItem *, gpointer mt);
 void on_split_activate (GtkMenuItem *, gpointer mt);
-void scroll_tracks (lives_mt *mt);
+void scroll_tracks (lives_mt *, gint top_track);
 gboolean track_arrow_pressed (GtkWidget *ahbox, GdkEventButton *, gpointer mt);
 
 // track mouse movement
@@ -653,7 +724,7 @@ void avel_reverse_toggled (GtkToggleButton *, gpointer mt);
 void avel_spin_changed (GtkSpinButton *, gpointer mt);
 
 // timeline functions
-void mt_tl_move(lives_mt *mt, gdouble pos_rel);
+void mt_tl_move(lives_mt *, gdouble pos_rel);
 void set_timeline_end_secs (lives_mt *, gdouble secs);
 gboolean on_timeline_press (GtkWidget *, GdkEventButton *, gpointer mt);
 gboolean on_timeline_release (GtkWidget *, GdkEventButton *, gpointer mt);
@@ -669,20 +740,27 @@ void re_to_tc (GtkMenuItem *, gpointer mt);
 gboolean mt_mark_callback (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer user_data);
 void multitrack_clear_marks (GtkMenuItem *, gpointer mt);
 void mt_show_current_frame(lives_mt *);  // preview th current frame (non-effects mode)
+void mt_clear_timeline(lives_mt *mt);
+
 
 // context box text
 void clear_context (lives_mt *);
 void add_context_label (lives_mt *, gchar *text);
 void mouse_mode_context(lives_mt *);
 void do_sel_context (lives_mt *);
-void do_fx_list_context (lives_mt *mt, gint fxcount);
+void do_fx_list_context (lives_mt *, gint fxcount);
+void do_fx_move_context(lives_mt *mt);
 
 // playback / animation
-void multitrack_playall (lives_mt *mt);
+void multitrack_playall (lives_mt *);
 void animate_multitrack (lives_mt *);
 void unpaint_line(lives_mt *, GtkWidget *eventbox);
 void unpaint_lines(lives_mt *);
 
+void mt_prepare_for_playback(lives_mt *);
+void mt_post_playback(lives_mt *);
+
+
 // effect node controls
 void on_next_node_clicked  (GtkWidget *, gpointer mt);
 void on_prev_node_clicked  (GtkWidget *, gpointer mt);
@@ -693,8 +771,8 @@ void on_frame_preview_clicked (GtkButton *, gpointer mt);
 void show_preview (lives_mt *, weed_timecode_t tc);
 
 // filter list controls
-weed_plant_t *get_prev_fm (lives_mt *mt, gint current_track, weed_plant_t *frame);
-weed_plant_t *get_next_fm (lives_mt *mt, gint current_track, weed_plant_t *frame);
+weed_plant_t *get_prev_fm (lives_mt *, gint current_track, weed_plant_t *frame);
+weed_plant_t *get_next_fm (lives_mt *, gint current_track, weed_plant_t *frame);
 
 void on_prev_fm_clicked  (GtkWidget *button, gpointer user_data);
 void on_next_fm_clicked  (GtkWidget *button, gpointer user_data);
@@ -702,47 +780,65 @@ void on_fx_insb_clicked  (GtkWidget *button, gpointer user_data);
 void on_fx_insa_clicked  (GtkWidget *button, gpointer user_data);
 
 // utils
-guint event_list_get_byte_size(weed_plant_t *event_list, int *num_events);  // returns bytes and sets num_events
-gboolean event_list_rectify(lives_mt *, weed_plant_t *event_list, gboolean check_clips);
+guint event_list_get_byte_size(lives_mt *mt, weed_plant_t *event_list, int *num_events);  // returns bytes and sets num_events
+gboolean event_list_rectify(lives_mt *, weed_plant_t *event_listy);
 gboolean make_backup_space (lives_mt *, size_t space_needed);
-void activate_mt_preview(lives_mt *mt); // sensitize Show Preview and Apply buttons
+void activate_mt_preview(lives_mt *); // sensitize Show Preview and Apply buttons
 void **mt_get_pchain(void);
+void event_list_free_undos(lives_mt *);
+gboolean used_in_current_layout(lives_mt *, gint file);
 
 // event_list utilities
 gboolean compare_filter_maps(weed_plant_t *fm1, weed_plant_t *fm2, gint ctrack); // ctrack can be -1 to compare all events, else we cf for ctrack
-void move_init_in_filter_map(weed_plant_t *event_list, weed_plant_t *fmap, weed_plant_t *ifrom, weed_plant_t *ito, gint track, gboolean after);
+void move_init_in_filter_map(lives_mt *mt, weed_plant_t *event_list, weed_plant_t *fmap, weed_plant_t *ifrom, weed_plant_t *ito, gint track, gboolean after);
 void update_filter_events(lives_mt *, weed_plant_t *first_event, weed_timecode_t start_tc, weed_timecode_t end_tc, int track, weed_timecode_t new_start_tc, int new_track);
-void mt_fixup_events(lives_mt *mt, weed_plant_t *old_event, weed_plant_t *new_event);
+void mt_fixup_events(lives_mt *, weed_plant_t *old_event, weed_plant_t *new_event);
 
 // event_list load/save
 weed_plant_t *load_event_list(lives_mt *, gchar *eload_file);
 
 
-// layout maps
+// layouts and layout maps
 GList *load_layout_map(void);
-void save_layout_map (int *lmap, double *lmap_audio, gchar *file, gchar *dir);
-void migrate_layouts (gchar *old_set_name, gchar *new_set_name);
-gboolean layout_frame_is_affected(gint clipno, gint frame);
-gboolean layout_audio_is_affected(gint clipno, gdouble time);
 void add_markers(lives_mt *, weed_plant_t *event_list);
 void remove_markers(weed_plant_t *event_list);
+void save_layout_map (int *lmap, double *lmap_audio, gchar *file, gchar *dir);
+
+void migrate_layouts (gchar *old_set_name, gchar *new_set_name);
+
+GList *layout_frame_is_affected(gint clipno, gint frame);
+GList *layout_audio_is_affected(gint clipno, gdouble time);
+
+gboolean check_for_layout_del (lives_mt *, gboolean exiting);
+
+void stored_event_list_free_all(gboolean wiped);
+void stored_event_list_free_undos(void);
+void remove_current_from_affected_layouts(lives_mt *);
 
 
 // auto backup
+guint mt_idle_add(lives_mt *);
 void recover_layout(GtkButton *, gpointer);
 void recover_layout_cancelled(GtkButton *, gpointer user_data);
+void write_backup_layout_numbering(lives_mt *);
 
 
 // internal functions
-void mouse_select_end(GtkWidget *widget, lives_mt *mt);
+void mouse_select_end(GtkWidget *, lives_mt *);
 
 
 // misc
 void mt_change_disp_tracks_ok (GtkButton *, gpointer user_data);
-void mt_swap_play_pause (lives_mt *mt, gboolean put_pause);
-void amixer_show (GtkButton *button, gpointer user_data);
+void mt_swap_play_pause (lives_mt *, gboolean put_pause);
+void amixer_show (GtkButton *, gpointer user_data);
+gchar *set_values_from_defs(lives_mt *, gboolean from_prefs);
 
 
+// clip boxes
+void mt_clip_select (lives_mt *, gboolean scroll);
+void mt_delete_clips(lives_mt *, gint file);
+void mt_init_clips (lives_mt *, gint orig_file, gboolean add);
+
 /* default to warn about */
 #define LMAP_ERROR_MISSING_CLIP 1
 #define LMAP_ERROR_CLOSE_FILE 2
