@@ -1,6 +1,6 @@
 // osc.c
 // LiVES (lives-exe)
-// (c) G. Finch 2004 - 2009
+// (c) G. Finch 2004 - 2010
 // Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
@@ -31,6 +31,7 @@ static lives_osc *livesOSC=NULL;
 #define FX_MAX FX_KEYS_MAX_VIRTUAL-1
 
 
+
 /* convert a big endian 32 bit string to an int for internal use */
 
 static int toInt(const char* b) {
@@ -139,7 +140,8 @@ gboolean lives_status_send (gchar *msg) {
 gboolean lives_osc_notify (int msgnumber,gchar *msgstring) {
   if (notify_socket==NULL) return FALSE;
   else {
-    gchar *msg=g_strdup_printf("%d %s",msgnumber,msgstring);
+    gchar *sep=((msgstring==NULL||strlen(msgstring)==0)?"":" ");
+    gchar *msg=g_strdup_printf("%d%s%s\n",msgnumber,sep,msgstring);
     gboolean retval = lives_stream_out (notify_socket,strlen (msg),msg);
     g_free(msg);
     return retval;
@@ -165,7 +167,8 @@ void lives_osc_close_notify_socket (void) {
 
 /* /video/play */
 void lives_osc_cb_play (void *context, int arglen, const void *vargs, OSCTimeTag when, NetworkReturnAddressPtr ra) {
-
+  if (mainw->go_away) return;
+  
   if (mainw->playing_file==-1&&mainw->current_file>0) on_playall_activate(NULL,NULL);
 
 }
@@ -190,6 +193,7 @@ void lives_osc_cb_bgplay_reverse(void *context, int arglen, const void *vargs, O
 
 
 void lives_osc_cb_play_forward (void *context, int arglen, const void *vargs, OSCTimeTag when, NetworkReturnAddressPtr ra) {
+  if (mainw->go_away) return;
 
   if (mainw->current_file<0||(cfile->clip_type!=CLIP_TYPE_DISK&&cfile->clip_type!=CLIP_TYPE_FILE)) return;
 
@@ -205,6 +209,7 @@ void lives_osc_cb_play_forward (void *context, int arglen, const void *vargs, OS
 
 
 void lives_osc_cb_play_backward (void *context, int arglen, const void *vargs, OSCTimeTag when, NetworkReturnAddressPtr ra) {
+  if (mainw->go_away) return;
 
   if (mainw->current_file<0||(cfile->clip_type!=CLIP_TYPE_DISK&&cfile->clip_type!=CLIP_TYPE_FILE)) return;
 
@@ -480,26 +485,27 @@ void lives_osc_cb_fgclip_select(void *context, int arglen, const void *vargs, OS
   if (!lives_osc_check_arguments (arglen,vargs,"i",TRUE)) return;
   lives_osc_parse_int_argument(vargs,&clip);
 
-  if (clip<1) return;
+  if (clip<1||mainw->cliplist==NULL) return;
 
-  for (i=1;i<MAX_FILES;i++) {
-    if (mainw->files[i]!=NULL&&(mainw->files[i]->clip_type==CLIP_TYPE_DISK||mainw->files[i]->clip_type==CLIP_TYPE_FILE)) {
-      if (--clip==0) {
-	if (i==mainw->current_file) break;
-	if (mainw->playing_file>0) {
-	  mainw->pre_src_file=i;
-	  mainw->new_clip=i;
-	}
-	else if (mainw->playing_file==-1) {
-	  switch_to_file(mainw->current_file,i);
-	}
-      }
-    }
+  if (mainw->scrap_file!=-1&&clip>=mainw->scrap_file) clip++;
+
+  if (clip>g_list_length(mainw->cliplist)) return;
+
+  i=GPOINTER_TO_INT(g_list_nth_data(mainw->cliplist,clip-1));
+    
+  if (i==mainw->current_file) return;
+  if (mainw->playing_file>0) {
+    mainw->pre_src_file=i;
+    mainw->new_clip=i;
+  }
+  else if (mainw->playing_file==-1) {
+    switch_to_file(mainw->current_file,i);
   }
 }
 
 
 
+
 void lives_osc_cb_bgclip_select(void *context, int arglen, const void *vargs, OSCTimeTag when,	NetworkReturnAddressPtr ra) {
   // switch bg clip
   int clip,i;
@@ -508,22 +514,378 @@ void lives_osc_cb_bgclip_select(void *context, int arglen, const void *vargs, OS
   if (!lives_osc_check_arguments (arglen,vargs,"i",TRUE)) return;
   lives_osc_parse_int_argument(vargs,&clip);
 
-  if (clip<1) return;
+  if (clip<1||mainw->cliplist==NULL) return;
+
+  if (mainw->scrap_file!=-1&&clip>=mainw->scrap_file) clip++;
+
+  if (clip>g_list_length(mainw->cliplist)) return;
 
   if (mainw->num_tr_applied<1) return;
 
-  for (i=1;i<MAX_FILES;i++) {
-    if (mainw->files[i]!=NULL&&(mainw->files[i]->clip_type==CLIP_TYPE_DISK||mainw->files[i]->clip_type==CLIP_TYPE_FILE)) {
-      if (--clip==0) {
-	if (i==mainw->blend_file) break;
-	mainw->blend_file=i;
-      }
-    }
-  }
+  i=GPOINTER_TO_INT(g_list_nth_data(mainw->cliplist,clip-1));
+    
+  if (i==mainw->blend_file) return;
+
+  mainw->blend_file=i;
+
 }
 
 
 
+
+void lives_osc_cb_fgclip_close(void *context, int arglen, const void *vargs, OSCTimeTag when,	NetworkReturnAddressPtr ra) {
+  int noaudio=0;
+  int clipno=mainw->current_file;
+  gint current_file=clipno;
+
+  if (mainw->playing_file>-1) return;
+
+  if (mainw->current_file<1||mainw->preview||mainw->is_processing) return;
+
+  if (lives_osc_check_arguments (arglen,vargs,"i",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"i",TRUE);
+    lives_osc_parse_int_argument(vargs,&noaudio);
+  }
+  else if (!lives_osc_check_arguments (arglen,vargs,"",TRUE)) {
+    return;
+  }
+
+  if (clipno<1||clipno>MAX_FILES||mainw->files[clipno]==NULL) return;
+
+  if (clipno==current_file) current_file=-1;
+
+  mainw->current_file=clipno;
+
+  close_current_file(current_file);
+
+}
+
+
+
+
+
+void lives_osc_cb_fgclip_copy(void *context, int arglen, const void *vargs, OSCTimeTag when,	NetworkReturnAddressPtr ra) {
+  int noaudio=0;
+  int clipno=mainw->current_file;
+  gint start,end,current_file=clipno;
+  gboolean ccpd;
+
+  if (mainw->playing_file>-1) return;
+
+  if (mainw->current_file<1||mainw->preview||mainw->is_processing) return;
+
+  if (lives_osc_check_arguments (arglen,vargs,"ii",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"ii",TRUE);
+    lives_osc_parse_int_argument(vargs,&noaudio);
+    lives_osc_parse_int_argument(vargs,&clipno);
+  }
+  else if (lives_osc_check_arguments (arglen,vargs,"i",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"i",TRUE);
+    lives_osc_parse_int_argument(vargs,&noaudio);
+  }
+  else if (!lives_osc_check_arguments (arglen,vargs,"",TRUE)) {
+    return;
+  }
+
+  if (clipno<1||clipno>MAX_FILES||mainw->files[clipno]==NULL||(mainw->files[clipno]->clip_type!=CLIP_TYPE_DISK&&mainw->files[clipno]->clip_type!=CLIP_TYPE_FILE)) return;
+
+  mainw->current_file=clipno;
+  start=cfile->start;
+  end=cfile->end;
+
+  cfile->start=1;
+  cfile->end=cfile->frames;
+
+  ccpd=mainw->ccpd_with_sound;
+
+  mainw->ccpd_with_sound=!noaudio;
+
+  on_copy_activate(NULL,NULL);
+
+  mainw->ccpd_with_sound=ccpd;
+
+  cfile->start=start;
+  cfile->end=end;
+
+  mainw->current_file=current_file;
+
+}
+
+
+
+void lives_osc_cb_fgclipsel_copy(void *context, int arglen, const void *vargs, OSCTimeTag when,	NetworkReturnAddressPtr ra) {
+
+  int noaudio=0;
+  int clipno=mainw->current_file;
+  gint current_file=clipno;
+  gboolean ccpd;
+
+  if (mainw->playing_file>-1) return;
+
+  if (mainw->current_file<1||mainw->preview||mainw->is_processing) return;
+
+  if (lives_osc_check_arguments (arglen,vargs,"ii",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"ii",TRUE);
+    lives_osc_parse_int_argument(vargs,&noaudio);
+    lives_osc_parse_int_argument(vargs,&clipno);
+  }
+  else if (lives_osc_check_arguments (arglen,vargs,"i",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"i",TRUE);
+    lives_osc_parse_int_argument(vargs,&noaudio);
+  }
+  else if (!lives_osc_check_arguments (arglen,vargs,"",TRUE)) {
+    return;
+  }
+
+  if (clipno<1||clipno>MAX_FILES||mainw->files[clipno]==NULL||(mainw->files[clipno]->clip_type!=CLIP_TYPE_DISK&&mainw->files[clipno]->clip_type!=CLIP_TYPE_FILE)) return;
+
+  mainw->current_file=clipno;
+
+  ccpd=mainw->ccpd_with_sound;
+
+  mainw->ccpd_with_sound=!noaudio;
+
+  on_copy_activate(NULL,NULL);
+
+  mainw->ccpd_with_sound=ccpd;
+
+  mainw->current_file=current_file;
+
+}
+
+
+
+
+
+void lives_osc_cb_fgclipsel_cut(void *context, int arglen, const void *vargs, OSCTimeTag when,	NetworkReturnAddressPtr ra) {
+
+  int noaudio=0;
+  int clipno=mainw->current_file;
+  gint current_file=clipno;
+  gboolean ccpd;
+
+  if (mainw->playing_file>-1) return;
+
+  if (mainw->current_file<1||mainw->preview||mainw->is_processing) return;
+
+  if (lives_osc_check_arguments (arglen,vargs,"ii",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"ii",TRUE);
+    lives_osc_parse_int_argument(vargs,&noaudio);
+    lives_osc_parse_int_argument(vargs,&clipno);
+  }
+  else if (lives_osc_check_arguments (arglen,vargs,"i",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"i",TRUE);
+    lives_osc_parse_int_argument(vargs,&noaudio);
+  }
+  else if (!lives_osc_check_arguments (arglen,vargs,"",TRUE)) {
+    return;
+  }
+
+  if (clipno<1||clipno>MAX_FILES||mainw->files[clipno]==NULL||(mainw->files[clipno]->clip_type!=CLIP_TYPE_DISK&&mainw->files[clipno]->clip_type!=CLIP_TYPE_FILE)) return;
+
+  mainw->current_file=clipno;
+
+  ccpd=mainw->ccpd_with_sound;
+
+  mainw->ccpd_with_sound=!noaudio;
+
+  on_cut_activate(GINT_TO_POINTER(1),NULL);
+
+  mainw->ccpd_with_sound=ccpd;
+
+  mainw->current_file=current_file;
+
+}
+
+
+
+
+void lives_osc_cb_fgclipsel_delete(void *context, int arglen, const void *vargs, OSCTimeTag when,	NetworkReturnAddressPtr ra) {
+
+  int noaudio=0;
+  int clipno=mainw->current_file;
+  gint current_file=clipno;
+  gboolean ccpd;
+
+  if (mainw->playing_file>-1) return;
+
+  if (mainw->current_file<1||mainw->preview||mainw->is_processing) return;
+
+  if (lives_osc_check_arguments (arglen,vargs,"ii",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"ii",TRUE);
+    lives_osc_parse_int_argument(vargs,&noaudio);
+    lives_osc_parse_int_argument(vargs,&clipno);
+  }
+  else if (lives_osc_check_arguments (arglen,vargs,"i",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"i",TRUE);
+    lives_osc_parse_int_argument(vargs,&noaudio);
+  }
+  else if (!lives_osc_check_arguments (arglen,vargs,"",TRUE)) {
+    return;
+  }
+
+  if (clipno<1||clipno>MAX_FILES||mainw->files[clipno]==NULL||(mainw->files[clipno]->clip_type!=CLIP_TYPE_DISK&&mainw->files[clipno]->clip_type!=CLIP_TYPE_FILE)) return;
+
+  mainw->current_file=clipno;
+
+  ccpd=mainw->ccpd_with_sound;
+
+  mainw->ccpd_with_sound=!noaudio;
+
+  on_delete_activate(GINT_TO_POINTER(1),NULL);
+
+  mainw->ccpd_with_sound=ccpd;
+
+  mainw->current_file=current_file;
+
+}
+
+
+
+
+void lives_osc_cb_clipbd_paste(void *context, int arglen, const void *vargs, OSCTimeTag when,	NetworkReturnAddressPtr ra) {
+  int noaudio=0;
+  gboolean ccpd;
+
+  if (mainw->playing_file>-1) return;
+
+  if (clipboard==NULL) return;
+
+  if (mainw->current_file<1||mainw->preview||mainw->is_processing) return;
+
+  if (lives_osc_check_arguments (arglen,vargs,"i",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"i",TRUE);
+    lives_osc_parse_int_argument(vargs,&noaudio);
+  }
+  else if (!lives_osc_check_arguments (arglen,vargs,"",TRUE)) {
+    return;
+  }
+
+  ccpd=mainw->ccpd_with_sound;
+
+  mainw->ccpd_with_sound=!noaudio;
+
+  on_paste_as_new_activate(NULL,NULL);
+
+  mainw->ccpd_with_sound=ccpd;
+
+}
+
+
+
+
+void lives_osc_cb_clipbd_insertb(void *context, int arglen, const void *vargs, OSCTimeTag when,	NetworkReturnAddressPtr ra) {
+
+  int noaudio=0;
+  int times=1;
+  int clipno=mainw->current_file;
+  gint current_file=clipno;
+
+
+  if (mainw->playing_file>-1) return;
+
+  if (mainw->current_file<1||mainw->preview||mainw->is_processing) return;
+
+  if (lives_osc_check_arguments (arglen,vargs,"iii",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"iii",TRUE);
+    lives_osc_parse_int_argument(vargs,&noaudio);
+    lives_osc_parse_int_argument(vargs,&times);
+    lives_osc_parse_int_argument(vargs,&clipno);
+  }
+  else if (lives_osc_check_arguments (arglen,vargs,"ii",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"ii",TRUE);
+    lives_osc_parse_int_argument(vargs,&noaudio);
+    lives_osc_parse_int_argument(vargs,&times);
+  }
+  else if (lives_osc_check_arguments (arglen,vargs,"i",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"i",TRUE);
+    lives_osc_parse_int_argument(vargs,&noaudio);
+  }
+  else if (!lives_osc_check_arguments (arglen,vargs,"",TRUE)) {
+    return;
+  }
+
+  if (clipno<1||clipno>MAX_FILES||mainw->files[clipno]==NULL||(mainw->files[clipno]->clip_type!=CLIP_TYPE_DISK&&mainw->files[clipno]->clip_type!=CLIP_TYPE_FILE)) return;
+
+  if (times==0||times<-1) return;
+
+  mainw->current_file=clipno;
+
+  mainw->insert_after=FALSE;
+
+  if (clipboard->achans==0&&cfile->achans==0) noaudio=TRUE;
+
+  mainw->fx1_bool=(times==-1); // fit to audio
+  mainw->fx1_val=times;       // times to insert otherwise
+  mainw->fx2_bool=!noaudio;  // with audio
+
+  on_insert_activate(NULL,NULL);
+
+  mainw->current_file=current_file;
+
+}
+
+
+
+
+
+void lives_osc_cb_clipbd_inserta(void *context, int arglen, const void *vargs, OSCTimeTag when,	NetworkReturnAddressPtr ra) {
+
+  int noaudio=0;
+  int times=1;
+  int clipno=mainw->current_file;
+  gint current_file=clipno;
+
+
+  if (mainw->playing_file>-1) return;
+
+  if (mainw->current_file<1||mainw->preview||mainw->is_processing) return;
+
+  if (lives_osc_check_arguments (arglen,vargs,"iii",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"iii",TRUE);
+    lives_osc_parse_int_argument(vargs,&noaudio);
+    lives_osc_parse_int_argument(vargs,&times);
+    lives_osc_parse_int_argument(vargs,&clipno);
+  }
+  else if (lives_osc_check_arguments (arglen,vargs,"ii",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"ii",TRUE);
+    lives_osc_parse_int_argument(vargs,&noaudio);
+    lives_osc_parse_int_argument(vargs,&times);
+  }
+  else if (lives_osc_check_arguments (arglen,vargs,"i",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"i",TRUE);
+    lives_osc_parse_int_argument(vargs,&noaudio);
+  }
+  else if (!lives_osc_check_arguments (arglen,vargs,"",TRUE)) {
+    return;
+  }
+
+  if (clipno<1||clipno>MAX_FILES||mainw->files[clipno]==NULL||(mainw->files[clipno]->clip_type!=CLIP_TYPE_DISK&&mainw->files[clipno]->clip_type!=CLIP_TYPE_FILE)) return;
+
+  if (times==0||times<-1) return;
+
+  mainw->current_file=clipno;
+
+  mainw->insert_after=TRUE;
+
+  if (clipboard->achans==0&&cfile->achans==0) noaudio=TRUE;
+
+  mainw->fx1_bool=(times==-1); // fit to audio
+  mainw->fx1_val=times;       // times to insert otherwise
+  mainw->fx2_bool=!noaudio;  // with audio
+
+  mainw->fx1_start=1;
+  mainw->fx2_start=count_resampled_frames(clipboard->frames,clipboard->fps,cfile->fps);
+
+  on_insert_activate(NULL,NULL);
+
+  mainw->current_file=current_file;
+
+}
+
+
+
+
 void lives_osc_cb_fgclip_retrigger (void *context, int arglen, const void *vargs, OSCTimeTag when, NetworkReturnAddressPtr ra) {
   // switch fg clip and reset framenumber
 
@@ -534,6 +896,8 @@ void lives_osc_cb_fgclip_retrigger (void *context, int arglen, const void *vargs
 
   if (cfile->pb_fps>0.||(cfile->play_paused&&cfile->freeze_fps>0.)) cfile->frameno=cfile->last_frameno=1;
   else cfile->frameno=cfile->last_frameno=cfile->frames;
+
+  resync_audio(cfile->frameno);
 }
 
 
@@ -555,8 +919,7 @@ void lives_osc_cb_bgclip_retrigger (void *context, int arglen, const void *vargs
 
 
 
-void lives_osc_cb_fgclip_select_next(void *context, int arglen, const void *vargs, OSCTimeTag when,
-	NetworkReturnAddressPtr ra) {
+void lives_osc_cb_fgclip_select_next(void *context, int arglen, const void *vargs, OSCTimeTag when, NetworkReturnAddressPtr ra) {
   // switch fg clip
 
   if (mainw->current_file<1||mainw->preview||mainw->is_processing) return; // TODO
@@ -566,8 +929,7 @@ void lives_osc_cb_fgclip_select_next(void *context, int arglen, const void *varg
 }
 
 
-void lives_osc_cb_bgclip_select_next(void *context, int arglen, const void *vargs, OSCTimeTag when,
-	NetworkReturnAddressPtr ra) {
+void lives_osc_cb_bgclip_select_next(void *context, int arglen, const void *vargs, OSCTimeTag when, NetworkReturnAddressPtr ra) {
   // switch bg clip
 
   if (mainw->current_file<1||mainw->preview||mainw->is_processing) return; // TODO
@@ -579,8 +941,7 @@ void lives_osc_cb_bgclip_select_next(void *context, int arglen, const void *varg
 
 
 
-void lives_osc_cb_fgclip_select_previous(void *context, int arglen, const void *vargs, OSCTimeTag when,
-	NetworkReturnAddressPtr ra) {
+void lives_osc_cb_fgclip_select_previous(void *context, int arglen, const void *vargs, OSCTimeTag when, NetworkReturnAddressPtr ra) {
   // switch fg clip
 
   if (mainw->current_file<1||mainw->preview||mainw->is_processing) return; // TODO
@@ -590,8 +951,7 @@ void lives_osc_cb_fgclip_select_previous(void *context, int arglen, const void *
 }
 
 
-void lives_osc_cb_bgclip_select_previous(void *context, int arglen, const void *vargs, OSCTimeTag when,
-	NetworkReturnAddressPtr ra) {
+void lives_osc_cb_bgclip_select_previous(void *context, int arglen, const void *vargs, OSCTimeTag when, NetworkReturnAddressPtr ra) {
   // switch bg clip
 
   if (mainw->current_file<1||mainw->preview||mainw->is_processing) return; // TODO
@@ -610,7 +970,15 @@ void lives_osc_cb_bgclip_select_previous(void *context, int arglen, const void *
 
 void lives_osc_cb_quit(void *context, int arglen, const void *vargs, OSCTimeTag when, NetworkReturnAddressPtr ra) {
 
-  if (mainw->was_set) mainw->leave_files=TRUE;
+  if (mainw->playing_file>-1) return;
+
+  mainw->only_close=mainw->no_exit=FALSE;
+  mainw->leave_recovery=FALSE;
+
+  if (mainw->was_set) {
+    on_save_set_activate(NULL,mainw->set_name);
+  }
+  else mainw->leave_files=FALSE;
   lives_exit();
 
 }
@@ -686,6 +1054,7 @@ void lives_osc_cb_clip_goto(void *context, int arglen, const void *vargs, OSCTim
 
   cfile->last_frameno=cfile->frameno=frame;
 
+  resync_audio(frame);
 }
 
 
@@ -728,28 +1097,68 @@ void lives_osc_cb_bgclip_get_current(void *context, int arglen, const void *varg
 
 
 void lives_osc_cb_clip_set_start(void *context, int arglen, const void *vargs, OSCTimeTag when, NetworkReturnAddressPtr ra) {
-
+  int current_file=mainw->current_file;
+  int clip=current_file;
   int frame;
+
+  file *sfile;
+
   if (mainw->current_file<1||mainw->preview||mainw->is_processing) return;
 
-  if (!lives_osc_check_arguments (arglen,vargs,"i",TRUE)) return;
+  if (lives_osc_check_arguments (arglen,vargs,"ii",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"ii",TRUE);
+    lives_osc_parse_int_argument(vargs,&frame);
+    lives_osc_parse_int_argument(vargs,&clip);
+  }
+  else if (lives_osc_check_arguments (arglen,vargs,"i",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"i",TRUE);
+    lives_osc_parse_int_argument(vargs,&frame);
+  }
+  else return;
 
-  lives_osc_parse_int_argument(vargs,&frame);
-  if (frame<1||frame>cfile->frames||(cfile->clip_type!=CLIP_TYPE_DISK&&cfile->clip_type!=CLIP_TYPE_FILE)) return;
+  if (clip<1||clip>MAX_FILES||mainw->files[clip]==NULL) return;
 
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),frame);
+  sfile=mainw->files[clip];
+
+  if (frame<1||(sfile->clip_type!=CLIP_TYPE_DISK&&sfile->clip_type!=CLIP_TYPE_FILE)) return;
+
+  if (frame>sfile->frames) frame=sfile->frames;
+
+  if (clip==mainw->current_file) gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),frame);
+  else sfile->start=frame;
 }
 
 void lives_osc_cb_clip_set_end(void *context, int arglen, const void *vargs, OSCTimeTag when, NetworkReturnAddressPtr ra) {
-
+  int current_file=mainw->current_file;
+  int clip=current_file;
   int frame;
+
+  file *sfile;
+
   if (mainw->current_file<1||mainw->preview||mainw->is_processing) return;
 
-  lives_osc_check_arguments (arglen,vargs,"i",TRUE);
-  lives_osc_parse_int_argument(vargs,&frame);
-  if (frame<1||frame>cfile->frames||(cfile->clip_type!=CLIP_TYPE_DISK&&cfile->clip_type!=CLIP_TYPE_FILE)) return;
+  if (lives_osc_check_arguments (arglen,vargs,"ii",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"ii",TRUE);
+    lives_osc_parse_int_argument(vargs,&frame);
+    lives_osc_parse_int_argument(vargs,&clip);
+  }
+  else if (lives_osc_check_arguments (arglen,vargs,"i",FALSE)) { 
+    lives_osc_check_arguments (arglen,vargs,"i",TRUE);
+    lives_osc_parse_int_argument(vargs,&frame);
+  }
+  else return;
+
+  if (clip<1||clip>MAX_FILES||mainw->files[clip]==NULL) return;
+
+  sfile=mainw->files[clip];
+
+  if (frame<1||(sfile->clip_type!=CLIP_TYPE_DISK&&sfile->clip_type!=CLIP_TYPE_FILE)) return;
+
+  if (frame>sfile->frames) frame=sfile->frames;
+
+  if (clip==mainw->current_file) gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),frame);
+  else sfile->end=frame;
 
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),frame);
 }
 
 
@@ -960,10 +1369,23 @@ static void setfx (gint effect_key, gint pnum, int arglen, const void *vargs) {
   
   switch (hint) {
   case WEED_HINT_INTEGER:
-    if (!lives_osc_check_arguments (arglen,vargs,"iii",TRUE)) return;
-    lives_osc_parse_int_argument(vargs,&valuei);
-    lives_osc_parse_int_argument(vargs,&valuei);
-    lives_osc_parse_int_argument(vargs,&valuei);
+    if (!lives_osc_check_arguments (arglen,vargs,"iii",FALSE)) {
+      if (!lives_osc_check_arguments (arglen,vargs,"iif",FALSE)) return;
+      // we wanted an int but we got a float
+      //so we will round to the nearest value
+      lives_osc_check_arguments (arglen,vargs,"iif",TRUE);
+      lives_osc_parse_int_argument(vargs,&valuei);
+      lives_osc_parse_int_argument(vargs,&valuei);
+      lives_osc_parse_float_argument(vargs,&valuef);
+      valuei=myround(valuef);
+
+    }
+    else {
+      lives_osc_check_arguments (arglen,vargs,"iii",TRUE);
+      lives_osc_parse_int_argument(vargs,&valuei);
+      lives_osc_parse_int_argument(vargs,&valuei);
+      lives_osc_parse_int_argument(vargs,&valuei);
+    }
 
     if (mainw->record&&!mainw->record_paused&&mainw->playing_file>-1&&(prefs->rec_opts&REC_EFFECTS)) {
       // if we are recording, add this change to our event_list
@@ -982,10 +1404,21 @@ static void setfx (gint effect_key, gint pnum, int arglen, const void *vargs) {
     break;
     
   case WEED_HINT_FLOAT:
-    if (!lives_osc_check_arguments (arglen,vargs,"iif",TRUE)) return;
-    lives_osc_parse_int_argument(vargs,&valuei);
-    lives_osc_parse_int_argument(vargs,&valuei);
-    lives_osc_parse_float_argument(vargs,&valuef);
+    if (!lives_osc_check_arguments (arglen,vargs,"iif",FALSE)) {
+      if (!lives_osc_check_arguments (arglen,vargs,"iii",FALSE)) return;
+      // we wanted a float but we got an int, we can convert
+      lives_osc_check_arguments (arglen,vargs,"iii",TRUE);
+      lives_osc_parse_int_argument(vargs,&valuei);
+      lives_osc_parse_int_argument(vargs,&valuei);
+      lives_osc_parse_int_argument(vargs,&valuei);
+      valuef=(gdouble)valuei;
+    }
+    else {
+      lives_osc_check_arguments (arglen,vargs,"iif",TRUE);
+      lives_osc_parse_int_argument(vargs,&valuei);
+      lives_osc_parse_int_argument(vargs,&valuei);
+      lives_osc_parse_float_argument(vargs,&valuef);
+    }
 
     if (mainw->record&&!mainw->record_paused&&mainw->playing_file>-1&&(prefs->rec_opts&REC_EFFECTS)) {
       // if we are recording, add this change to our event_list
@@ -1006,26 +1439,40 @@ static void setfx (gint effect_key, gint pnum, int arglen, const void *vargs) {
     switch (cspace) {
     case WEED_COLORSPACE_RGB:
       if (weed_leaf_seed_type(tparamtmpl,"default")==WEED_SEED_INT) {
-	if (!lives_osc_check_arguments (arglen,vargs,"iiiii",TRUE)) return;
-	lives_osc_parse_int_argument(vargs,&valuei);
-	lives_osc_parse_int_argument(vargs,&valuei);
-	lives_osc_parse_int_argument(vargs,&valuesi[0]);
-	lives_osc_parse_int_argument(vargs,&valuesi[1]);
-	lives_osc_parse_int_argument(vargs,&valuesi[2]);
-
+	if (!lives_osc_check_arguments (arglen,vargs,"iiiii",FALSE)) {
+	  if (!lives_osc_check_arguments (arglen,vargs,"iifff",FALSE)) return;
+	  // we wanted ints but we got floats
+	  lives_osc_check_arguments (arglen,vargs,"iifff",TRUE);
+	  lives_osc_parse_int_argument(vargs,&valuei);
+	  lives_osc_parse_int_argument(vargs,&valuei);
+	  lives_osc_parse_float_argument(vargs,&valuef);
+	  valuesi[0]=myround(valuef);
+	  lives_osc_parse_float_argument(vargs,&valuef);
+	  valuesi[1]=myround(valuef);
+	  lives_osc_parse_float_argument(vargs,&valuef);
+	  valuesi[2]=myround(valuef);
+	}
+	else { 
+	  lives_osc_check_arguments (arglen,vargs,"iiiii",TRUE);
+	  lives_osc_parse_int_argument(vargs,&valuei);
+	  lives_osc_parse_int_argument(vargs,&valuei);
+	  lives_osc_parse_int_argument(vargs,&valuesi[0]);
+	  lives_osc_parse_int_argument(vargs,&valuesi[1]);
+	  lives_osc_parse_int_argument(vargs,&valuesi[2]);
+	} 
 	if (mainw->record&&!mainw->record_paused&&mainw->playing_file>-1&&(prefs->rec_opts&REC_EFFECTS)) {
 	  // if we are recording, add this change to our event_list
 	  rec_param_change(inst,pnum);
 	}
-
-
+	
+	
 	weed_set_int_array(tparam,"value",3,valuesi);
-
+	
 	if (mainw->record&&!mainw->record_paused&&mainw->playing_file>-1&&(prefs->rec_opts&REC_EFFECTS)) {
 	  // if we are recording, add this change to our event_list
 	  rec_param_change(inst,pnum);
 	}
-
+	
       }
       else {
 	if (!lives_osc_check_arguments (arglen,vargs,"iifff",TRUE)) return;
@@ -1140,6 +1587,294 @@ void lives_osc_cb_rte_setnparam(void *context, int arglen, const void *vargs, OS
 
 
 
+void lives_osc_cb_rte_nparamcount(void *context, int arglen, const void *vargs, OSCTimeTag when, NetworkReturnAddressPtr ra) {
+
+  int effect_key;
+  int pnum,i;
+
+  // return number of numeric single valued, non-reinit
+  // i.e. simple numeric parameters
+
+  int error;
+  weed_plant_t *filter;
+  weed_plant_t **in_ptmpls;
+  weed_plant_t *ptmpl;
+  int hint,flags;
+  int nparams;
+
+  int count=0;
+  gchar *msg;
+  
+  if (!lives_osc_check_arguments (arglen,vargs,"ii",TRUE)) return;
+
+  lives_osc_parse_int_argument(vargs,&effect_key);
+  lives_osc_parse_int_argument(vargs,&pnum);
+
+  if (effect_key<1||effect_key>FX_MAX) return;
+  //g_print("key %d pnum %d\n",effect_key,pnum);
+
+  filter=rte_keymode_get_filter(effect_key,rte_key_getmode(effect_key));
+
+  if (!weed_plant_has_leaf(filter,"in_parameter_templates")) return;
+  nparams=weed_leaf_num_elements(filter,"in_parameter_templates");
+  if (pnum>=nparams) return;
+  
+  in_ptmpls=weed_get_plantptr_array(filter,"in_parameter_templates",&error);
+  
+  for (i=0;i<nparams;i++) {
+    ptmpl=in_ptmpls[i];
+    hint=weed_get_int_value(ptmpl,"hint",&error);
+    flags=weed_get_int_value(ptmpl,"flags",&error);
+    if ((hint==WEED_HINT_INTEGER||hint==WEED_HINT_FLOAT)&&flags==0&&weed_leaf_num_elements(ptmpl,"default")==1) {
+      count++;
+    }
+  }
+  msg=g_strdup_printf("%d\n",count);
+  lives_status_send(msg);
+  g_free(msg);
+}
+
+
+
+void lives_osc_cb_rte_getnparammin(void *context, int arglen, const void *vargs, OSCTimeTag when, NetworkReturnAddressPtr ra) {
+
+  int effect_key;
+  int pnum,i;
+
+  // pick pnum which is numeric single valued, non-reinit
+  // i.e. simple numeric parameter
+
+  int error;
+  weed_plant_t *filter;
+  weed_plant_t **in_ptmpls;
+  weed_plant_t *ptmpl;
+  int hint,flags;
+  int nparams;
+
+  int vali;
+  double vald;
+
+  gchar *msg;
+  
+  if (!lives_osc_check_arguments (arglen,vargs,"ii",TRUE)) return;
+
+  lives_osc_parse_int_argument(vargs,&effect_key);
+  lives_osc_parse_int_argument(vargs,&pnum);
+
+  if (effect_key<1||effect_key>FX_MAX) return;
+  //g_print("key %d pnum %d\n",effect_key,pnum);
+
+  filter=rte_keymode_get_filter(effect_key,rte_key_getmode(effect_key));
+
+  if (!weed_plant_has_leaf(filter,"in_parameter_templates")) return;
+  nparams=weed_leaf_num_elements(filter,"in_parameter_templates");
+  if (pnum>=nparams) return;
+  
+  in_ptmpls=weed_get_plantptr_array(filter,"in_parameter_templates",&error);
+  
+  for (i=0;i<nparams;i++) {
+    ptmpl=in_ptmpls[i];
+    hint=weed_get_int_value(ptmpl,"hint",&error);
+    flags=weed_get_int_value(ptmpl,"flags",&error);
+    if ((hint==WEED_HINT_INTEGER||hint==WEED_HINT_FLOAT)&&flags==0&&weed_leaf_num_elements(ptmpl,"default")==1) {
+      if (pnum==0) {
+	if (hint==WEED_HINT_INTEGER) {
+	  vali=weed_get_int_value(ptmpl,"minimum",&error);
+	  msg=g_strdup_printf("%d\n",vali);
+	}
+	else {
+	  vald=weed_get_double_value(ptmpl,"minimum",&error);
+	  msg=g_strdup_printf("%.8f\n",vald);
+	}
+	lives_status_send(msg);
+	g_free(msg);
+	weed_free(in_ptmpls);
+	return;
+      }
+      pnum--;
+    }
+  }
+}
+
+
+
+void lives_osc_cb_rte_getnparammax(void *context, int arglen, const void *vargs, OSCTimeTag when, NetworkReturnAddressPtr ra) {
+
+  int effect_key;
+  int pnum,i;
+
+  // pick pnum which is numeric single valued, non-reinit
+  // i.e. simple numeric parameter
+
+  int error;
+  weed_plant_t *filter;
+  weed_plant_t **in_ptmpls;
+  weed_plant_t *ptmpl;
+  int hint,flags;
+  int nparams;
+
+  int vali;
+  double vald;
+
+  gchar *msg;
+  
+  if (!lives_osc_check_arguments (arglen,vargs,"ii",TRUE)) return;
+
+  lives_osc_parse_int_argument(vargs,&effect_key);
+  lives_osc_parse_int_argument(vargs,&pnum);
+
+  if (effect_key<1||effect_key>FX_MAX) return;
+  //g_print("key %d pnum %d\n",effect_key,pnum);
+
+  filter=rte_keymode_get_filter(effect_key,rte_key_getmode(effect_key));
+
+  if (!weed_plant_has_leaf(filter,"in_parameter_templates")) return;
+  nparams=weed_leaf_num_elements(filter,"in_parameter_templates");
+  if (pnum>=nparams) return;
+  
+  in_ptmpls=weed_get_plantptr_array(filter,"in_parameter_templates",&error);
+  
+  for (i=0;i<nparams;i++) {
+    ptmpl=in_ptmpls[i];
+    hint=weed_get_int_value(ptmpl,"hint",&error);
+    flags=weed_get_int_value(ptmpl,"flags",&error);
+    if ((hint==WEED_HINT_INTEGER||hint==WEED_HINT_FLOAT)&&flags==0&&weed_leaf_num_elements(ptmpl,"default")==1) {
+      if (pnum==0) {
+	if (hint==WEED_HINT_INTEGER) {
+	  vali=weed_get_int_value(ptmpl,"maximum",&error);
+	  msg=g_strdup_printf("%d\n",vali);
+	}
+	else {
+	  vald=weed_get_double_value(ptmpl,"maximum",&error);
+	  msg=g_strdup_printf("%.8f\n",vald);
+	}
+	lives_status_send(msg);
+	g_free(msg);
+	weed_free(in_ptmpls);
+	return;
+      }
+      pnum--;
+    }
+  }
+}
+
+
+void lives_osc_cb_rte_getnparamdef(void *context, int arglen, const void *vargs, OSCTimeTag when, NetworkReturnAddressPtr ra) {
+
+  int effect_key;
+  int pnum,i;
+
+  // pick pnum which is numeric single valued, non-reinit
+  // i.e. simple numeric parameter
+
+  int error;
+  weed_plant_t *filter;
+  weed_plant_t **in_ptmpls;
+  weed_plant_t *ptmpl;
+  int hint,flags;
+  int nparams;
+
+  int vali;
+  double vald;
+
+  gchar *msg;
+  
+  if (!lives_osc_check_arguments (arglen,vargs,"ii",TRUE)) return;
+
+  lives_osc_parse_int_argument(vargs,&effect_key);
+  lives_osc_parse_int_argument(vargs,&pnum);
+
+  if (effect_key<1||effect_key>FX_MAX) return;
+  //g_print("key %d pnum %d\n",effect_key,pnum);
+
+  filter=rte_keymode_get_filter(effect_key,rte_key_getmode(effect_key));
+
+  if (!weed_plant_has_leaf(filter,"in_parameter_templates")) return;
+  nparams=weed_leaf_num_elements(filter,"in_parameter_templates");
+  if (pnum>=nparams) return;
+  
+  in_ptmpls=weed_get_plantptr_array(filter,"in_parameter_templates",&error);
+  
+  for (i=0;i<nparams;i++) {
+    ptmpl=in_ptmpls[i];
+    hint=weed_get_int_value(ptmpl,"hint",&error);
+    flags=weed_get_int_value(ptmpl,"flags",&error);
+    if ((hint==WEED_HINT_INTEGER||hint==WEED_HINT_FLOAT)&&flags==0&&weed_leaf_num_elements(ptmpl,"default")==1) {
+      if (pnum==0) {
+	if (hint==WEED_HINT_INTEGER) {
+	  if (!weed_plant_has_leaf(ptmpl,"host_default")) vali=weed_get_int_value(ptmpl,"default",&error);
+	  else vali=weed_get_int_value(ptmpl,"host_default",&error);
+	  msg=g_strdup_printf("%d\n",vali);
+	}
+	else {
+	  if (!weed_plant_has_leaf(ptmpl,"host_default")) vald=weed_get_double_value(ptmpl,"default",&error);
+	  else vald=weed_get_double_value(ptmpl,"host_default",&error);
+	  msg=g_strdup_printf("%.8f\n",vald);
+	}
+	lives_status_send(msg);
+	g_free(msg);
+	weed_free(in_ptmpls);
+	return;
+      }
+      pnum--;
+    }
+  }
+}
+
+
+void lives_osc_cb_rte_getnparamtrans(void *context, int arglen, const void *vargs, OSCTimeTag when, NetworkReturnAddressPtr ra) {
+
+  int effect_key;
+  int pnum,i;
+
+  // pick pnum which is numeric single valued, non-reinit
+  // i.e. simple numeric parameter
+
+  int error;
+  weed_plant_t *filter;
+  weed_plant_t **in_ptmpls;
+  weed_plant_t *ptmpl;
+  int hint,flags;
+  int nparams;
+
+  gboolean res=FALSE;
+
+  gchar *msg;
+  
+  if (!lives_osc_check_arguments (arglen,vargs,"ii",TRUE)) return;
+
+  lives_osc_parse_int_argument(vargs,&effect_key);
+  lives_osc_parse_int_argument(vargs,&pnum);
+
+  if (effect_key<1||effect_key>FX_MAX) return;
+  //g_print("key %d pnum %d\n",effect_key,pnum);
+
+  filter=rte_keymode_get_filter(effect_key,rte_key_getmode(effect_key));
+
+  if (!weed_plant_has_leaf(filter,"in_parameter_templates")) return;
+  nparams=weed_leaf_num_elements(filter,"in_parameter_templates");
+  if (pnum>=nparams) return;
+  
+  in_ptmpls=weed_get_plantptr_array(filter,"in_parameter_templates",&error);
+  
+  for (i=0;i<nparams;i++) {
+    ptmpl=in_ptmpls[i];
+    hint=weed_get_int_value(ptmpl,"hint",&error);
+    flags=weed_get_int_value(ptmpl,"flags",&error);
+    if ((hint==WEED_HINT_INTEGER||hint==WEED_HINT_FLOAT)&&flags==0&&weed_leaf_num_elements(ptmpl,"default")==1) {
+      if (pnum==0) {
+	if (weed_plant_has_leaf(ptmpl,"transition")&&weed_get_boolean_value(ptmpl,"transition",&error)==WEED_TRUE) res=TRUE;
+	msg=g_strdup_printf("%d\n",res);
+	lives_status_send(msg);
+	g_free(msg);
+	weed_free(in_ptmpls);
+	return;
+      }
+      pnum--;
+    }
+  }
+}
+
 void lives_osc_cb_rte_getmode(void *context, int arglen, const void *vargs, OSCTimeTag when, NetworkReturnAddressPtr ra) {
   gchar *tmp;
 
@@ -1206,7 +1941,7 @@ void lives_osc_cb_rte_getmodespk(void *context, int arglen, const void *vargs, O
 
 
 
-
+// deprecated - do not use
 void lives_osc_cb_rte_getusermode(void *context, int arglen, const void *vargs, OSCTimeTag when, NetworkReturnAddressPtr ra) {
   int nmode;
   int effect_key;
@@ -1376,7 +2111,12 @@ static struct
     { "/effect_key/toggle",		"toggle",	lives_osc_cb_fx_toggle,		        25	},
     { "/effect_key/count",		"count",	lives_osc_cb_rte_count,		        25	},
     { "/effect_key/parameter/value/set",		"set",	lives_osc_cb_rte_setparam,		        42	},
+    { "/effect_key/nparameter/count",		"count",	lives_osc_cb_rte_nparamcount,		        91	},
     { "/effect_key/nparameter/value/set",		"set",	lives_osc_cb_rte_setnparam,		        92	},
+    { "/effect_key/nparameter/min/get",		"get",	lives_osc_cb_rte_getnparammin,		        93	},
+    { "/effect_key/nparameter/max/get",		"get",	lives_osc_cb_rte_getnparammax,		        94	},
+    { "/effect_key/nparameter/default/get",		"get",	lives_osc_cb_rte_getnparamdef,		        95	},
+    { "/effect_key/nparameter/is_transition/get",		"get",	lives_osc_cb_rte_getnparamtrans,		        96	},
     { "/effect_key/mode/set",		"set",	lives_osc_cb_rte_setmode,		        43	},
     { "/effect_key/mode/get",		"get",	lives_osc_cb_rte_getmode,		        43	},
     { "/effect_key/mode/next",		"next",	lives_osc_cb_rte_nextmode,		        43	},
@@ -1386,6 +2126,14 @@ static struct
     { "/effect_key/usermode/get",		"get",	lives_osc_cb_rte_getusermode,		        56	},
     { "/clip/encode_as",		"encode_as",	lives_osc_cb_clip_encodeas,			1	},
     { "/clip/select",		"select",	lives_osc_cb_fgclip_select,			1	},
+    { "/clip/close",		"close",	lives_osc_cb_fgclip_close,	  		        1	},
+    { "/clip/copy",		"copy",	lives_osc_cb_fgclip_copy,	  		        1	},
+    { "/clip/selection/copy",		"copy",	lives_osc_cb_fgclipsel_copy,	  		        55	},
+    { "/clip/selection/cut",		"cut",	lives_osc_cb_fgclipsel_cut,	  		        55	},
+    { "/clip/selection/delete",		"delete",	lives_osc_cb_fgclipsel_delete,	  		        55	},
+    { "/clipboard/paste",		"paste",	lives_osc_cb_clipbd_paste,			70	},
+    { "/clipboard/insert_before",		"insert_before",	lives_osc_cb_clipbd_insertb,			70	},
+    { "/clipboard/insert_after",		"insert_after",	lives_osc_cb_clipbd_inserta,			70	},
     { "/clip/retrigger",		"retrigger",	lives_osc_cb_fgclip_retrigger,			1	},
     { "/clip/select/next",		"next",	lives_osc_cb_fgclip_select_next,			54	},
     { "/clip/select/previous",		"previous",	lives_osc_cb_fgclip_select_previous,			54	},
@@ -1465,6 +2213,8 @@ static struct
     {	"/clip/start/", 	"start",         50, 1,0	},
     {	"/clip/end/", 	        "end",           51, 1,0	},
     {	"/clip/select/", 	        "select",           54, 1,0	},
+    {	"/clip/selection/", 	        "selection",           55, 1,0	},
+    {	"/clipboard/", 		"clipboard",		 70, -1,0	},
     {	"/record/", 		"record",	 3, -1,0	},
     {	"/effect/" , 		"effect",	 4, -1,0	},
     {	"/effect_key/" , 		"effect_key",	 25, -1,0	},
@@ -1472,6 +2222,10 @@ static struct
     {	"/effect_key/parameter/value/" ,"value",	 42, 41,0	},
     {	"/effect_key/nparameter/" , 	"nparameter",	 91, 25,0	},
     {	"/effect_key/nparameter/value/" ,"value",	 92, 91,0	},
+    {	"/effect_key/nparameter/min/" ,"min",	 93, 91,0	},
+    {	"/effect_key/nparameter/max/" ,"max",	 94, 91,0	},
+    {	"/effect_key/nparameter/default/" ,"default",	 95, 91,0	},
+    {	"/effect_key/nparameter/is_transition/" ,"is_transition", 96, 91,0  },
     {	"/effect_key/map/" , 		"map",	 32, 25,0	},
     {	"/effect_key/mode/" , 		"mode",	 43, 25,0	},
     {	"/effect_key/name/" , 		"name",	 44, 25,0	},
