@@ -1,6 +1,6 @@
 // plugins.c
 // LiVES
-// (c) G. Finch 2003 - 2009 <salsaman@xs4all.nl>
+// (c) G. Finch 2003 - 2010 <salsaman@xs4all.nl>
 // released under the GNU GPL 3 or later
 // see file ../COPYING or www.gnu.org for licensing details
 
@@ -82,7 +82,7 @@ static GList *get_plugin_result (gchar *command, gchar *delim, gboolean allow_bl
   }
   pthread_mutex_unlock(&mainw->gtk_mutex);
 
-  while (!(outfile_fd=open(outfile,O_RDONLY))&&(count-->0||list_plugins)) {
+  while ((outfile_fd=open(outfile,O_RDONLY))==-1&&(count-->0||list_plugins)) {
     g_usleep (prefs->sleep_time);
   }
   
@@ -169,7 +169,11 @@ plugin_request_common (const gchar *plugin_type, const gchar *plugin_name, const
       com=g_strdup_printf("%s/%s %s",prefs->tmpdir,plugin_name,request);
     }
     else {
+#ifdef DEBUG_PLUGINS
       com=g_strdup_printf ("%s%s%s/%s %s",prefs->lib_dir,PLUGIN_EXEC_DIR,plugin_type,plugin_name,request);
+#else
+      com=g_strdup_printf ("%s%s%s/%s %s 2>/dev/null",prefs->lib_dir,PLUGIN_EXEC_DIR,plugin_type,plugin_name,request);
+#endif
     }
     if (plugin_name==NULL||!strlen(plugin_name)) {
       return reslist;
@@ -275,7 +279,7 @@ void save_vpp_defaults(_vid_playback_plugin *vpp) {
     return;
   }
 
-  if (!(fd=open(vpp_file,O_WRONLY|O_CREAT|O_TRUNC,S_IRUSR|S_IWUSR))) {
+  if ((fd=open(vpp_file,O_WRONLY|O_CREAT|O_TRUNC,S_IRUSR|S_IWUSR))==-1) {
     msg=g_strdup_printf (_("\n\nUnable to write video playback plugin defaults file\n%s\nError code %d\n"),vpp_file,errno);
     g_printerr ("%s",msg);
     g_free (msg);
@@ -339,7 +343,7 @@ void load_vpp_defaults(_vid_playback_plugin *vpp) {
   d_print(msg);
   g_free(msg);
 
-  if (!(fd=open(vpp_file,O_RDONLY))) {
+  if ((fd=open(vpp_file,O_RDONLY))==-1) {
     msg=g_strdup_printf (_("unable to read file\n%s\nError code %d\n"),vpp_file,errno);
     d_print (msg);
     g_free (msg);
@@ -694,7 +698,9 @@ _vppaw *on_vpp_advanced_clicked (GtkButton *button, gpointer user_data) {
   if (prefsw!=NULL) gtk_widget_hide (prefsw->prefs_dialog);
 
   gtk_window_set_position (GTK_WINDOW (vppa->dialog), GTK_WIN_POS_CENTER);
-  gtk_window_set_transient_for(GTK_WINDOW(vppa->dialog),GTK_WINDOW(mainw->LiVES));
+  if (!prefs->show_gui) {
+    gtk_window_set_transient_for(GTK_WINDOW(vppa->dialog),GTK_WINDOW(mainw->LiVES));
+  }
   gtk_window_set_modal (GTK_WINDOW (vppa->dialog), TRUE);
 
   if (palette->style&STYLE_1) {
@@ -1217,9 +1223,9 @@ do_plugin_encoder_error(const gchar *plugin_name) {
     g_free(msg);
     return;
   }
+
   msg=g_strdup_printf(_("LiVES did not receive a response from the encoder plugin called '%s'.\nPlease make sure you have that plugin installed correctly in\n%s%s%s\nor switch to another plugin using Tools|Preferences|Encoding\n"),plugin_name,prefs->lib_dir,PLUGIN_EXEC_DIR,PLUGIN_ENCODERS);
-  if (rdet!=NULL) do_error_dialog_with_check_transient(msg,FALSE,0,GTK_WINDOW(rdet->dialog));
-  else do_error_dialog(msg);
+  do_blocking_error_dialog(msg);
   g_free(msg);
 }
 
@@ -1248,7 +1254,7 @@ gboolean check_encoder_restrictions (gboolean get_extension, gboolean user_audio
 
   gint best_fps_num=0,best_fps_denom=0;
   gdouble fps;
-  gint arate,achans,asampsize;
+  gint arate,achans,asampsize,asigned=0;
 
   if (rdet==NULL) {
     width=owidth=cfile->hsize;
@@ -1291,6 +1297,7 @@ gboolean check_encoder_restrictions (gboolean get_extension, gboolean user_audio
 	  prefs->encoder.of_allowed_acodecs=atoi (array[2]);
 	  g_list_free_strings (ofmt_all);
 	  g_list_free (ofmt_all);
+	  g_strfreev(array);
 	  break;
 	}
 	g_strfreev(array);
@@ -1416,6 +1423,29 @@ gboolean check_encoder_restrictions (gboolean get_extension, gboolean user_audio
       continue;
     }
 
+
+
+    if (!strncmp (checks[r],"asigned=",8)&&((mainw->save_with_sound||rdet!=NULL)&&(resaudw==NULL||resaudw->aud_checkbutton==NULL||gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(resaudw->aud_checkbutton))))&&prefs->encoder.audio_codec!=AUDIO_CODEC_NONE&&(arate*achans*asampsize)) {
+      array=g_strsplit(checks[r],"=",2);
+      if (!strcmp(array[1],"signed")) {
+	asigned=1;
+      }
+
+      if (!strcmp(array[1],"unsigned")) {
+	asigned=2;
+      }
+
+      g_strfreev(array);
+      
+      if (asigned!=0&&!capable->has_sox) {
+	do_encoder_sox_error();
+	g_strfreev(checks);
+	return FALSE;
+      }
+      continue;
+    }
+
+
     if (!strncmp (checks[r],"arate=",6)&&((mainw->save_with_sound||rdet!=NULL)&&(resaudw==NULL||resaudw->aud_checkbutton==NULL||gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(resaudw->aud_checkbutton))))&&prefs->encoder.audio_codec!=AUDIO_CODEC_NONE&&(arate*achans*asampsize)) {
       // we only perform this test if we are encoding with audio
       // find next highest allowed rate from list,
@@ -1486,6 +1516,8 @@ gboolean check_encoder_restrictions (gboolean get_extension, gboolean user_audio
     gint xwidth=width;
     gint xheight=height;
 
+    width=height=1000000;
+
     numtok=get_token_count (aspect_buffer,';');
     array2=g_strsplit(aspect_buffer,";",numtok);
     
@@ -1504,7 +1536,6 @@ gboolean check_encoder_restrictions (gboolean get_extension, gboolean user_audio
     if (owidth%hblock==0&&oheight%vblock==0) allow_aspect_override=TRUE;
 
     // end recheck
-
   }
 
   // fps can't be altered if we have a multitrack event_list
@@ -1512,7 +1543,7 @@ gboolean check_encoder_restrictions (gboolean get_extension, gboolean user_audio
 
   if (sizer) allow_aspect_override=FALSE;
 
-  if (((width!=owidth||height!=oheight)&&width*height>0)||(best_fps_delta>0.)||(best_arate_delta>0&&best_arate>0)||best_arate<0) {
+  if (((width!=owidth||height!=oheight)&&width*height>0)||(best_fps_delta>0.)||(best_arate_delta>0&&best_arate>0)||best_arate<0||asigned!=0) {
     gboolean ofx1_bool=mainw->fx1_bool;
     mainw->fx1_bool=FALSE;
     if ((width!=owidth||height!=oheight)&&width*height>0) {
@@ -1527,19 +1558,23 @@ gboolean check_encoder_restrictions (gboolean get_extension, gboolean user_audio
 	}
       }
     }
-
     if (rdet!=NULL&&!rdet->is_encoding) {
       rdet->arate=(gint)atoi (gtk_entry_get_text(GTK_ENTRY(resaudw->entry_arate)));
       rdet->achans=(gint)atoi (gtk_entry_get_text(GTK_ENTRY(resaudw->entry_achans)));
       rdet->asamps=(gint)atoi (gtk_entry_get_text(GTK_ENTRY(resaudw->entry_asamps)));
+      rdet->aendian=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(resaudw->rb_unsigned))?AFORM_UNSIGNED:AFORM_SIGNED;
 
-      if (width!=rdet->width||height!=rdet->height||best_fps_delta!=0.||best_arate!=rdet->arate) {
-	if (rdet_suggest_values(width,height,best_fps,best_fps_num,best_fps_denom,best_arate,allow_aspect_override,(best_fps_delta==0.))) {
+      if (width!=rdet->width||height!=rdet->height||best_fps_delta!=0.||best_arate!=rdet->arate||((asigned==1&&rdet->aendian==AFORM_UNSIGNED)||(asigned==2&&rdet->aendian==AFORM_SIGNED))) {
+	if (rdet_suggest_values(width,height,best_fps,best_fps_num,best_fps_denom,best_arate,asigned,allow_aspect_override,(best_fps_delta==0.))) {
 	  gchar *arate_string;
 	  rdet->width=width;
 	  rdet->height=height;
 	  if (best_arate!=-1) rdet->arate=best_arate;
 	  else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(resaudw->aud_checkbutton),FALSE);
+
+	  if (asigned==1) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(resaudw->rb_signed),TRUE);
+	  else if (asigned==2) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(resaudw->rb_unsigned),TRUE);
+
 	  if (best_fps_delta>0.) {
 	    if (best_fps_denom>0) {
 	      rdet->fps=(best_fps_num*1.)/(best_fps_denom*1.);
@@ -1561,13 +1596,13 @@ gboolean check_encoder_restrictions (gboolean get_extension, gboolean user_audio
       return FALSE;
     }
 
-    if (mainw->osc_auto||do_encoder_restrict_dialog (width,height,best_fps,best_fps_num,best_fps_denom,best_arate,allow_aspect_override)) {
+    if (mainw->osc_auto||do_encoder_restrict_dialog (width,height,best_fps,best_fps_num,best_fps_denom,best_arate,asigned,allow_aspect_override)) {
       if (!mainw->fx1_bool&&mainw->osc_enc_width==0) {
 	width=owidth;
 	height=oheight;
       }
 
-      if (!auto_resample_resize (width,height,best_fps,best_fps_num,best_fps_denom,best_arate)) {
+      if (!auto_resample_resize (width,height,best_fps,best_fps_num,best_fps_denom,best_arate,asigned)) {
 	mainw->fx1_bool=ofx1_bool;
 	return FALSE;
       }
@@ -1593,8 +1628,6 @@ GList *filter_encoders_by_img_ext(GList *encoders, const gchar *img_ext) {
   while (list!=NULL) {
     listnext=list->next;
     if ((encoder_capabilities=plugin_request(PLUGIN_ENCODERS,list->data,"get_capabilities"))==NULL) {
-      do_plugin_encoder_error(list->data);
-
       encoders=g_list_delete_link(encoders,list);
     }
     else {
@@ -1754,11 +1787,12 @@ _decoder_plugin *open_decoder_plugin(const gchar *plname, file *sfile) {
   g_free(plugname);
 
   dplug->handle=dlopen(nplugname,RTLD_LAZY);
+  g_free(nplugname);
+
   if (dplug->handle==NULL) {
     gchar *msg=g_strdup_printf (_("\n\nFailed to open decoder plugin %s\nError was %s\n"),plname,dlerror());
     d_print(msg);
     g_free (msg);
-    g_free(nplugname);
     g_free(dplug);
     return NULL;
   }
@@ -1781,7 +1815,6 @@ _decoder_plugin *open_decoder_plugin(const gchar *plname, file *sfile) {
     d_print(msg);
     g_free (msg);
     close_decoder_plugin(0,dplug);
-    g_free(nplugname);
     g_free(dplug);
     return NULL;
   }
@@ -1790,7 +1823,7 @@ _decoder_plugin *open_decoder_plugin(const gchar *plname, file *sfile) {
   dplug->module_unload=dlsym (dplug->handle,"module_unload");
   dplug->set_palette=dlsym (dplug->handle,"set_palette");
   dplug->rip_audio=dlsym (dplug->handle,"rip_audio");
-  dplug->set_audio_fmt=dlsym (dplug->handle,"set_audio_fmt");
+  dplug->rip_audio_cleanup=dlsym (dplug->handle,"rip_audio_cleanup");
 
   err=(*dplug->module_check_init)();
 
@@ -2641,6 +2674,7 @@ lives_param_t *weed_params_to_rfx(gint npar, weed_plant_t *plant, gboolean show_
     if (gui!=NULL) {
       if (weed_plant_has_leaf(gui,"label")) {
 	string=weed_get_string_value(gui,"label",&error);
+	g_free(rpar[i].label);
 	rpar[i].label=g_strdup(string);
 	weed_free(string);
       }
@@ -2784,107 +2818,111 @@ gchar *plugin_run_param_window(gchar *get_com, GtkVBox *vbox, lives_rfx_t **ret_
   // if the user closes the window with Cancel, NULL is returned instead
 
 
-    FILE *sfile;
-    lives_rfx_t *rfx=(lives_rfx_t *)g_malloc(sizeof(lives_rfx_t));
-    gchar *string;
-    gchar *rfx_scrapname=g_strdup_printf("rfx.%d",getpid());
-    gchar *rfxfile=g_strdup_printf ("%s/.%s.script",prefs->tmpdir,rfx_scrapname);
-    int res;
-    gchar *com;
-    gchar *res_string=NULL;
-    gchar buff[32];
+  FILE *sfile;
+  lives_rfx_t *rfx=(lives_rfx_t *)g_malloc(sizeof(lives_rfx_t));
+  gchar *string;
+  gchar *rfx_scrapname=g_strdup_printf("rfx.%d",getpid());
+  gchar *rfxfile=g_strdup_printf ("%s/.%s.script",prefs->tmpdir,rfx_scrapname);
+  int res;
+  gchar *com;
+  gchar *res_string=NULL;
+  gchar buff[32];
 
-    string=g_strdup_printf("<name>\n%s\n</name>\n",rfx_scrapname);
-    sfile=fopen(rfxfile,"w");
-    fputs(string,sfile);
-    fclose(sfile);
-    g_free(string);
+  string=g_strdup_printf("<name>\n%s\n</name>\n",rfx_scrapname);
+  sfile=fopen(rfxfile,"w");
+  fputs(string,sfile);
+  fclose(sfile);
+  g_free(string);
 
-    com=g_strdup_printf("%s >>%s",get_com,rfxfile);
+  com=g_strdup_printf("%s >>%s",get_com,rfxfile);
+  dummyvar=system(com);
+  g_free(com);
+
+  // OK, we should now have an RFX fragment in a file, we can compile it, then build a parameter window from it
+    
+  com=g_strdup_printf("%s %s %s >/dev/null",RFX_BUILDER,rfxfile,prefs->tmpdir);
+  res=system(com);
+  g_free(com);
+    
+  unlink(rfxfile);
+  g_free(rfxfile);
+
+  if (res==0) {
+    // the script compiled correctly
+
+    // now we pop up the parameter window, get the values of our parameters, and marshall them as extra_params
+      
+    // first create a lives_rfx_t from the scrap
+    rfx->name=g_strdup(rfx_scrapname);
+    rfx->action_desc=rfx->extra=NULL;
+    rfx->status=RFX_STATUS_SCRAP;
+
+    rfx->num_in_channels=0;
+    rfx->min_frames=-1;
+
+    // get the delimiter
+    rfxfile=g_strdup_printf("%ssmdef.%d",prefs->tmpdir,getpid());
+    com=g_strdup_printf("%s%s get_define > %s",prefs->tmpdir,rfx_scrapname,rfxfile);
     dummyvar=system(com);
     g_free(com);
 
-    // OK, we should now have an RFX fragment in a file, we can compile it, then build a parameter window from it
-    
-    com=g_strdup_printf("%s %s %s >/dev/null",RFX_BUILDER,rfxfile,prefs->tmpdir);
-    res=system(com);
-    g_free(com);
-    
+    sfile=fopen(rfxfile,"r");
+    dummychar=fgets(buff,32,sfile);
+    fclose(sfile);
+
     unlink(rfxfile);
     g_free(rfxfile);
 
-    if (res==0) {
-      // the script compiled correctly
+    g_snprintf(rfx->delim,2,"%s",buff);
 
-      // now we pop up the parameter window, get the values of our parameters, and marshall them as extra_params
+    rfx->menu_text=(vbox==NULL?g_strdup_printf(_("%s advanced settings"),prefs->encoder.of_desc):g_strdup(""));
+    rfx->is_template=FALSE;
       
-      // first create a lives_rfx_t from the scrap
-      rfx->name=g_strdup(rfx_scrapname);
-      rfx->action_desc=rfx->extra=NULL;
-      rfx->status=RFX_STATUS_SCRAP;
+    render_fx_get_params(rfx,rfx_scrapname,RFX_STATUS_SCRAP);
 
-      rfx->num_in_channels=0;
-      rfx->min_frames=-1;
+    // now we build our window and get param values
+    if (vbox==NULL) {
+      on_render_fx_pre_activate(NULL,rfx);
 
-      // get the delimiter
-      rfxfile=g_strdup_printf("%ssmdef.%d",prefs->tmpdir,getpid());
-      com=g_strdup_printf("%s%s get_define > %s",prefs->tmpdir,rfx_scrapname,rfxfile);
-      dummyvar=system(com);
-      g_free(com);
-
-      sfile=fopen(rfxfile,"r");
-      dummychar=fgets(buff,32,sfile);
-      fclose(sfile);
-
-      unlink(rfxfile);
-      g_free(rfxfile);
-
-      g_snprintf(rfx->delim,2,"%s",buff);
-
-      rfx->menu_text=(vbox==NULL?g_strdup_printf(_("%s advanced settings"),prefs->encoder.of_desc):g_strdup(""));
-      rfx->is_template=FALSE;
-      
-      render_fx_get_params(rfx,rfx_scrapname,RFX_STATUS_SCRAP);
-
-      // now we build our window and get param values
-      if (vbox==NULL) {
-	on_render_fx_pre_activate(NULL,rfx);
-
-	gtk_window_set_transient_for(GTK_WINDOW(fx_dialog[1]),GTK_WINDOW(mainw->LiVES));
-	gtk_window_set_modal (GTK_WINDOW (fx_dialog[1]), TRUE);
-      
-	if (gtk_dialog_run(GTK_DIALOG(fx_dialog[1]))==GTK_RESPONSE_OK) {
-	  // marshall our params for passing to the plugin
-	  res_string=param_marshall(rfx,FALSE);
-	}
-      }
-      else {
-	make_param_box(vbox,rfx);
+      if (prefs->show_gui) {
+	if (mainw->multitrack==NULL) gtk_window_set_transient_for(GTK_WINDOW(fx_dialog[1]),GTK_WINDOW(mainw->LiVES));
+	else gtk_window_set_transient_for(GTK_WINDOW(fx_dialog[1]),GTK_WINDOW(mainw->multitrack->window));
       }
+      gtk_window_set_modal (GTK_WINDOW (fx_dialog[1]), TRUE);
       
-      rfxfile=g_strdup_printf ("%s/%s",prefs->tmpdir,rfx_scrapname);
-      unlink(rfxfile);
-      g_free(rfx_scrapname);
-      g_free(rfxfile);
-
-      if (ret_rfx!=NULL) {
-	*ret_rfx=rfx;
-      }
-      else {
-	rfx_free(rfx);
-	g_free(rfx);
+      if (gtk_dialog_run(GTK_DIALOG(fx_dialog[1]))==GTK_RESPONSE_OK) {
+	// marshall our params for passing to the plugin
+	res_string=param_marshall(rfx,FALSE);
       }
     }
     else {
-      if (ret_rfx!=NULL) {
-	*ret_rfx=NULL;
-	if (rfx!=NULL) {
-	  g_free(rfx);
-	}
-      }
-      else {
-	res_string=g_strdup("");
-      }
+      make_param_box(vbox,rfx);
     }
-    return res_string;
+      
+    rfxfile=g_strdup_printf ("%s/%s",prefs->tmpdir,rfx_scrapname);
+    unlink(rfxfile);
+    g_free(rfxfile);
+
+    if (ret_rfx!=NULL) {
+      *ret_rfx=rfx;
+    }
+    else {
+      rfx_free(rfx);
+      g_free(rfx);
+    }
+  }
+  else {
+    if (ret_rfx!=NULL) {
+      *ret_rfx=NULL;
+    }
+    else {
+      res_string=g_strdup("");
+    }
+    if (rfx!=NULL) {
+      g_free(rfx);
+    }
+  }
+
+  g_free(rfx_scrapname);
+  return res_string;
 }
