@@ -1,6 +1,6 @@
 // pulse.c
 // LiVES (lives-exe)
-// (c) G. Finch 2005 - 2009
+// (c) G. Finch 2005 - 2010
 // Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
@@ -60,14 +60,14 @@ gboolean lives_pulse_init (short startup_phase) {
   pa_state=pa_context_get_state(pcon);
 
   gettimeofday(&otv, NULL);
-  stime=otv.tv_sec*1000000000+otv.tv_usec;
+  stime=otv.tv_sec*1000000+otv.tv_usec;
 
   while (pa_state!=PA_CONTEXT_READY&&ntime<PULSE_START_WAIT) {
     g_usleep(prefs->sleep_time);
     sched_yield();
     pa_state=pa_context_get_state(pcon);
     gettimeofday(&otv, NULL);
-    ntime=(otv.tv_sec*1000000000+otv.tv_usec-stime)>>10;
+    ntime=(otv.tv_sec*1000000+otv.tv_usec-stime);
   }
 
   if (ntime>=PULSE_START_WAIT) {
@@ -86,12 +86,12 @@ gboolean lives_pulse_init (short startup_phase) {
     }
     else {
       msg=g_strdup(_("\nUnable to connect to pulse audio server.\n"));
-      if (startup_phase<1) {
+      if (startup_phase==0) {
 	do_error_dialog(msg);
       }
       else {
 	msg2=g_strdup_printf("%s%s",msg,_("LiVES will exit and you can choose another audio player.\n"));
-	do_error_dialog(msg2);
+	do_blocking_error_dialog(msg2);
 	g_free(msg2);
       }
       g_free(msg);
@@ -104,9 +104,11 @@ gboolean lives_pulse_init (short startup_phase) {
 
 
 void pulse_get_rec_avals(pulse_driver_t *pulsed) {
-  mainw->rec_aseek=pulsed->seek_pos/(gdouble)(afile->arate*afile->achans*afile->asampsize/8);
   mainw->rec_aclip=pulsed->playing_file;
-  mainw->rec_avel=afile->pb_fps/afile->fps;
+  if (mainw->rec_aclip!=-1) {
+    mainw->rec_aseek=pulsed->seek_pos/(gdouble)(afile->arate*afile->achans*afile->asampsize/8);
+    mainw->rec_avel=afile->pb_fps/afile->fps;
+  }
 }
 
 
@@ -128,9 +130,7 @@ static void sample_silence_pulse (pulse_driver_t *pdriver, size_t nbytes, size_t
 
 static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *arg) {
   // PULSE AUDIO calls this periodically to get the next audio buffer
-
   static float old_volume=-1.;
-
   pulse_driver_t* pulsed = (pulse_driver_t*)arg;
 
   gulong nframes=nbytes/pulsed->out_achans/(pulsed->out_asamps>>3);
@@ -221,6 +221,7 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
     glong in_frames=0;
     gulong in_bytes=0;
     gfloat shrink_factor=1.f;
+    int swap_sign;
 
 #ifdef DEBUG_PULSE
     g_printerr("playing... pulseFramesAvailable = %ld\n", pulseFramesAvailable);
@@ -256,7 +257,7 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
       else {
 	if (G_LIKELY(pulsed->fd>0)) {
 	  pulsed->aPlayPtr->size=0;
-	  in_bytes=ABS((in_frames=((gdouble)pulsed->in_arate/(gdouble)pulsed->out_arate*(gdouble)pulseFramesAvailable+(fastrand()/G_MAXUINT32))))*pulsed->in_achans*(pulsed->in_asamps>>3);
+	  in_bytes=ABS((in_frames=((gdouble)pulsed->in_arate/(gdouble)pulsed->out_arate*(gdouble)pulseFramesAvailable+((gdouble)fastrand()/(gdouble)G_MAXUINT32))))*pulsed->in_achans*(pulsed->in_asamps>>3);
 	  //g_print("in bytes=%ld %ld %ld %ld %d %d\n",in_bytes,pulsed->in_arate,pulsed->out_arate,pulseFramesAvailable,pulsed->in_achans,pulsed->in_asamps);
 	  if ((shrink_factor=(gfloat)in_frames/(gfloat)pulseFramesAvailable)<0.f) {
 	    // reverse playback
@@ -384,7 +385,9 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
     g_printerr("nframes == %ld, pulseFramesAvailable == %ld,\n\tpulsed->num_input_channels == %ld, pulsed->out_achans == %ld\n",  nframes, pulseFramesAvailable, pulsed->in_achans, pulsed->out_achans);
 #endif
     
-    if (pulsed->in_asamps==pulsed->out_asamps&&shrink_factor==1.&&pulsed->in_achans==pulsed->out_achans&&!pulsed->reverse_endian) {
+    swap_sign=afile->signed_endian&AFORM_UNSIGNED;
+
+    if (pulsed->in_asamps==pulsed->out_asamps&&shrink_factor==1.&&pulsed->in_achans==pulsed->out_achans&&!pulsed->reverse_endian&&!swap_sign) {
       // no transformation needed
       pulsed->sound_buffer=buffer;
     }
@@ -392,10 +395,10 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
       pulsed->sound_buffer=g_malloc0(pulsed->chunk_size);
 
       if (pulsed->in_asamps==8) {
-	sample_move_d8_d16 ((short *)(pulsed->sound_buffer),(guchar *)buffer, numFramesToWrite, in_bytes, shrink_factor, pulsed->out_achans, pulsed->in_achans);
+	sample_move_d8_d16 ((short *)(pulsed->sound_buffer),(guchar *)buffer, numFramesToWrite, in_bytes, shrink_factor, pulsed->out_achans, pulsed->in_achans, swap_sign?SWAP_U_TO_S:0);
       }
       else {
-	sample_move_d16_d16((short*)pulsed->sound_buffer, (short*)buffer, numFramesToWrite, in_bytes, shrink_factor, pulsed->out_achans, pulsed->in_achans, pulsed->reverse_endian,FALSE);
+	sample_move_d16_d16((short*)pulsed->sound_buffer, (short*)buffer, numFramesToWrite, in_bytes, shrink_factor, pulsed->out_achans, pulsed->in_achans, pulsed->reverse_endian?SWAP_X_TO_L:0, swap_sign?SWAP_U_TO_S:0);
       }
     }
 
@@ -410,24 +413,29 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
   // playback from memory or file
 
   if (mainw->volume!=old_volume) {
+    pa_operation *pa_op;
     pavol=pa_sw_volume_from_linear(mainw->volume);
     pa_cvolume_set(&out_vol,pulsed->out_achans,pavol);
-    pa_context_set_sink_input_volume(pulsed->con,pa_stream_get_index(pulsed->pstream),&out_vol,NULL,NULL);
-    old_volume=pavol;
+    pa_op=pa_context_set_sink_input_volume(pulsed->con,pa_stream_get_index(pulsed->pstream),&out_vol,NULL,NULL);
+    pa_operation_unref(pa_op);
+    old_volume=mainw->volume;
   }
 
+
   while (nbytes>0) {
     if (nbytes<xbytes) xbytes=nbytes;
-      
+    
     if (!from_memory) {
-      if (xbytes/pulsed->out_achans/(pulsed->out_asamps>>3)<=numFramesToWrite&&offs==0) buffer=pulsed->sound_buffer;
+      if (xbytes/pulsed->out_achans/(pulsed->out_asamps>>3)<=numFramesToWrite&&offs==0) {
+	buffer=pulsed->sound_buffer;
+      }
       else {
 	buffer=g_malloc(xbytes);
 	w_memcpy(buffer,pulsed->sound_buffer+offs,xbytes);
 	offs+=xbytes;
 	needs_free=TRUE;
       }
-      pa_stream_write(pulsed->pstream,buffer,xbytes,needs_free?pulse_buff_free:NULL,0,PA_SEEK_RELATIVE);
+      pa_stream_write(pulsed->pstream,buffer,xbytes,buffer==pulsed->aPlayPtr->data?NULL:pulse_buff_free,0,PA_SEEK_RELATIVE);
     }
     else {
       if (pulsed->read_abuf>-1&&!pulsed->mute) {
@@ -494,10 +502,10 @@ void pulse_flush_read_data(pulse_driver_t *pulsed, size_t rbytes, void *data) {
   if (frames_out != pulsed->chunk_size) pulsed->chunk_size = frames_out;
 
   if (afile->asampsize==16) {
-    sample_move_d16_d16((short *)holding_buff,gbuf,frames_out,prb,out_scale,afile->achans,pulsed->in_achans,pulsed->reverse_endian,(gboolean)swap_sign);
+    sample_move_d16_d16((short *)holding_buff,gbuf,frames_out,prb,out_scale,afile->achans,pulsed->in_achans,pulsed->reverse_endian?SWAP_L_TO_X:0,swap_sign?SWAP_S_TO_U:0);
   }
   else {
-    sample_move_d16_d8((uint8_t *)holding_buff,gbuf,frames_out,prb,out_scale,afile->achans,pulsed->in_achans,(gboolean)swap_sign);
+    sample_move_d16_d8((uint8_t *)holding_buff,gbuf,frames_out,prb,out_scale,afile->achans,pulsed->in_achans,swap_sign?SWAP_S_TO_U:0);
   }
     
   pulsed->frames_written+=frames_out;
@@ -610,6 +618,7 @@ int pulse_audio_init(void) {
   pulsed.is_paused=FALSE;
   pulsed.pstream=NULL;
   pulsed.pa_props=NULL;
+  pulsed.playing_file=-1;
   return 0;
 }
 
@@ -644,10 +653,11 @@ int pulse_audio_read_init(void) {
 
 
 
+
 int pulse_driver_activate(pulse_driver_t *pdriver) {
 // create a new client and connect it to pulse server
   gchar *pa_clientname="LiVES_audio_out";
-  gchar *mypid=g_strdup_printf("%d",getpid());
+  gchar *mypid;
 
   pa_sample_spec pa_spec;
   pa_channel_map pa_map;
@@ -657,6 +667,10 @@ int pulse_driver_activate(pulse_driver_t *pdriver) {
 
   pa_volume_t pavol;
 
+  if (pdriver->pstream!=NULL) return 0;
+
+  mypid=g_strdup_printf("%d",getpid());
+
   pdriver->pa_props=pa_proplist_new();
 
   pa_proplist_sets(pdriver->pa_props, PA_PROP_APPLICATION_ICON_NAME, "LiVES");
@@ -679,10 +693,10 @@ int pulse_driver_activate(pulse_driver_t *pdriver) {
   if (G_BYTE_ORDER==G_BIG_ENDIAN) pdriver->out_endian=AFORM_BIG_ENDIAN;
   else pdriver->out_endian=AFORM_LITTLE_ENDIAN;
 
-  pa_battr.maxlength=(uint32_t)-1;
+  pa_battr.maxlength=LIVES_PA_BUFF_MAXLEN;
   pa_battr.minreq=(uint32_t)-1;
-  pa_battr.tlength=2048;
-  pa_battr.prebuf=(uint32_t)-1;
+  pa_battr.tlength=LIVES_PA_BUFF_TARGET;
+  pa_battr.prebuf=0;
 
   if (pulse_server_rate==0) {
     pa_op=pa_context_get_server_info(pdriver->con,pulse_server_cb,NULL);
@@ -690,8 +704,10 @@ int pulse_driver_activate(pulse_driver_t *pdriver) {
     while (pa_operation_get_state(pa_op)!=PA_OPERATION_DONE) {
       g_usleep(prefs->sleep_time);
     }
+    pa_operation_unref(pa_op);
   }
 
+
   pa_spec.rate=pdriver->out_arate=pdriver->in_arate=pulse_server_rate;
 
   pdriver->pstream=pa_stream_new_with_proplist(pdriver->con,pa_clientname,&pa_spec,&pa_map,pdriver->pa_props);
@@ -701,6 +717,9 @@ int pulse_driver_activate(pulse_driver_t *pdriver) {
     pavol=pa_sw_volume_from_linear(mainw->volume);
     pa_cvolume_set(&out_vol,pdriver->out_achans,pavol);
 
+    // set write callback
+    pa_stream_set_write_callback(pdriver->pstream,pulse_audio_write_process,pdriver);
+
 #ifdef PA_STREAM_START_UNMUTED
     pa_stream_connect_playback(pdriver->pstream,NULL,&pa_battr,PA_STREAM_START_UNMUTED|PA_STREAM_ADJUST_LATENCY,&out_vol,NULL);
 #else
@@ -710,12 +729,11 @@ int pulse_driver_activate(pulse_driver_t *pdriver) {
     while (pa_stream_get_state(pdriver->pstream)!=PA_STREAM_READY) {
       g_usleep(prefs->sleep_time);
     }
-
-    // set write callback
-    pa_stream_set_write_callback(pdriver->pstream,pulse_audio_write_process,pdriver);
-
   }
   else {
+    // set read callback
+    pa_stream_set_read_callback(pdriver->pstream,pulse_audio_read_process,pdriver);
+
     pa_stream_connect_record(pdriver->pstream,NULL,&pa_battr,0);
 
     prb=0;
@@ -724,8 +742,6 @@ int pulse_driver_activate(pulse_driver_t *pdriver) {
       g_usleep(prefs->sleep_time);
     }
 
-    // set read callback
-    pa_stream_set_read_callback(pdriver->pstream,pulse_audio_read_process,pdriver);
   }
 
   g_free(mypid);
@@ -754,22 +770,31 @@ volatile aserver_message_t *pulse_get_msgq(pulse_driver_t *pulsed) {
 
 
 
-gint64 lives_pulse_get_time(pulse_driver_t *pulsed) {
+gint64 lives_pulse_get_time(pulse_driver_t *pulsed, gboolean absolute) {
+  // get the time in ticks since either playback started or since last seek
+
   volatile aserver_message_t *msg=pulsed->msgq;
   gdouble frames_written;
   if (msg!=NULL&&msg->command==ASERVER_CMD_FILE_SEEK) while (pulse_get_msgq(pulsed)!=NULL); // wait for seek
   frames_written=pulsed->frames_written;
   if (frames_written<0.) frames_written=0.;
-  if (pulsed->is_output) return pulsed->audio_ticks+(gint64)(frames_written/(gdouble)pulsed->out_arate*U_SEC);
-  return pulsed->audio_ticks+(gint64)(frames_written/(gdouble)pulsed->in_arate*U_SEC);
+  if (pulsed->is_output) return pulsed->audio_ticks*absolute+(gint64)(frames_written/(gdouble)pulsed->out_arate*U_SEC);
+  return pulsed->audio_ticks*absolute+(gint64)(frames_written/(gdouble)pulsed->in_arate*U_SEC);
 
 }
 
+gdouble lives_pulse_get_pos(pulse_driver_t *pulsed) {
+  // get current time position (seconds) in audio file
+  return pulsed->seek_pos/(gdouble)(afile->arate*afile->achans*afile->asampsize/8);
+}
+
 
 static aserver_message_t pulse_message;
 
 void pulse_audio_seek_frame (pulse_driver_t *pulsed, gint frame) {
-  // seek to frame "frame" in current file
+  // seek to frame "frame" in current audio file
+  // position will be adjusted to (floor) nearest sample
+
   volatile aserver_message_t *pmsg;
   if (frame<1) frame=1;
   long seekstart;
@@ -784,7 +809,11 @@ void pulse_audio_seek_frame (pulse_driver_t *pulsed, gint frame) {
 
 
 long pulse_audio_seek_bytes (pulse_driver_t *pulsed, long bytes) {
-  // seek to relative "secs" in current file
+  // seek to position "bytes" in current audio file
+  // position will be adjusted to (floor) nearest sample
+
+  // if the position is > size of file, we will seek to the end of the file
+
   volatile aserver_message_t *pmsg;
   long seekstart;
   do {
