@@ -1,6 +1,6 @@
 // resample.c
 // LiVES
-// (c) G. Finch 2004 - 2009 <salsaman@xs4all.nl>
+// (c) G. Finch 2004 - 2010 <salsaman@xs4all.nl>
 // released under the GNU GPL 3 or later
 // see file ../COPYING or www.gnu.org for licensing details
 
@@ -24,20 +24,24 @@ static gboolean reorder_leave_back=FALSE;
 
 /////////////////////////////////////////////////////
 
-
-
 LIVES_INLINE weed_timecode_t q_gint64 (weed_timecode_t in, gdouble fps) {
-  if (in>0) return ((weed_timecode_t)((long double)in/(long double)U_SEC*(long double)fps+(long double).5)/(long double)fps)*(weed_timecode_t)U_SECL; // quantise to frame timing
-  if (in<0) return ((weed_timecode_t)((long double)in/(long double)U_SEC*(long double)fps-(long double).5)/(long double)fps)*(weed_timecode_t)U_SECL; // quantise to frame timing
+  if (in>(weed_timecode_t)0) return ((weed_timecode_t)((long double)in/(long double)U_SEC*(long double)fps+(long double).5)/(long double)fps)*(weed_timecode_t)U_SECL; // quantise to frame timing
+  if (in<(weed_timecode_t)0) return ((weed_timecode_t)((long double)in/(long double)U_SEC*(long double)fps-(long double).5)/(long double)fps)*(weed_timecode_t)U_SECL; // quantise to frame timing
+  return (weed_timecode_t)0;
+}
+
+LIVES_INLINE weed_timecode_t q_gint64_floor (weed_timecode_t in, gdouble fps) {
+  if (in!=(weed_timecode_t)0) return ((weed_timecode_t)((long double)in/(long double)U_SEC*(long double)fps)/(long double)fps)*(weed_timecode_t)U_SECL; // quantise to frame timing
   return (weed_timecode_t)0;
 }
 
 LIVES_INLINE weed_timecode_t q_dbl (gdouble in, gdouble fps) {
-  if (in>0) return ((weed_timecode_t)((long double)in*(long double)fps+(long double).5)/(long double)fps)*(weed_timecode_t)U_SECL; // quantise to frame timing
-  if (in<0) return ((weed_timecode_t)((long double)in*(long double)fps-(long double).5)/(long double)fps)*(weed_timecode_t)U_SECL; // quantise to frame timing
+  if (in>0.) return ((weed_timecode_t)((long double)in*(long double)fps+(long double).5)/(long double)fps)*(weed_timecode_t)U_SECL; // quantise to frame timing
+  if (in<0.) return ((weed_timecode_t)((long double)in*(long double)fps-(long double).5)/(long double)fps)*(weed_timecode_t)U_SECL; // quantise to frame timing
   return (weed_timecode_t)0;
 }
 
+
 LIVES_INLINE gint count_resampled_frames (gint in_frames, gdouble orig_fps, gdouble resampled_fps) {
   gint res_frames;
   return ((res_frames=(gint)((gdouble)in_frames/orig_fps*resampled_fps+.49999))<1)?1:res_frames;
@@ -45,7 +49,7 @@ LIVES_INLINE gint count_resampled_frames (gint in_frames, gdouble orig_fps, gdou
 
 /////////////////////////////////////////////////////
 
-gboolean auto_resample_resize (gint width,gint height,gdouble fps,gint fps_num,gint fps_denom, gint arate) {
+gboolean auto_resample_resize (gint width,gint height,gdouble fps,gint fps_num,gint fps_denom, gint arate, gint asigned) {
   // do a block atomic: resample audio, then resample video/resize or joint resample/resize
 
   gchar *com,*msg=NULL;
@@ -57,11 +61,15 @@ gboolean auto_resample_resize (gint width,gint height,gdouble fps,gint fps_num,g
 
   reorder_leave_back=FALSE;
 
-  if (arate>0&&arate!=cfile->arate) {
+  if (asigned!=0||(arate>0&&arate!=cfile->arate)) {
     cfile->undo1_int=arate;
     cfile->undo2_int=cfile->achans;
     cfile->undo3_int=cfile->asampsize;
     cfile->undo1_uint=cfile->signed_endian;
+
+    if (asigned==1&&(cfile->signed_endian&AFORM_UNSIGNED)==AFORM_UNSIGNED) cfile->undo1_uint^=AFORM_UNSIGNED;
+    else if (asigned==2&&(cfile->signed_endian&AFORM_UNSIGNED)!=AFORM_UNSIGNED) cfile->undo1_uint|=AFORM_UNSIGNED;
+
     on_resaudio_ok_clicked (NULL,NULL);
     if (mainw->error) return FALSE;
     audio_resampled=TRUE;
@@ -88,7 +96,7 @@ gboolean auto_resample_resize (gint width,gint height,gdouble fps,gint fps_num,g
 	    cfile->fx_frame_pump=1;
 	  }
 
-	  com=g_strdup_printf ("smogrify resize_all %s %d %d %d",cfile->handle,cfile->frames,width,height);
+	  com=g_strdup_printf ("smogrify resize_all %s %d %d %d %s",cfile->handle,cfile->frames,width,height,cfile->img_type==IMG_TYPE_JPEG?"jpg":"png");
 	  
 	  cfile->progress_start=1;
 	  cfile->progress_end=cfile->frames;
@@ -232,7 +240,7 @@ gboolean auto_resample_resize (gint width,gint height,gdouble fps,gint fps_num,g
 	  cfile->fx_frame_pump=1;
 	}
 
-	com=g_strdup_printf ("smogrify resize_all %s %d %d %d",cfile->handle,cfile->frames,width,height);
+	com=g_strdup_printf ("smogrify resize_all %s %d %d %d %s",cfile->handle,cfile->frames,width,height,cfile->img_type==IMG_TYPE_JPEG?"jpg":"png");
 	
 	cfile->progress_start=1;
 	cfile->progress_end=cfile->frames;
@@ -325,7 +333,7 @@ gboolean auto_resample_resize (gint width,gint height,gdouble fps,gint fps_num,g
 
 //////////////////////////////////////////////////////////////////
 
-weed_plant_t *
+WARN_UNUSED weed_plant_t *
 quantise_events (weed_plant_t *in_list, gdouble qfps, gboolean allow_gap) { 
   // new style event system, now we quantise from event_list_t *in_list to *out_list with period tl/U_SEC
 
@@ -547,12 +555,18 @@ static void on_reorder_activate (void) {
   gchar *msg;
   gboolean has_lmap_error=FALSE;
 
-  if (!(prefs->warning_mask&WARN_MASK_LAYOUT_ALTER_FRAMES)&&cfile->layout_map!=NULL&&layout_frame_is_affected(mainw->current_file,1)) {
+  if (!(prefs->warning_mask&WARN_MASK_LAYOUT_ALTER_FRAMES)&&(mainw->xlays=layout_frame_is_affected(mainw->current_file,1))!=NULL) {
     if (!do_layout_alter_frames_warning()) {
+      g_list_free_strings(mainw->xlays);
+      g_list_free(mainw->xlays);
+      mainw->xlays=NULL;
       return;
     }
-    add_lmap_error(LMAP_ERROR_ALTER_FRAMES,cfile->name,(gpointer)cfile->layout_map,mainw->current_file,0,0.);
+    add_lmap_error(LMAP_ERROR_ALTER_FRAMES,cfile->name,(gpointer)cfile->layout_map,mainw->current_file,0,0.,cfile->stored_layout_frame>0);
     has_lmap_error=TRUE;
+    g_list_free_strings(mainw->xlays);
+    g_list_free(mainw->xlays);
+    mainw->xlays=NULL;
   }
 
   cfile->old_frames=cfile->frames;
@@ -598,6 +612,11 @@ static void on_reorder_activate (void) {
 
   if (has_lmap_error) popup_lmap_errors(NULL,NULL);
 
+  if (mainw->sl_undo_mem!=NULL&&cfile->stored_layout_frame!=0) {
+    // need to invalidate undo/redo stack, in case file was used in some layout undo
+    stored_event_list_free_undos();
+  }
+
 
 }
 
@@ -660,12 +679,18 @@ on_resaudio_ok_clicked                      (GtkButton *button,
     aendian=!(cfile->undo1_uint&AFORM_BIG_ENDIAN);
   }
 
-  if (!(prefs->warning_mask&WARN_MASK_LAYOUT_ALTER_AUDIO)&&cfile->layout_map!=NULL&&layout_audio_is_affected(mainw->current_file,0.)) {
+  if (!(prefs->warning_mask&WARN_MASK_LAYOUT_ALTER_AUDIO)&&(mainw->xlays=layout_audio_is_affected(mainw->current_file,0.))) {
     if (!do_layout_alter_audio_warning()) {
+      g_list_free_strings(mainw->xlays);
+      g_list_free(mainw->xlays);
+      mainw->xlays=NULL;
       return;
     }
-    add_lmap_error(LMAP_ERROR_ALTER_AUDIO,cfile->name,(gpointer)cfile->layout_map,mainw->current_file,0,0.);
+    add_lmap_error(LMAP_ERROR_ALTER_AUDIO,cfile->name,(gpointer)cfile->layout_map,mainw->current_file,0,0.,cfile->stored_layout_audio>0.);
     has_lmap_error=TRUE;
+    g_list_free_strings(mainw->xlays);
+    g_list_free(mainw->xlays);
+    mainw->xlays=NULL;
   }
 
   // store old values for undo/redo
@@ -722,6 +747,8 @@ on_resaudio_ok_clicked                      (GtkButton *button,
 
   switch_to_file(mainw->current_file,mainw->current_file);
 
+  d_print("");  // force printing of switch message
+
   msg=g_strdup_printf (_ ("Audio was resampled to %d Hz, %d channels, %d bit"),arate,achans,asampsize);
   d_print (msg);
   g_free (msg);
@@ -743,6 +770,12 @@ on_resaudio_ok_clicked                      (GtkButton *button,
   }
   d_print ("\n");
   if (has_lmap_error) popup_lmap_errors(NULL,NULL);
+
+  if (mainw->sl_undo_mem!=NULL&&cfile->stored_layout_audio>0.) {
+    // need to invalidate undo/redo stack, in case file was used in some layout undo
+    stored_event_list_free_undos();
+  }
+
 }
 
 
@@ -778,8 +811,6 @@ static void on_resaudw_achans_changed (GtkWidget *widg, gpointer user_data) {
       gtk_widget_set_sensitive (resaudw->rb_bigend,TRUE);
       gtk_widget_set_sensitive (resaudw->rb_littleend,TRUE);
     }
-    gtk_widget_set_sensitive (resaudw->rb_signed,TRUE);
-    gtk_widget_set_sensitive (resaudw->rb_unsigned,TRUE);
     gtk_widget_set_sensitive (resaudw->entry_arate,TRUE);
     gtk_widget_set_sensitive (resaudw->entry_asamps,TRUE);
     gtk_widget_set_sensitive (resaudw->entry_achans,TRUE);
@@ -807,10 +838,18 @@ on_resaudw_asamps_changed (GtkWidget *irrelevant, gpointer rubbish) {
   if (atoi (gtk_entry_get_text (GTK_ENTRY (resaudw->entry_asamps)))==8) {
     gtk_widget_set_sensitive (resaudw->rb_bigend,FALSE);
     gtk_widget_set_sensitive (resaudw->rb_littleend,FALSE);
+    gtk_widget_set_sensitive (resaudw->rb_signed,FALSE);
+    gtk_widget_set_sensitive (resaudw->rb_unsigned,TRUE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(resaudw->rb_unsigned),TRUE);
   }
   else {
     gtk_widget_set_sensitive (resaudw->rb_bigend,TRUE);
     gtk_widget_set_sensitive (resaudw->rb_littleend,TRUE);
+    if (atoi (gtk_entry_get_text (GTK_ENTRY (resaudw->entry_asamps)))==16) {
+      gtk_widget_set_sensitive (resaudw->rb_signed,TRUE);
+      gtk_widget_set_sensitive (resaudw->rb_unsigned,FALSE);
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(resaudw->rb_signed),TRUE);
+    }
   }
 }
 
@@ -1032,6 +1071,7 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
   gint aendian;
 
   gboolean chans_fixed=FALSE;
+  gboolean is_8bit;
 
   if (type==10) {
     if (mainw->multitrack!=NULL) chans_fixed=TRUE; // TODO *
@@ -1081,7 +1121,10 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     }
     gtk_window_set_position (GTK_WINDOW (resaudw->dialog), GTK_WIN_POS_CENTER);
     gtk_window_set_modal (GTK_WINDOW (resaudw->dialog), TRUE);
-    gtk_window_set_transient_for(GTK_WINDOW(resaudw->dialog),GTK_WINDOW(mainw->LiVES));
+
+    if (prefs->show_gui) {
+      gtk_window_set_transient_for(GTK_WINDOW(resaudw->dialog),GTK_WINDOW(mainw->LiVES));
+    }
 
     dialog_vbox = GTK_DIALOG (resaudw->dialog)->vbox;
     gtk_widget_show (dialog_vbox);
@@ -1475,6 +1518,10 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     else if (type==3) tmp=g_strdup_printf ("%d",rdet->asamps);
     else tmp=g_strdup_printf ("%d",prefs->mt_def_asamps);
     gtk_entry_set_text (GTK_ENTRY (resaudw->entry_asamps),tmp);
+
+    if (!strcmp(tmp,"8")) is_8bit=TRUE;
+    else is_8bit=FALSE;
+
     g_free (tmp);
 
     vseparator5 = gtk_vseparator_new ();
@@ -1516,7 +1563,7 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     gtk_box_pack_start (GTK_BOX (hbox), eventbox, FALSE, FALSE, 10);
   
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_signed), TRUE);
-    if (type==7) gtk_widget_set_sensitive(resaudw->rb_signed,FALSE);
+    if (type==7||is_8bit) gtk_widget_set_sensitive(resaudw->rb_signed,FALSE);
     
 
     hbox = gtk_hbox_new (FALSE, 0);
@@ -1549,10 +1596,10 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     }
     gtk_box_pack_start (GTK_BOX (hbox), eventbox, FALSE, FALSE, 10);
 
-    if (type==7) gtk_widget_set_sensitive(resaudw->rb_unsigned,FALSE);
+    if (type==7||!is_8bit) gtk_widget_set_sensitive(resaudw->rb_unsigned,FALSE);
 
     if (type<3||(type>4&&type<8)) aendian=mainw->fx4_val;
-    else if (type==8) aendian=DEFAULT_AUDIO_SIGNED|((G_BYTE_ORDER==G_BIG_ENDIAN)&AFORM_BIG_ENDIAN);
+    else if (type==8) aendian=DEFAULT_AUDIO_SIGNED16|((G_BYTE_ORDER==G_BIG_ENDIAN)&AFORM_BIG_ENDIAN);
     else if (type==3) aendian=rdet->aendian;
     else aendian=prefs->mt_def_signed_endian;
 
@@ -1925,7 +1972,10 @@ create_new_pb_speed (gshort type)
   gtk_window_set_position (GTK_WINDOW (new_pb_speed), GTK_WIN_POS_CENTER);
   gtk_window_set_modal (GTK_WINDOW (new_pb_speed), TRUE);
   gtk_window_set_default_size (GTK_WINDOW (new_pb_speed), 300, 200);
-  gtk_window_set_transient_for(GTK_WINDOW(new_pb_speed),GTK_WINDOW(mainw->LiVES));
+
+  if (prefs->show_gui) {
+    gtk_window_set_transient_for(GTK_WINDOW(new_pb_speed),GTK_WINDOW(mainw->LiVES));
+  }
 
   if (palette->style&STYLE_1) {
     gtk_widget_modify_bg(new_pb_speed, GTK_STATE_NORMAL, &palette->normal_back);
@@ -2100,54 +2150,94 @@ on_change_speed_ok_clicked                (GtkButton *button,
     if (mainw->fx1_val>FPS_MAX) mainw->fx1_val=FPS_MAX;
   }
 
-  if (!(prefs->warning_mask&WARN_MASK_LAYOUT_DELETE_FRAMES)&&mainw->fx1_val>cfile->fps&&cfile->layout_map!=NULL) {
+  if (!(prefs->warning_mask&WARN_MASK_LAYOUT_DELETE_FRAMES)&&mainw->fx1_val>cfile->fps) {
     gint new_frames=count_resampled_frames(cfile->frames,mainw->fx1_val,cfile->fps);
-    if (layout_frame_is_affected(mainw->current_file,new_frames)) {
-      if (!do_warning_dialog(_("\nSpeeding up the clip will cause missing frames in some multitrack layouts.\nAre you sure you wish to change the speed ?\n"))) return;
-      add_lmap_error(LMAP_ERROR_DELETE_FRAMES,cfile->name,(gpointer)cfile->layout_map,mainw->current_file,new_frames,0.);
+    if ((mainw->xlays=layout_frame_is_affected(mainw->current_file,new_frames))!=NULL) {
+      if (!do_warning_dialog(_("\nSpeeding up the clip will cause missing frames in some multitrack layouts.\nAre you sure you wish to change the speed ?\n"))) {
+	g_list_free_strings(mainw->xlays);
+	g_list_free(mainw->xlays);
+	mainw->xlays=NULL;
+	return;
+      }
+      add_lmap_error(LMAP_ERROR_DELETE_FRAMES,cfile->name,(gpointer)cfile->layout_map,mainw->current_file,new_frames,0.,new_frames<=count_resampled_frames(cfile->stored_layout_frame,cfile->stored_layout_fps,cfile->fps));
       has_lmap_error=TRUE;
+      g_list_free_strings(mainw->xlays);
+      g_list_free(mainw->xlays);
+      mainw->xlays=NULL;
     }
   }
 
-  if (mainw->fx1_bool&&!(prefs->warning_mask&WARN_MASK_LAYOUT_DELETE_AUDIO)&&mainw->fx1_val>cfile->fps&&cfile->layout_map!=NULL) {
+  if (mainw->fx1_bool&&!(prefs->warning_mask&WARN_MASK_LAYOUT_DELETE_AUDIO)&&mainw->fx1_val>cfile->fps) {
     gint new_frames=count_resampled_frames(cfile->frames,mainw->fx1_val,cfile->fps);
-    if (layout_audio_is_affected(mainw->current_file,(new_frames-1.)/cfile->fps)) {
-      if (!do_warning_dialog(_("\nSpeeding up the clip will cause missing audio in some multitrack layouts.\nAre you sure you wish to change the speed ?\n"))) return;
-      add_lmap_error(LMAP_ERROR_DELETE_AUDIO,cfile->name,(gpointer)cfile->layout_map,mainw->current_file,0,(new_frames-1.)/cfile->fps);
+    if ((mainw->xlays=layout_audio_is_affected(mainw->current_file,(new_frames-1.)/cfile->fps))!=NULL) {
+      if (!do_warning_dialog(_("\nSpeeding up the clip will cause missing audio in some multitrack layouts.\nAre you sure you wish to change the speed ?\n"))) {
+	g_list_free_strings(mainw->xlays);
+	g_list_free(mainw->xlays);
+	mainw->xlays=NULL;
+	return;
+      }
+      add_lmap_error(LMAP_ERROR_DELETE_AUDIO,cfile->name,(gpointer)cfile->layout_map,mainw->current_file,0,(new_frames-1.)/cfile->fps,(new_frames-1.)/cfile->fps<cfile->stored_layout_audio);
       has_lmap_error=TRUE;
+      g_list_free_strings(mainw->xlays);
+      g_list_free(mainw->xlays);
+      mainw->xlays=NULL;
     }
   }
 
-  if (!has_lmap_error&&!(prefs->warning_mask&WARN_MASK_LAYOUT_SHIFT_FRAMES)&&cfile->layout_map!=NULL&&layout_frame_is_affected(mainw->current_file,1)) {
+  if (!has_lmap_error&&!(prefs->warning_mask&WARN_MASK_LAYOUT_SHIFT_FRAMES)&&(mainw->xlays=layout_frame_is_affected(mainw->current_file,1))!=NULL) {
     if (!do_warning_dialog(_("\nChanging the speed will cause frames to shift some multitrack layouts.\nAre you sure you wish to continue ?\n"))) {
+      g_list_free_strings(mainw->xlays);
+      g_list_free(mainw->xlays);
+      mainw->xlays=NULL;
       return;
     }
-    add_lmap_error(LMAP_ERROR_SHIFT_FRAMES,cfile->name,(gpointer)cfile->layout_map,mainw->current_file,0,0.);
+    add_lmap_error(LMAP_ERROR_SHIFT_FRAMES,cfile->name,(gpointer)cfile->layout_map,mainw->current_file,0,0.,cfile->stored_layout_frame>0);
     has_lmap_error=TRUE;
+    g_list_free_strings(mainw->xlays);
+    g_list_free(mainw->xlays);
+    mainw->xlays=NULL;
   }
 
-  if (mainw->fx1_bool&&!has_lmap_error&&!(prefs->warning_mask&WARN_MASK_LAYOUT_SHIFT_AUDIO)&&cfile->layout_map!=NULL&&layout_audio_is_affected(mainw->current_file,0.)) {
+  if (mainw->fx1_bool&&!has_lmap_error&&!(prefs->warning_mask&WARN_MASK_LAYOUT_SHIFT_AUDIO)&&(mainw->xlays=layout_audio_is_affected(mainw->current_file,0.))!=NULL) {
     if (!do_warning_dialog(_("\nChanging the speed will cause audio to shift some multitrack layouts.\nAre you sure you wish to continue ?\n"))) {
+      g_list_free_strings(mainw->xlays);
+      g_list_free(mainw->xlays);
+      mainw->xlays=NULL;
       return;
     }
-    add_lmap_error(LMAP_ERROR_SHIFT_AUDIO,cfile->name,(gpointer)cfile->layout_map,mainw->current_file,0,0.);
+    add_lmap_error(LMAP_ERROR_SHIFT_AUDIO,cfile->name,(gpointer)cfile->layout_map,mainw->current_file,0,0.,cfile->stored_layout_audio>0.);
     has_lmap_error=TRUE;
+    g_list_free_strings(mainw->xlays);
+    g_list_free(mainw->xlays);
+    mainw->xlays=NULL;
   }
 
-  if (!has_lmap_error&&!(prefs->warning_mask&WARN_MASK_LAYOUT_ALTER_FRAMES)&&cfile->layout_map!=NULL&&layout_frame_is_affected(mainw->current_file,1)) {
+  if (!has_lmap_error&&!(prefs->warning_mask&WARN_MASK_LAYOUT_ALTER_FRAMES)&&(mainw->xlays=layout_frame_is_affected(mainw->current_file,1))!=NULL) {
     if (!do_layout_alter_frames_warning()) {
+      g_list_free_strings(mainw->xlays);
+      g_list_free(mainw->xlays);
+      mainw->xlays=NULL;
       return;
     }
-    add_lmap_error(LMAP_ERROR_ALTER_FRAMES,cfile->name,(gpointer)cfile->layout_map,mainw->current_file,0,0.);
+    add_lmap_error(LMAP_ERROR_ALTER_FRAMES,cfile->name,(gpointer)cfile->layout_map,mainw->current_file,0,0.,cfile->stored_layout_frame>0);
     has_lmap_error=TRUE;
+    g_list_free_strings(mainw->xlays);
+    g_list_free(mainw->xlays);
+    mainw->xlays=NULL;
   }
 
-  if (mainw->fx1_bool&&!has_lmap_error&&!(prefs->warning_mask&WARN_MASK_LAYOUT_ALTER_AUDIO)&&cfile->layout_map!=NULL&&layout_audio_is_affected(mainw->current_file,0.)) {
+  if (mainw->fx1_bool&&!has_lmap_error&&!(prefs->warning_mask&WARN_MASK_LAYOUT_ALTER_AUDIO)&&(mainw->xlays=layout_audio_is_affected(mainw->current_file,0.))!=NULL) {
     if (!do_layout_alter_audio_warning()) {
+      g_list_free_strings(mainw->xlays);
+      g_list_free(mainw->xlays);
+      mainw->xlays=NULL;
       return;
     }
-    add_lmap_error(LMAP_ERROR_ALTER_AUDIO,cfile->name,(gpointer)cfile->layout_map,mainw->current_file,0,0.);
+    add_lmap_error(LMAP_ERROR_ALTER_AUDIO,cfile->name,(gpointer)cfile->layout_map,mainw->current_file,0,0.,cfile->stored_layout_audio>0.);
     has_lmap_error=TRUE;
+    g_list_free_strings(mainw->xlays);
+    g_list_free(mainw->xlays);
+    mainw->xlays=NULL;
   }
 
   if (button==NULL) {
@@ -2181,6 +2271,12 @@ on_change_speed_ok_clicked                (GtkButton *button,
   switch_to_file(mainw->current_file,mainw->current_file);
 
   if (has_lmap_error) popup_lmap_errors(NULL,NULL);
+
+  if (mainw->sl_undo_mem!=NULL&&cfile->stored_layout_frame!=0) {
+    // need to invalidate undo/redo stack, in case file was used in some layout undo
+    stored_event_list_free_undos();
+  }
+
 }
 
 
@@ -2195,8 +2291,8 @@ reorder_frames(void) {
   gchar **array;
   gchar *com;
 
-  if (reorder_width*reorder_height==0) com=g_strdup_printf("smogrify reorder %s %d 0 0 %d %d",cfile->handle,!mainw->endian,reorder_leave_back,cfile->frames);
-  else com=g_strdup_printf("smogrify reorder %s %d %d %d 0 %d",cfile->handle,!mainw->endian,reorder_width,reorder_height,cfile->frames);
+  if (reorder_width*reorder_height==0) com=g_strdup_printf("smogrify reorder %s %s %d 0 0 %d %d",cfile->handle,cfile->img_type==IMG_TYPE_JPEG?"jpg":"png",!mainw->endian,reorder_leave_back,cfile->frames);
+  else com=g_strdup_printf("smogrify reorder %s %s %d %d %d 0 %d",cfile->handle,cfile->img_type==IMG_TYPE_JPEG?"jpg":"png",!mainw->endian,reorder_width,reorder_height,cfile->frames);
   cfile->frames=0;
 
   cfile->progress_start=1;
@@ -2276,14 +2372,13 @@ deorder_frames(gint old_frames, gboolean leave_bak) {
     perf_start=(gint)(cfile->fps*(gdouble)time_start/U_SEC)+1;
     perf_end=perf_start+count_events (cfile->event_list,FALSE,0,0)-1;
   }
-  com=g_strdup_printf("smogrify deorder %s %d %d %d %d",cfile->handle,perf_start,perf_end,old_frames,leave_bak);
+  com=g_strdup_printf("smogrify deorder %s %d %d %d %s %d",cfile->handle,perf_start,cfile->frames,perf_end,cfile->img_type==IMG_TYPE_JPEG?"jpg":"png",leave_bak);
 
   unlink(cfile->info_file);
   dummyvar=system(com);
   do_progress_dialog(TRUE,FALSE,_ ("Deordering frames"));
   g_free(com);
 
-
   if (cfile->frame_index_back!=NULL) {
     restore_frame_index_back(mainw->current_file);
   }
@@ -2298,6 +2393,8 @@ gboolean resample_clipboard(gdouble new_fps) {
   gchar *msg;
   gint current_file=mainw->current_file;
 
+  if (new_fps==clipboard->fps) return TRUE;
+
   if (clipboard->undo1_dbl==cfile->fps&&!prefs->conserve_space) {
     gint old_frames=clipboard->old_frames;
     gdouble old_fps=clipboard->fps;
@@ -2305,7 +2402,7 @@ gboolean resample_clipboard(gdouble new_fps) {
     
     // we already resampled to this fps
     mainw->current_file=0;
-    com=g_strdup_printf("smogrify redo %s %d %d",cfile->handle,1,cfile->old_frames);
+    com=g_strdup_printf("smogrify redo %s %d %d %s",cfile->handle,1,cfile->old_frames,cfile->img_type==IMG_TYPE_JPEG?"jpg":"png");
     unlink(cfile->info_file);
     dummyvar=system(com);
     cfile->progress_start=1;
